
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "meta-private-enum-types.h"

/* enumerations from "/build/mutter/src/build/../mutter/src/backends/meta-screen-cast-session.h" */
#include "/build/mutter/src/build/../mutter/src/backends/meta-screen-cast-session.h"
GType
meta_screen_cast_session_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_SCREEN_CAST_SESSION_TYPE_NORMAL, "META_SCREEN_CAST_SESSION_TYPE_NORMAL", "normal" },
        { META_SCREEN_CAST_SESSION_TYPE_REMOTE_DESKTOP, "META_SCREEN_CAST_SESSION_TYPE_REMOTE_DESKTOP", "remote-desktop" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaScreenCastSessionType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/backends/meta-screen-cast.h" */
#include "/build/mutter/src/build/../mutter/src/backends/meta-screen-cast.h"
GType
meta_screen_cast_cursor_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_SCREEN_CAST_CURSOR_MODE_HIDDEN, "META_SCREEN_CAST_CURSOR_MODE_HIDDEN", "hidden" },
        { META_SCREEN_CAST_CURSOR_MODE_EMBEDDED, "META_SCREEN_CAST_CURSOR_MODE_EMBEDDED", "embedded" },
        { META_SCREEN_CAST_CURSOR_MODE_METADATA, "META_SCREEN_CAST_CURSOR_MODE_METADATA", "metadata" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaScreenCastCursorMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_screen_cast_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_SCREEN_CAST_FLAG_NONE, "META_SCREEN_CAST_FLAG_NONE", "none" },
        { META_SCREEN_CAST_FLAG_IS_RECORDING, "META_SCREEN_CAST_FLAG_IS_RECORDING", "is-recording" },
        { META_SCREEN_CAST_FLAG_IS_PLATFORM, "META_SCREEN_CAST_FLAG_IS_PLATFORM", "is-platform" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaScreenCastFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/backends/native/meta-backend-native-types.h" */
#include "/build/mutter/src/build/../mutter/src/backends/native/meta-backend-native-types.h"
GType
meta_seat_native_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_SEAT_NATIVE_FLAG_NONE, "META_SEAT_NATIVE_FLAG_NONE", "none" },
        { META_SEAT_NATIVE_FLAG_NO_LIBINPUT, "META_SEAT_NATIVE_FLAG_NO_LIBINPUT", "no-libinput" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaSeatNativeFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_backend_native_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_BACKEND_NATIVE_MODE_DEFAULT, "META_BACKEND_NATIVE_MODE_DEFAULT", "default" },
        { META_BACKEND_NATIVE_MODE_HEADLESS, "META_BACKEND_NATIVE_MODE_HEADLESS", "headless" },
        { META_BACKEND_NATIVE_MODE_TEST_VKMS, "META_BACKEND_NATIVE_MODE_TEST_VKMS", "test-vkms" },
        { META_BACKEND_NATIVE_MODE_TEST_HEADLESS, "META_BACKEND_NATIVE_MODE_TEST_HEADLESS", "test-headless" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaBackendNativeMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/backends/native/meta-drm-buffer.h" */
#include "/build/mutter/src/build/../mutter/src/backends/native/meta-drm-buffer.h"
GType
meta_drm_buffer_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_DRM_BUFFER_FLAG_NONE, "META_DRM_BUFFER_FLAG_NONE", "none" },
        { META_DRM_BUFFER_FLAG_DISABLE_MODIFIERS, "META_DRM_BUFFER_FLAG_DISABLE_MODIFIERS", "disable-modifiers" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaDrmBufferFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/backends/native/meta-kms-types.h" */
#include "/build/mutter/src/build/../mutter/src/backends/native/meta-kms-types.h"
GType
meta_kms_device_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_KMS_DEVICE_FLAG_NONE, "META_KMS_DEVICE_FLAG_NONE", "none" },
        { META_KMS_DEVICE_FLAG_BOOT_VGA, "META_KMS_DEVICE_FLAG_BOOT_VGA", "boot-vga" },
        { META_KMS_DEVICE_FLAG_PLATFORM_DEVICE, "META_KMS_DEVICE_FLAG_PLATFORM_DEVICE", "platform-device" },
        { META_KMS_DEVICE_FLAG_DISABLE_MODIFIERS, "META_KMS_DEVICE_FLAG_DISABLE_MODIFIERS", "disable-modifiers" },
        { META_KMS_DEVICE_FLAG_PREFERRED_PRIMARY, "META_KMS_DEVICE_FLAG_PREFERRED_PRIMARY", "preferred-primary" },
        { META_KMS_DEVICE_FLAG_NO_MODE_SETTING, "META_KMS_DEVICE_FLAG_NO_MODE_SETTING", "no-mode-setting" },
        { META_KMS_DEVICE_FLAG_HAS_ADDFB2, "META_KMS_DEVICE_FLAG_HAS_ADDFB2", "has-addfb2" },
        { META_KMS_DEVICE_FLAG_FORCE_LEGACY, "META_KMS_DEVICE_FLAG_FORCE_LEGACY", "force-legacy" },
        { META_KMS_DEVICE_FLAG_DISABLE_CLIENT_MODIFIERS, "META_KMS_DEVICE_FLAG_DISABLE_CLIENT_MODIFIERS", "disable-client-modifiers" },
        { META_KMS_DEVICE_FLAG_SUPPORTS_COLOR_MODES, "META_KMS_DEVICE_FLAG_SUPPORTS_COLOR_MODES", "supports-color-modes" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaKmsDeviceFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_kms_resource_changes_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_KMS_RESOURCE_CHANGE_NONE, "META_KMS_RESOURCE_CHANGE_NONE", "none" },
        { META_KMS_RESOURCE_CHANGE_CRTC_COLOR_PIPELINE, "META_KMS_RESOURCE_CHANGE_CRTC_COLOR_PIPELINE", "crtc-color-pipeline" },
        { META_KMS_RESOURCE_CHANGE_NO_DEVICES, "META_KMS_RESOURCE_CHANGE_NO_DEVICES", "no-devices" },
        { META_KMS_RESOURCE_CHANGE_PRIVACY_SCREEN, "META_KMS_RESOURCE_CHANGE_PRIVACY_SCREEN", "privacy-screen" },
        { META_KMS_RESOURCE_CHANGE_FULL, "META_KMS_RESOURCE_CHANGE_FULL", "full" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaKmsResourceChanges"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_kms_update_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_KMS_UPDATE_FLAG_NONE, "META_KMS_UPDATE_FLAG_NONE", "none" },
        { META_KMS_UPDATE_FLAG_TEST_ONLY, "META_KMS_UPDATE_FLAG_TEST_ONLY", "test-only" },
        { META_KMS_UPDATE_FLAG_MODE_SET, "META_KMS_UPDATE_FLAG_MODE_SET", "mode-set" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaKmsUpdateFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_kms_prop_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_KMS_PROP_TYPE_RAW, "META_KMS_PROP_TYPE_RAW", "raw" },
        { META_KMS_PROP_TYPE_FIXED_16, "META_KMS_PROP_TYPE_FIXED_16", "fixed-16" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaKmsPropType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_kms_inhibit_subset_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_KMS_INHIBIT_NONE, "META_KMS_INHIBIT_NONE", "none" },
        { META_KMS_INHIBIT_ALL, "META_KMS_INHIBIT_ALL", "all" },
        { META_KMS_INHIBIT_NON_TEST_ONLY, "META_KMS_INHIBIT_NON_TEST_ONLY", "non-test-only" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaKmsInhibitSubset"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/backends/native/meta-thread.h" */
#include "/build/mutter/src/build/../mutter/src/backends/native/meta-thread.h"
GType
meta_thread_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_THREAD_TYPE_KERNEL, "META_THREAD_TYPE_KERNEL", "kernel" },
        { META_THREAD_TYPE_USER, "META_THREAD_TYPE_USER", "user" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaThreadType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_scheduling_priority_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_SCHEDULING_PRIORITY_NORMAL, "META_SCHEDULING_PRIORITY_NORMAL", "normal" },
        { META_SCHEDULING_PRIORITY_REALTIME, "META_SCHEDULING_PRIORITY_REALTIME", "realtime" },
        { META_SCHEDULING_PRIORITY_HIGH_PRIORITY, "META_SCHEDULING_PRIORITY_HIGH_PRIORITY", "high-priority" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaSchedulingPriority"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/core/window-private.h" */
#include "/build/mutter/src/build/../mutter/src/core/window-private.h"
GType
meta_client_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_CLIENT_TYPE_UNKNOWN, "META_CLIENT_TYPE_UNKNOWN", "unknown" },
        { META_CLIENT_TYPE_APPLICATION, "META_CLIENT_TYPE_APPLICATION", "application" },
        { META_CLIENT_TYPE_PAGER, "META_CLIENT_TYPE_PAGER", "pager" },
        { META_CLIENT_TYPE_MAX_RECOGNIZED, "META_CLIENT_TYPE_MAX_RECOGNIZED", "max-recognized" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaClientType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_move_resize_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_MOVE_RESIZE_CONFIGURE_REQUEST, "META_MOVE_RESIZE_CONFIGURE_REQUEST", "configure-request" },
        { META_MOVE_RESIZE_USER_ACTION, "META_MOVE_RESIZE_USER_ACTION", "user-action" },
        { META_MOVE_RESIZE_MOVE_ACTION, "META_MOVE_RESIZE_MOVE_ACTION", "move-action" },
        { META_MOVE_RESIZE_RESIZE_ACTION, "META_MOVE_RESIZE_RESIZE_ACTION", "resize-action" },
        { META_MOVE_RESIZE_WAYLAND_FINISH_MOVE_RESIZE, "META_MOVE_RESIZE_WAYLAND_FINISH_MOVE_RESIZE", "wayland-finish-move-resize" },
        { META_MOVE_RESIZE_STATE_CHANGED, "META_MOVE_RESIZE_STATE_CHANGED", "state-changed" },
        { META_MOVE_RESIZE_UNMAXIMIZE, "META_MOVE_RESIZE_UNMAXIMIZE", "unmaximize" },
        { META_MOVE_RESIZE_UNFULLSCREEN, "META_MOVE_RESIZE_UNFULLSCREEN", "unfullscreen" },
        { META_MOVE_RESIZE_FORCE_MOVE, "META_MOVE_RESIZE_FORCE_MOVE", "force-move" },
        { META_MOVE_RESIZE_WAYLAND_STATE_CHANGED, "META_MOVE_RESIZE_WAYLAND_STATE_CHANGED", "wayland-state-changed" },
        { META_MOVE_RESIZE_FORCE_UPDATE_MONITOR, "META_MOVE_RESIZE_FORCE_UPDATE_MONITOR", "force-update-monitor" },
        { META_MOVE_RESIZE_PLACEMENT_CHANGED, "META_MOVE_RESIZE_PLACEMENT_CHANGED", "placement-changed" },
        { META_MOVE_RESIZE_WAYLAND_CLIENT_RESIZE, "META_MOVE_RESIZE_WAYLAND_CLIENT_RESIZE", "wayland-client-resize" },
        { META_MOVE_RESIZE_CONSTRAIN, "META_MOVE_RESIZE_CONSTRAIN", "constrain" },
        { META_MOVE_RESIZE_RECT_INVALID, "META_MOVE_RESIZE_RECT_INVALID", "rect-invalid" },
        { META_MOVE_RESIZE_WAYLAND_FORCE_CONFIGURE, "META_MOVE_RESIZE_WAYLAND_FORCE_CONFIGURE", "wayland-force-configure" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaMoveResizeFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_place_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_PLACE_FLAG_NONE, "META_PLACE_FLAG_NONE", "none" },
        { META_PLACE_FLAG_FORCE_MOVE, "META_PLACE_FLAG_FORCE_MOVE", "force-move" },
        { META_PLACE_FLAG_DENIED_FOCUS_AND_NOT_TRANSIENT, "META_PLACE_FLAG_DENIED_FOCUS_AND_NOT_TRANSIENT", "denied-focus-and-not-transient" },
        { META_PLACE_FLAG_CALCULATE, "META_PLACE_FLAG_CALCULATE", "calculate" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaPlaceFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_move_resize_result_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_MOVE_RESIZE_RESULT_MOVED, "META_MOVE_RESIZE_RESULT_MOVED", "moved" },
        { META_MOVE_RESIZE_RESULT_RESIZED, "META_MOVE_RESIZE_RESULT_RESIZED", "resized" },
        { META_MOVE_RESIZE_RESULT_STATE_CHANGED, "META_MOVE_RESIZE_RESULT_STATE_CHANGED", "state-changed" },
        { META_MOVE_RESIZE_RESULT_UPDATE_UNCONSTRAINED, "META_MOVE_RESIZE_RESULT_UPDATE_UNCONSTRAINED", "update-unconstrained" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaMoveResizeResultFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_placement_gravity_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_PLACEMENT_GRAVITY_NONE, "META_PLACEMENT_GRAVITY_NONE", "none" },
        { META_PLACEMENT_GRAVITY_TOP, "META_PLACEMENT_GRAVITY_TOP", "top" },
        { META_PLACEMENT_GRAVITY_BOTTOM, "META_PLACEMENT_GRAVITY_BOTTOM", "bottom" },
        { META_PLACEMENT_GRAVITY_LEFT, "META_PLACEMENT_GRAVITY_LEFT", "left" },
        { META_PLACEMENT_GRAVITY_RIGHT, "META_PLACEMENT_GRAVITY_RIGHT", "right" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaPlacementGravity"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_placement_anchor_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_PLACEMENT_ANCHOR_NONE, "META_PLACEMENT_ANCHOR_NONE", "none" },
        { META_PLACEMENT_ANCHOR_TOP, "META_PLACEMENT_ANCHOR_TOP", "top" },
        { META_PLACEMENT_ANCHOR_BOTTOM, "META_PLACEMENT_ANCHOR_BOTTOM", "bottom" },
        { META_PLACEMENT_ANCHOR_LEFT, "META_PLACEMENT_ANCHOR_LEFT", "left" },
        { META_PLACEMENT_ANCHOR_RIGHT, "META_PLACEMENT_ANCHOR_RIGHT", "right" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaPlacementAnchor"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_placement_constraint_adjustment_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_PLACEMENT_CONSTRAINT_ADJUSTMENT_NONE, "META_PLACEMENT_CONSTRAINT_ADJUSTMENT_NONE", "none" },
        { META_PLACEMENT_CONSTRAINT_ADJUSTMENT_SLIDE_X, "META_PLACEMENT_CONSTRAINT_ADJUSTMENT_SLIDE_X", "slide-x" },
        { META_PLACEMENT_CONSTRAINT_ADJUSTMENT_SLIDE_Y, "META_PLACEMENT_CONSTRAINT_ADJUSTMENT_SLIDE_Y", "slide-y" },
        { META_PLACEMENT_CONSTRAINT_ADJUSTMENT_FLIP_X, "META_PLACEMENT_CONSTRAINT_ADJUSTMENT_FLIP_X", "flip-x" },
        { META_PLACEMENT_CONSTRAINT_ADJUSTMENT_FLIP_Y, "META_PLACEMENT_CONSTRAINT_ADJUSTMENT_FLIP_Y", "flip-y" },
        { META_PLACEMENT_CONSTRAINT_ADJUSTMENT_RESIZE_X, "META_PLACEMENT_CONSTRAINT_ADJUSTMENT_RESIZE_X", "resize-x" },
        { META_PLACEMENT_CONSTRAINT_ADJUSTMENT_RESIZE_Y, "META_PLACEMENT_CONSTRAINT_ADJUSTMENT_RESIZE_Y", "resize-y" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaPlacementConstraintAdjustment"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_window_update_monitor_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_WINDOW_UPDATE_MONITOR_FLAGS_NONE, "META_WINDOW_UPDATE_MONITOR_FLAGS_NONE", "none" },
        { META_WINDOW_UPDATE_MONITOR_FLAGS_USER_OP, "META_WINDOW_UPDATE_MONITOR_FLAGS_USER_OP", "user-op" },
        { META_WINDOW_UPDATE_MONITOR_FLAGS_FORCE, "META_WINDOW_UPDATE_MONITOR_FLAGS_FORCE", "force" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaWindowUpdateMonitorFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_window_suspend_state_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_WINDOW_SUSPEND_STATE_ACTIVE, "META_WINDOW_SUSPEND_STATE_ACTIVE", "active" },
        { META_WINDOW_SUSPEND_STATE_HIDDEN, "META_WINDOW_SUSPEND_STATE_HIDDEN", "hidden" },
        { META_WINDOW_SUSPEND_STATE_SUSPENDED, "META_WINDOW_SUSPEND_STATE_SUSPENDED", "suspended" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaWindowSuspendState"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_window_apply_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_WINDOW_APPLY_FLAG_NONE, "META_WINDOW_APPLY_FLAG_NONE", "none" },
        { META_WINDOW_APPLY_FLAG_ALWAYS_MOVE_RESIZE, "META_WINDOW_APPLY_FLAG_ALWAYS_MOVE_RESIZE", "always-move-resize" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaWindowApplyFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_placement_state_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_PLACEMENT_STATE_UNCONSTRAINED, "META_PLACEMENT_STATE_UNCONSTRAINED", "unconstrained" },
        { META_PLACEMENT_STATE_CONSTRAINED_PENDING, "META_PLACEMENT_STATE_CONSTRAINED_PENDING", "constrained-pending" },
        { META_PLACEMENT_STATE_CONSTRAINED_CONFIGURED, "META_PLACEMENT_STATE_CONSTRAINED_CONFIGURED", "constrained-configured" },
        { META_PLACEMENT_STATE_CONSTRAINED_FINISHED, "META_PLACEMENT_STATE_CONSTRAINED_FINISHED", "constrained-finished" },
        { META_PLACEMENT_STATE_INVALIDATED, "META_PLACEMENT_STATE_INVALIDATED", "invalidated" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaPlacementState"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_edge_constraint_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_EDGE_CONSTRAINT_NONE, "META_EDGE_CONSTRAINT_NONE", "none" },
        { META_EDGE_CONSTRAINT_WINDOW, "META_EDGE_CONSTRAINT_WINDOW", "window" },
        { META_EDGE_CONSTRAINT_MONITOR, "META_EDGE_CONSTRAINT_MONITOR", "monitor" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaEdgeConstraint"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_edge_resistance_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_EDGE_RESISTANCE_DEFAULT, "META_EDGE_RESISTANCE_DEFAULT", "default" },
        { META_EDGE_RESISTANCE_SNAP, "META_EDGE_RESISTANCE_SNAP", "snap" },
        { META_EDGE_RESISTANCE_KEYBOARD_OP, "META_EDGE_RESISTANCE_KEYBOARD_OP", "keyboard-op" },
        { META_EDGE_RESISTANCE_WINDOWS, "META_EDGE_RESISTANCE_WINDOWS", "windows" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaEdgeResistanceFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_size_hints_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_SIZE_HINTS_USER_POSITION, "META_SIZE_HINTS_USER_POSITION", "user-position" },
        { META_SIZE_HINTS_USER_SIZE, "META_SIZE_HINTS_USER_SIZE", "user-size" },
        { META_SIZE_HINTS_PROGRAM_POSITION, "META_SIZE_HINTS_PROGRAM_POSITION", "program-position" },
        { META_SIZE_HINTS_PROGRAM_SIZE, "META_SIZE_HINTS_PROGRAM_SIZE", "program-size" },
        { META_SIZE_HINTS_PROGRAM_MIN_SIZE, "META_SIZE_HINTS_PROGRAM_MIN_SIZE", "program-min-size" },
        { META_SIZE_HINTS_PROGRAM_MAX_SIZE, "META_SIZE_HINTS_PROGRAM_MAX_SIZE", "program-max-size" },
        { META_SIZE_HINTS_PROGRAM_RESIZE_INCREMENTS, "META_SIZE_HINTS_PROGRAM_RESIZE_INCREMENTS", "program-resize-increments" },
        { META_SIZE_HINTS_PROGRAM_ASPECT, "META_SIZE_HINTS_PROGRAM_ASPECT", "program-aspect" },
        { META_SIZE_HINTS_PROGRAM_BASE_SIZE, "META_SIZE_HINTS_PROGRAM_BASE_SIZE", "program-base-size" },
        { META_SIZE_HINTS_PROGRAM_WIN_GRAVITY, "META_SIZE_HINTS_PROGRAM_WIN_GRAVITY", "program-win-gravity" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaSizeHintsFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* Generated data ends here */

