
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS
/* enumerations from "meta-screen-cast-session.h" */
GType meta_screen_cast_session_type_get_type (void) G_GNUC_CONST;
#define META_TYPE_SCREEN_CAST_SESSION_TYPE (meta_screen_cast_session_type_get_type())
/* enumerations from "meta-screen-cast.h" */
GType meta_screen_cast_cursor_mode_get_type (void) G_GNUC_CONST;
#define META_TYPE_SCREEN_CAST_CURSOR_MODE (meta_screen_cast_cursor_mode_get_type())
GType meta_screen_cast_flag_get_type (void) G_GNUC_CONST;
#define META_TYPE_SCREEN_CAST_FLAG (meta_screen_cast_flag_get_type())
/* enumerations from "meta-backend-native-types.h" */
GType meta_seat_native_flag_get_type (void) G_GNUC_CONST;
#define META_TYPE_SEAT_NATIVE_FLAG (meta_seat_native_flag_get_type())
GType meta_backend_native_mode_get_type (void) G_GNUC_CONST;
#define META_TYPE_BACKEND_NATIVE_MODE (meta_backend_native_mode_get_type())
/* enumerations from "meta-drm-buffer.h" */
GType meta_drm_buffer_flags_get_type (void) G_GNUC_CONST;
#define META_TYPE_DRM_BUFFER_FLAGS (meta_drm_buffer_flags_get_type())
/* enumerations from "meta-kms-types.h" */
GType meta_kms_device_flag_get_type (void) G_GNUC_CONST;
#define META_TYPE_KMS_DEVICE_FLAG (meta_kms_device_flag_get_type())
GType meta_kms_resource_changes_get_type (void) G_GNUC_CONST;
#define META_TYPE_KMS_RESOURCE_CHANGES (meta_kms_resource_changes_get_type())
GType meta_kms_update_flag_get_type (void) G_GNUC_CONST;
#define META_TYPE_KMS_UPDATE_FLAG (meta_kms_update_flag_get_type())
GType meta_kms_prop_type_get_type (void) G_GNUC_CONST;
#define META_TYPE_KMS_PROP_TYPE (meta_kms_prop_type_get_type())
GType meta_kms_inhibit_subset_get_type (void) G_GNUC_CONST;
#define META_TYPE_KMS_INHIBIT_SUBSET (meta_kms_inhibit_subset_get_type())
/* enumerations from "meta-thread.h" */
GType meta_thread_type_get_type (void) G_GNUC_CONST;
#define META_TYPE_THREAD_TYPE (meta_thread_type_get_type())
GType meta_scheduling_priority_get_type (void) G_GNUC_CONST;
#define META_TYPE_SCHEDULING_PRIORITY (meta_scheduling_priority_get_type())
/* enumerations from "window-private.h" */
GType meta_client_type_get_type (void) G_GNUC_CONST;
#define META_TYPE_CLIENT_TYPE (meta_client_type_get_type())
GType meta_move_resize_flags_get_type (void) G_GNUC_CONST;
#define META_TYPE_MOVE_RESIZE_FLAGS (meta_move_resize_flags_get_type())
GType meta_place_flag_get_type (void) G_GNUC_CONST;
#define META_TYPE_PLACE_FLAG (meta_place_flag_get_type())
GType meta_move_resize_result_flags_get_type (void) G_GNUC_CONST;
#define META_TYPE_MOVE_RESIZE_RESULT_FLAGS (meta_move_resize_result_flags_get_type())
GType meta_placement_gravity_get_type (void) G_GNUC_CONST;
#define META_TYPE_PLACEMENT_GRAVITY (meta_placement_gravity_get_type())
GType meta_placement_anchor_get_type (void) G_GNUC_CONST;
#define META_TYPE_PLACEMENT_ANCHOR (meta_placement_anchor_get_type())
GType meta_placement_constraint_adjustment_get_type (void) G_GNUC_CONST;
#define META_TYPE_PLACEMENT_CONSTRAINT_ADJUSTMENT (meta_placement_constraint_adjustment_get_type())
GType meta_window_update_monitor_flags_get_type (void) G_GNUC_CONST;
#define META_TYPE_WINDOW_UPDATE_MONITOR_FLAGS (meta_window_update_monitor_flags_get_type())
GType meta_window_suspend_state_get_type (void) G_GNUC_CONST;
#define META_TYPE_WINDOW_SUSPEND_STATE (meta_window_suspend_state_get_type())
GType meta_window_apply_flags_get_type (void) G_GNUC_CONST;
#define META_TYPE_WINDOW_APPLY_FLAGS (meta_window_apply_flags_get_type())
GType meta_placement_state_get_type (void) G_GNUC_CONST;
#define META_TYPE_PLACEMENT_STATE (meta_placement_state_get_type())
GType meta_edge_constraint_get_type (void) G_GNUC_CONST;
#define META_TYPE_EDGE_CONSTRAINT (meta_edge_constraint_get_type())
GType meta_edge_resistance_flags_get_type (void) G_GNUC_CONST;
#define META_TYPE_EDGE_RESISTANCE_FLAGS (meta_edge_resistance_flags_get_type())
GType meta_size_hints_flags_get_type (void) G_GNUC_CONST;
#define META_TYPE_SIZE_HINTS_FLAGS (meta_size_hints_flags_get_type())
G_END_DECLS

/* Generated data ends here */

