/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __META_DBUS_SYSPROF3_PROFILER_H__
#define __META_DBUS_SYSPROF3_PROFILER_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Sysprof3.Profiler */

#define META_DBUS_TYPE_SYSPROF3_PROFILER (meta_dbus_sysprof3_profiler_get_type ())
#define META_DBUS_SYSPROF3_PROFILER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_SYSPROF3_PROFILER, MetaDBusSysprof3Profiler))
#define META_DBUS_IS_SYSPROF3_PROFILER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_SYSPROF3_PROFILER))
#define META_DBUS_SYSPROF3_PROFILER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), META_DBUS_TYPE_SYSPROF3_PROFILER, MetaDBusSysprof3ProfilerIface))

struct _MetaDBusSysprof3Profiler;
typedef struct _MetaDBusSysprof3Profiler MetaDBusSysprof3Profiler;
typedef struct _MetaDBusSysprof3ProfilerIface MetaDBusSysprof3ProfilerIface;

struct _MetaDBusSysprof3ProfilerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_start) (
    MetaDBusSysprof3Profiler *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    GVariant *arg_options,
    GVariant *arg_fd);

  gboolean (*handle_stop) (
    MetaDBusSysprof3Profiler *object,
    GDBusMethodInvocation *invocation);

  GVariant * (*get_capabilities) (MetaDBusSysprof3Profiler *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusSysprof3Profiler, g_object_unref)
#endif

GType meta_dbus_sysprof3_profiler_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *meta_dbus_sysprof3_profiler_interface_info (void);
guint meta_dbus_sysprof3_profiler_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void meta_dbus_sysprof3_profiler_complete_start (
    MetaDBusSysprof3Profiler *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list);

void meta_dbus_sysprof3_profiler_complete_stop (
    MetaDBusSysprof3Profiler *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void meta_dbus_sysprof3_profiler_call_start (
    MetaDBusSysprof3Profiler *proxy,
    GVariant *arg_options,
    GVariant *arg_fd,
    GUnixFDList *fd_list,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_sysprof3_profiler_call_start_finish (
    MetaDBusSysprof3Profiler *proxy,
    GUnixFDList **out_fd_list,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_sysprof3_profiler_call_start_sync (
    MetaDBusSysprof3Profiler *proxy,
    GVariant *arg_options,
    GVariant *arg_fd,
    GUnixFDList  *fd_list,
    GUnixFDList **out_fd_list,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_sysprof3_profiler_call_stop (
    MetaDBusSysprof3Profiler *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_sysprof3_profiler_call_stop_finish (
    MetaDBusSysprof3Profiler *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_sysprof3_profiler_call_stop_sync (
    MetaDBusSysprof3Profiler *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
GVariant *meta_dbus_sysprof3_profiler_get_capabilities (MetaDBusSysprof3Profiler *object);
GVariant *meta_dbus_sysprof3_profiler_dup_capabilities (MetaDBusSysprof3Profiler *object);
void meta_dbus_sysprof3_profiler_set_capabilities (MetaDBusSysprof3Profiler *object, GVariant *value);


/* ---- */

#define META_DBUS_TYPE_SYSPROF3_PROFILER_PROXY (meta_dbus_sysprof3_profiler_proxy_get_type ())
#define META_DBUS_SYSPROF3_PROFILER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_SYSPROF3_PROFILER_PROXY, MetaDBusSysprof3ProfilerProxy))
#define META_DBUS_SYSPROF3_PROFILER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_SYSPROF3_PROFILER_PROXY, MetaDBusSysprof3ProfilerProxyClass))
#define META_DBUS_SYSPROF3_PROFILER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_SYSPROF3_PROFILER_PROXY, MetaDBusSysprof3ProfilerProxyClass))
#define META_DBUS_IS_SYSPROF3_PROFILER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_SYSPROF3_PROFILER_PROXY))
#define META_DBUS_IS_SYSPROF3_PROFILER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_SYSPROF3_PROFILER_PROXY))

typedef struct _MetaDBusSysprof3ProfilerProxy MetaDBusSysprof3ProfilerProxy;
typedef struct _MetaDBusSysprof3ProfilerProxyClass MetaDBusSysprof3ProfilerProxyClass;
typedef struct _MetaDBusSysprof3ProfilerProxyPrivate MetaDBusSysprof3ProfilerProxyPrivate;

struct _MetaDBusSysprof3ProfilerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MetaDBusSysprof3ProfilerProxyPrivate *priv;
};

struct _MetaDBusSysprof3ProfilerProxyClass
{
  GDBusProxyClass parent_class;
};

GType meta_dbus_sysprof3_profiler_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusSysprof3ProfilerProxy, g_object_unref)
#endif

void meta_dbus_sysprof3_profiler_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusSysprof3Profiler *meta_dbus_sysprof3_profiler_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusSysprof3Profiler *meta_dbus_sysprof3_profiler_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void meta_dbus_sysprof3_profiler_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusSysprof3Profiler *meta_dbus_sysprof3_profiler_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusSysprof3Profiler *meta_dbus_sysprof3_profiler_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define META_DBUS_TYPE_SYSPROF3_PROFILER_SKELETON (meta_dbus_sysprof3_profiler_skeleton_get_type ())
#define META_DBUS_SYSPROF3_PROFILER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_SYSPROF3_PROFILER_SKELETON, MetaDBusSysprof3ProfilerSkeleton))
#define META_DBUS_SYSPROF3_PROFILER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_SYSPROF3_PROFILER_SKELETON, MetaDBusSysprof3ProfilerSkeletonClass))
#define META_DBUS_SYSPROF3_PROFILER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_SYSPROF3_PROFILER_SKELETON, MetaDBusSysprof3ProfilerSkeletonClass))
#define META_DBUS_IS_SYSPROF3_PROFILER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_SYSPROF3_PROFILER_SKELETON))
#define META_DBUS_IS_SYSPROF3_PROFILER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_SYSPROF3_PROFILER_SKELETON))

typedef struct _MetaDBusSysprof3ProfilerSkeleton MetaDBusSysprof3ProfilerSkeleton;
typedef struct _MetaDBusSysprof3ProfilerSkeletonClass MetaDBusSysprof3ProfilerSkeletonClass;
typedef struct _MetaDBusSysprof3ProfilerSkeletonPrivate MetaDBusSysprof3ProfilerSkeletonPrivate;

struct _MetaDBusSysprof3ProfilerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MetaDBusSysprof3ProfilerSkeletonPrivate *priv;
};

struct _MetaDBusSysprof3ProfilerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType meta_dbus_sysprof3_profiler_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusSysprof3ProfilerSkeleton, g_object_unref)
#endif

MetaDBusSysprof3Profiler *meta_dbus_sysprof3_profiler_skeleton_new (void);


G_END_DECLS

#endif /* __META_DBUS_SYSPROF3_PROFILER_H__ */
