/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __MDK_DBUS_DEVKIT_H__
#define __MDK_DBUS_DEVKIT_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.Devkit */

#define MDK_DBUS_TYPE_DEVKIT (mdk_dbus_devkit_get_type ())
#define MDK_DBUS_DEVKIT(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MDK_DBUS_TYPE_DEVKIT, MdkDBusDevkit))
#define MDK_DBUS_IS_DEVKIT(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MDK_DBUS_TYPE_DEVKIT))
#define MDK_DBUS_DEVKIT_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), MDK_DBUS_TYPE_DEVKIT, MdkDBusDevkitIface))

struct _MdkDBusDevkit;
typedef struct _MdkDBusDevkit MdkDBusDevkit;
typedef struct _MdkDBusDevkitIface MdkDBusDevkitIface;

struct _MdkDBusDevkitIface
{
  GTypeInterface parent_iface;

  GVariant * (*get_env) (MdkDBusDevkit *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MdkDBusDevkit, g_object_unref)
#endif

GType mdk_dbus_devkit_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *mdk_dbus_devkit_interface_info (void);
guint mdk_dbus_devkit_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
GVariant *mdk_dbus_devkit_get_env (MdkDBusDevkit *object);
GVariant *mdk_dbus_devkit_dup_env (MdkDBusDevkit *object);
void mdk_dbus_devkit_set_env (MdkDBusDevkit *object, GVariant *value);


/* ---- */

#define MDK_DBUS_TYPE_DEVKIT_PROXY (mdk_dbus_devkit_proxy_get_type ())
#define MDK_DBUS_DEVKIT_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MDK_DBUS_TYPE_DEVKIT_PROXY, MdkDBusDevkitProxy))
#define MDK_DBUS_DEVKIT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MDK_DBUS_TYPE_DEVKIT_PROXY, MdkDBusDevkitProxyClass))
#define MDK_DBUS_DEVKIT_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MDK_DBUS_TYPE_DEVKIT_PROXY, MdkDBusDevkitProxyClass))
#define MDK_DBUS_IS_DEVKIT_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MDK_DBUS_TYPE_DEVKIT_PROXY))
#define MDK_DBUS_IS_DEVKIT_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MDK_DBUS_TYPE_DEVKIT_PROXY))

typedef struct _MdkDBusDevkitProxy MdkDBusDevkitProxy;
typedef struct _MdkDBusDevkitProxyClass MdkDBusDevkitProxyClass;
typedef struct _MdkDBusDevkitProxyPrivate MdkDBusDevkitProxyPrivate;

struct _MdkDBusDevkitProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MdkDBusDevkitProxyPrivate *priv;
};

struct _MdkDBusDevkitProxyClass
{
  GDBusProxyClass parent_class;
};

GType mdk_dbus_devkit_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MdkDBusDevkitProxy, g_object_unref)
#endif

void mdk_dbus_devkit_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MdkDBusDevkit *mdk_dbus_devkit_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MdkDBusDevkit *mdk_dbus_devkit_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void mdk_dbus_devkit_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MdkDBusDevkit *mdk_dbus_devkit_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MdkDBusDevkit *mdk_dbus_devkit_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define MDK_DBUS_TYPE_DEVKIT_SKELETON (mdk_dbus_devkit_skeleton_get_type ())
#define MDK_DBUS_DEVKIT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), MDK_DBUS_TYPE_DEVKIT_SKELETON, MdkDBusDevkitSkeleton))
#define MDK_DBUS_DEVKIT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), MDK_DBUS_TYPE_DEVKIT_SKELETON, MdkDBusDevkitSkeletonClass))
#define MDK_DBUS_DEVKIT_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), MDK_DBUS_TYPE_DEVKIT_SKELETON, MdkDBusDevkitSkeletonClass))
#define MDK_DBUS_IS_DEVKIT_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), MDK_DBUS_TYPE_DEVKIT_SKELETON))
#define MDK_DBUS_IS_DEVKIT_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), MDK_DBUS_TYPE_DEVKIT_SKELETON))

typedef struct _MdkDBusDevkitSkeleton MdkDBusDevkitSkeleton;
typedef struct _MdkDBusDevkitSkeletonClass MdkDBusDevkitSkeletonClass;
typedef struct _MdkDBusDevkitSkeletonPrivate MdkDBusDevkitSkeletonPrivate;

struct _MdkDBusDevkitSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MdkDBusDevkitSkeletonPrivate *priv;
};

struct _MdkDBusDevkitSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType mdk_dbus_devkit_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MdkDBusDevkitSkeleton, g_object_unref)
#endif

MdkDBusDevkit *mdk_dbus_devkit_skeleton_new (void);


G_END_DECLS

#endif /* __MDK_DBUS_DEVKIT_H__ */
