/*
 * Copyright (C) 2021-2024 Red Hat Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <glib-object.h>
#include <gtk/gtk.h>

#include "mdk-types.h"

#define DEFAULT_MONITOR_WIDTH 1280
#define DEFAULT_MONITOR_HEIGHT 800

#define MDK_TYPE_STREAM (mdk_stream_get_type ())
G_DECLARE_FINAL_TYPE (MdkStream, mdk_stream,
                      MDK, STREAM,
                      GtkMediaStream)

MdkStream * mdk_stream_new_resizable (MdkSession  *session,
                                      double       scale,
                                      GError     **error);

MdkStream * mdk_stream_new_with_modes (MdkSession  *session,
                                       double       scale,
                                       GError     **error);

MdkSession * mdk_stream_get_session (MdkStream *stream);

const char * mdk_stream_get_path (MdkStream *stream);

const char * mdk_stream_get_mapping_id (MdkStream *stream);

double mdk_stream_get_scale (MdkStream *stream);

void mdk_stream_set_scale (MdkStream *stream,
                           double     scale);

void mdk_stream_resize (MdkStream *stream,
                        int        width,
                        int        height);
