
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#if !defined(__COGL_H_INSIDE__) && !defined(COGL_COMPILATION)
#error "Only <cogl/cogl.h> can be included directly."
#endif

#include "cogl/cogl-types.h"

G_BEGIN_DECLS
/* enumerations from "cogl-buffer.h" */
COGL_EXPORT GType cogl_buffer_update_hint_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_BUFFER_UPDATE_HINT (cogl_buffer_update_hint_get_type())
COGL_EXPORT GType cogl_buffer_access_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_BUFFER_ACCESS (cogl_buffer_access_get_type())
COGL_EXPORT GType cogl_buffer_map_hint_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_BUFFER_MAP_HINT (cogl_buffer_map_hint_get_type())
COGL_EXPORT GType cogl_buffer_bind_target_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_BUFFER_BIND_TARGET (cogl_buffer_bind_target_get_type())
/* enumerations from "cogl-pixel-format.h" */
COGL_EXPORT GType cogl_pixel_format_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_PIXEL_FORMAT (cogl_pixel_format_get_type())
/* enumerations from "cogl-types.h" */
COGL_EXPORT GType cogl_system_error_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_SYSTEM_ERROR (cogl_system_error_get_type())
COGL_EXPORT GType cogl_attribute_type_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_ATTRIBUTE_TYPE (cogl_attribute_type_get_type())
COGL_EXPORT GType cogl_indices_type_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_INDICES_TYPE (cogl_indices_type_get_type())
COGL_EXPORT GType cogl_vertices_mode_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_VERTICES_MODE (cogl_vertices_mode_get_type())
COGL_EXPORT GType cogl_depth_test_function_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_DEPTH_TEST_FUNCTION (cogl_depth_test_function_get_type())
COGL_EXPORT GType cogl_renderer_error_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_RENDERER_ERROR (cogl_renderer_error_get_type())
COGL_EXPORT GType cogl_winsys_feature_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_WINSYS_FEATURE (cogl_winsys_feature_get_type())
COGL_EXPORT GType cogl_winding_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_WINDING (cogl_winding_get_type())
COGL_EXPORT GType cogl_buffer_bit_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_BUFFER_BIT (cogl_buffer_bit_get_type())
COGL_EXPORT GType cogl_read_pixels_flags_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_READ_PIXELS_FLAGS (cogl_read_pixels_flags_get_type())
COGL_EXPORT GType cogl_shader_type_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_SHADER_TYPE (cogl_shader_type_get_type())
COGL_EXPORT GType cogl_scanout_error_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_SCANOUT_ERROR (cogl_scanout_error_get_type())
G_END_DECLS

/* Generated data ends here */

