/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __META_DBUS_REMOTE_DESKTOP_H__
#define __META_DBUS_REMOTE_DESKTOP_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.RemoteDesktop */

#define META_DBUS_TYPE_REMOTE_DESKTOP (meta_dbus_remote_desktop_get_type ())
#define META_DBUS_REMOTE_DESKTOP(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_REMOTE_DESKTOP, MetaDBusRemoteDesktop))
#define META_DBUS_IS_REMOTE_DESKTOP(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_REMOTE_DESKTOP))
#define META_DBUS_REMOTE_DESKTOP_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), META_DBUS_TYPE_REMOTE_DESKTOP, MetaDBusRemoteDesktopIface))

struct _MetaDBusRemoteDesktop;
typedef struct _MetaDBusRemoteDesktop MetaDBusRemoteDesktop;
typedef struct _MetaDBusRemoteDesktopIface MetaDBusRemoteDesktopIface;

struct _MetaDBusRemoteDesktopIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_create_session) (
    MetaDBusRemoteDesktop *object,
    GDBusMethodInvocation *invocation);

  guint  (*get_supported_device_types) (MetaDBusRemoteDesktop *object);

  gint  (*get_version) (MetaDBusRemoteDesktop *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusRemoteDesktop, g_object_unref)
#endif

GType meta_dbus_remote_desktop_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *meta_dbus_remote_desktop_interface_info (void);
guint meta_dbus_remote_desktop_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void meta_dbus_remote_desktop_complete_create_session (
    MetaDBusRemoteDesktop *object,
    GDBusMethodInvocation *invocation,
    const gchar *session_path);



/* D-Bus method calls: */
void meta_dbus_remote_desktop_call_create_session (
    MetaDBusRemoteDesktop *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_call_create_session_finish (
    MetaDBusRemoteDesktop *proxy,
    gchar ** out_session_path,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_call_create_session_sync (
    MetaDBusRemoteDesktop *proxy,
    gchar ** out_session_path,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
guint meta_dbus_remote_desktop_get_supported_device_types (MetaDBusRemoteDesktop *object);
void meta_dbus_remote_desktop_set_supported_device_types (MetaDBusRemoteDesktop *object, guint value);

gint meta_dbus_remote_desktop_get_version (MetaDBusRemoteDesktop *object);
void meta_dbus_remote_desktop_set_version (MetaDBusRemoteDesktop *object, gint value);


/* ---- */

#define META_DBUS_TYPE_REMOTE_DESKTOP_PROXY (meta_dbus_remote_desktop_proxy_get_type ())
#define META_DBUS_REMOTE_DESKTOP_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_REMOTE_DESKTOP_PROXY, MetaDBusRemoteDesktopProxy))
#define META_DBUS_REMOTE_DESKTOP_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_REMOTE_DESKTOP_PROXY, MetaDBusRemoteDesktopProxyClass))
#define META_DBUS_REMOTE_DESKTOP_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_REMOTE_DESKTOP_PROXY, MetaDBusRemoteDesktopProxyClass))
#define META_DBUS_IS_REMOTE_DESKTOP_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_REMOTE_DESKTOP_PROXY))
#define META_DBUS_IS_REMOTE_DESKTOP_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_REMOTE_DESKTOP_PROXY))

typedef struct _MetaDBusRemoteDesktopProxy MetaDBusRemoteDesktopProxy;
typedef struct _MetaDBusRemoteDesktopProxyClass MetaDBusRemoteDesktopProxyClass;
typedef struct _MetaDBusRemoteDesktopProxyPrivate MetaDBusRemoteDesktopProxyPrivate;

struct _MetaDBusRemoteDesktopProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MetaDBusRemoteDesktopProxyPrivate *priv;
};

struct _MetaDBusRemoteDesktopProxyClass
{
  GDBusProxyClass parent_class;
};

GType meta_dbus_remote_desktop_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusRemoteDesktopProxy, g_object_unref)
#endif

void meta_dbus_remote_desktop_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusRemoteDesktop *meta_dbus_remote_desktop_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusRemoteDesktop *meta_dbus_remote_desktop_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void meta_dbus_remote_desktop_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusRemoteDesktop *meta_dbus_remote_desktop_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusRemoteDesktop *meta_dbus_remote_desktop_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define META_DBUS_TYPE_REMOTE_DESKTOP_SKELETON (meta_dbus_remote_desktop_skeleton_get_type ())
#define META_DBUS_REMOTE_DESKTOP_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SKELETON, MetaDBusRemoteDesktopSkeleton))
#define META_DBUS_REMOTE_DESKTOP_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_REMOTE_DESKTOP_SKELETON, MetaDBusRemoteDesktopSkeletonClass))
#define META_DBUS_REMOTE_DESKTOP_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SKELETON, MetaDBusRemoteDesktopSkeletonClass))
#define META_DBUS_IS_REMOTE_DESKTOP_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SKELETON))
#define META_DBUS_IS_REMOTE_DESKTOP_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_REMOTE_DESKTOP_SKELETON))

typedef struct _MetaDBusRemoteDesktopSkeleton MetaDBusRemoteDesktopSkeleton;
typedef struct _MetaDBusRemoteDesktopSkeletonClass MetaDBusRemoteDesktopSkeletonClass;
typedef struct _MetaDBusRemoteDesktopSkeletonPrivate MetaDBusRemoteDesktopSkeletonPrivate;

struct _MetaDBusRemoteDesktopSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MetaDBusRemoteDesktopSkeletonPrivate *priv;
};

struct _MetaDBusRemoteDesktopSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType meta_dbus_remote_desktop_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusRemoteDesktopSkeleton, g_object_unref)
#endif

MetaDBusRemoteDesktop *meta_dbus_remote_desktop_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.RemoteDesktop.Session */

#define META_DBUS_TYPE_REMOTE_DESKTOP_SESSION (meta_dbus_remote_desktop_session_get_type ())
#define META_DBUS_REMOTE_DESKTOP_SESSION(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION, MetaDBusRemoteDesktopSession))
#define META_DBUS_IS_REMOTE_DESKTOP_SESSION(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION))
#define META_DBUS_REMOTE_DESKTOP_SESSION_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION, MetaDBusRemoteDesktopSessionIface))

struct _MetaDBusRemoteDesktopSession;
typedef struct _MetaDBusRemoteDesktopSession MetaDBusRemoteDesktopSession;
typedef struct _MetaDBusRemoteDesktopSessionIface MetaDBusRemoteDesktopSessionIface;

struct _MetaDBusRemoteDesktopSessionIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_connect_to_eis) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    GVariant *arg_options);

  gboolean (*handle_disable_clipboard) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_enable_clipboard) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_options);

  gboolean (*handle_notify_keyboard_keycode) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    guint arg_keycode,
    gboolean arg_state);

  gboolean (*handle_notify_keyboard_keysym) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    guint arg_keysym,
    gboolean arg_state);

  gboolean (*handle_notify_pointer_axis) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    gdouble arg_dx,
    gdouble arg_dy,
    guint arg_flags);

  gboolean (*handle_notify_pointer_axis_discrete) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    guint arg_axis,
    gint arg_steps);

  gboolean (*handle_notify_pointer_button) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    gint arg_button,
    gboolean arg_state);

  gboolean (*handle_notify_pointer_motion_absolute) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_stream,
    gdouble arg_x,
    gdouble arg_y);

  gboolean (*handle_notify_pointer_motion_relative) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    gdouble arg_dx,
    gdouble arg_dy);

  gboolean (*handle_notify_touch_down) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_stream,
    guint arg_slot,
    gdouble arg_x,
    gdouble arg_y);

  gboolean (*handle_notify_touch_motion) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_stream,
    guint arg_slot,
    gdouble arg_x,
    gdouble arg_y);

  gboolean (*handle_notify_touch_up) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    guint arg_slot);

  gboolean (*handle_selection_read) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    const gchar *arg_mime_type);

  gboolean (*handle_selection_write) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    guint arg_serial);

  gboolean (*handle_selection_write_done) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    guint arg_serial,
    gboolean arg_success);

  gboolean (*handle_set_keymap) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    GVariant *arg_options);

  gboolean (*handle_set_keymap_layout_index) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    guint arg_index);

  gboolean (*handle_set_selection) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_options);

  gboolean (*handle_start) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_stop) (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

  gboolean  (*get_caps_lock_state) (MetaDBusRemoteDesktopSession *object);

  GVariant * (*get_current_keymap) (MetaDBusRemoteDesktopSession *object);

  GVariant * (*get_keymap_capabilities) (MetaDBusRemoteDesktopSession *object);

  gboolean  (*get_num_lock_state) (MetaDBusRemoteDesktopSession *object);

  const gchar * (*get_session_id) (MetaDBusRemoteDesktopSession *object);

  void (*closed) (
    MetaDBusRemoteDesktopSession *object);

  void (*selection_owner_changed) (
    MetaDBusRemoteDesktopSession *object,
    GVariant *arg_options);

  void (*selection_transfer) (
    MetaDBusRemoteDesktopSession *object,
    const gchar *arg_mime_type,
    guint arg_serial);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusRemoteDesktopSession, g_object_unref)
#endif

GType meta_dbus_remote_desktop_session_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *meta_dbus_remote_desktop_session_interface_info (void);
guint meta_dbus_remote_desktop_session_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void meta_dbus_remote_desktop_session_complete_start (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_stop (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_keyboard_keycode (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_keyboard_keysym (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_pointer_button (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_pointer_axis (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_pointer_axis_discrete (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_pointer_motion_relative (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_pointer_motion_absolute (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_touch_down (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_touch_motion (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_notify_touch_up (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_enable_clipboard (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_disable_clipboard (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_set_selection (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_selection_write (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    GVariant *fd);

void meta_dbus_remote_desktop_session_complete_selection_write_done (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_remote_desktop_session_complete_selection_read (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    GVariant *fd);

void meta_dbus_remote_desktop_session_complete_connect_to_eis (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list,
    GVariant *fd);

void meta_dbus_remote_desktop_session_complete_set_keymap (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation,
    GUnixFDList *fd_list);

void meta_dbus_remote_desktop_session_complete_set_keymap_layout_index (
    MetaDBusRemoteDesktopSession *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void meta_dbus_remote_desktop_session_emit_closed (
    MetaDBusRemoteDesktopSession *object);

void meta_dbus_remote_desktop_session_emit_selection_owner_changed (
    MetaDBusRemoteDesktopSession *object,
    GVariant *arg_options);

void meta_dbus_remote_desktop_session_emit_selection_transfer (
    MetaDBusRemoteDesktopSession *object,
    const gchar *arg_mime_type,
    guint arg_serial);



/* D-Bus method calls: */
void meta_dbus_remote_desktop_session_call_start (
    MetaDBusRemoteDesktopSession *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_start_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_start_sync (
    MetaDBusRemoteDesktopSession *proxy,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_stop (
    MetaDBusRemoteDesktopSession *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_stop_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_stop_sync (
    MetaDBusRemoteDesktopSession *proxy,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_keyboard_keycode (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_keycode,
    gboolean arg_state,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_keyboard_keycode_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_keyboard_keycode_sync (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_keycode,
    gboolean arg_state,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_keyboard_keysym (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_keysym,
    gboolean arg_state,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_keyboard_keysym_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_keyboard_keysym_sync (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_keysym,
    gboolean arg_state,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_pointer_button (
    MetaDBusRemoteDesktopSession *proxy,
    gint arg_button,
    gboolean arg_state,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_button_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_button_sync (
    MetaDBusRemoteDesktopSession *proxy,
    gint arg_button,
    gboolean arg_state,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_pointer_axis (
    MetaDBusRemoteDesktopSession *proxy,
    gdouble arg_dx,
    gdouble arg_dy,
    guint arg_flags,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_axis_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_axis_sync (
    MetaDBusRemoteDesktopSession *proxy,
    gdouble arg_dx,
    gdouble arg_dy,
    guint arg_flags,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_pointer_axis_discrete (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_axis,
    gint arg_steps,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_axis_discrete_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_axis_discrete_sync (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_axis,
    gint arg_steps,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_pointer_motion_relative (
    MetaDBusRemoteDesktopSession *proxy,
    gdouble arg_dx,
    gdouble arg_dy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_motion_relative_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_motion_relative_sync (
    MetaDBusRemoteDesktopSession *proxy,
    gdouble arg_dx,
    gdouble arg_dy,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_pointer_motion_absolute (
    MetaDBusRemoteDesktopSession *proxy,
    const gchar *arg_stream,
    gdouble arg_x,
    gdouble arg_y,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_motion_absolute_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_pointer_motion_absolute_sync (
    MetaDBusRemoteDesktopSession *proxy,
    const gchar *arg_stream,
    gdouble arg_x,
    gdouble arg_y,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_touch_down (
    MetaDBusRemoteDesktopSession *proxy,
    const gchar *arg_stream,
    guint arg_slot,
    gdouble arg_x,
    gdouble arg_y,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_touch_down_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_touch_down_sync (
    MetaDBusRemoteDesktopSession *proxy,
    const gchar *arg_stream,
    guint arg_slot,
    gdouble arg_x,
    gdouble arg_y,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_touch_motion (
    MetaDBusRemoteDesktopSession *proxy,
    const gchar *arg_stream,
    guint arg_slot,
    gdouble arg_x,
    gdouble arg_y,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_touch_motion_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_touch_motion_sync (
    MetaDBusRemoteDesktopSession *proxy,
    const gchar *arg_stream,
    guint arg_slot,
    gdouble arg_x,
    gdouble arg_y,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_notify_touch_up (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_slot,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_notify_touch_up_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_notify_touch_up_sync (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_slot,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_enable_clipboard (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant *arg_options,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_enable_clipboard_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_enable_clipboard_sync (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant *arg_options,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_disable_clipboard (
    MetaDBusRemoteDesktopSession *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_disable_clipboard_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_disable_clipboard_sync (
    MetaDBusRemoteDesktopSession *proxy,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_set_selection (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant *arg_options,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_set_selection_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_set_selection_sync (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant *arg_options,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_selection_write (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_serial,
    GUnixFDList *fd_list,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_selection_write_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant ** out_fd,
    GUnixFDList **out_fd_list,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_selection_write_sync (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_serial,
    GUnixFDList  *fd_list,
    GVariant ** out_fd,
    GUnixFDList **out_fd_list,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_selection_write_done (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_serial,
    gboolean arg_success,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_selection_write_done_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_selection_write_done_sync (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_serial,
    gboolean arg_success,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_selection_read (
    MetaDBusRemoteDesktopSession *proxy,
    const gchar *arg_mime_type,
    GUnixFDList *fd_list,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_selection_read_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant ** out_fd,
    GUnixFDList **out_fd_list,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_selection_read_sync (
    MetaDBusRemoteDesktopSession *proxy,
    const gchar *arg_mime_type,
    GUnixFDList  *fd_list,
    GVariant ** out_fd,
    GUnixFDList **out_fd_list,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_connect_to_eis (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant *arg_options,
    GUnixFDList *fd_list,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_connect_to_eis_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant ** out_fd,
    GUnixFDList **out_fd_list,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_connect_to_eis_sync (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant *arg_options,
    GUnixFDList  *fd_list,
    GVariant ** out_fd,
    GUnixFDList **out_fd_list,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_set_keymap (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant *arg_options,
    GUnixFDList *fd_list,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_set_keymap_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GUnixFDList **out_fd_list,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_set_keymap_sync (
    MetaDBusRemoteDesktopSession *proxy,
    GVariant *arg_options,
    GUnixFDList  *fd_list,
    GUnixFDList **out_fd_list,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_remote_desktop_session_call_set_keymap_layout_index (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_index,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_remote_desktop_session_call_set_keymap_layout_index_finish (
    MetaDBusRemoteDesktopSession *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_remote_desktop_session_call_set_keymap_layout_index_sync (
    MetaDBusRemoteDesktopSession *proxy,
    guint arg_index,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
const gchar *meta_dbus_remote_desktop_session_get_session_id (MetaDBusRemoteDesktopSession *object);
gchar *meta_dbus_remote_desktop_session_dup_session_id (MetaDBusRemoteDesktopSession *object);
void meta_dbus_remote_desktop_session_set_session_id (MetaDBusRemoteDesktopSession *object, const gchar *value);

G_GNUC_DEPRECATED gboolean meta_dbus_remote_desktop_session_get_caps_lock_state (MetaDBusRemoteDesktopSession *object);
G_GNUC_DEPRECATED void meta_dbus_remote_desktop_session_set_caps_lock_state (MetaDBusRemoteDesktopSession *object, gboolean value);

G_GNUC_DEPRECATED gboolean meta_dbus_remote_desktop_session_get_num_lock_state (MetaDBusRemoteDesktopSession *object);
G_GNUC_DEPRECATED void meta_dbus_remote_desktop_session_set_num_lock_state (MetaDBusRemoteDesktopSession *object, gboolean value);

GVariant *meta_dbus_remote_desktop_session_get_keymap_capabilities (MetaDBusRemoteDesktopSession *object);
GVariant *meta_dbus_remote_desktop_session_dup_keymap_capabilities (MetaDBusRemoteDesktopSession *object);
void meta_dbus_remote_desktop_session_set_keymap_capabilities (MetaDBusRemoteDesktopSession *object, GVariant *value);

GVariant *meta_dbus_remote_desktop_session_get_current_keymap (MetaDBusRemoteDesktopSession *object);
GVariant *meta_dbus_remote_desktop_session_dup_current_keymap (MetaDBusRemoteDesktopSession *object);
void meta_dbus_remote_desktop_session_set_current_keymap (MetaDBusRemoteDesktopSession *object, GVariant *value);


/* ---- */

#define META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_PROXY (meta_dbus_remote_desktop_session_proxy_get_type ())
#define META_DBUS_REMOTE_DESKTOP_SESSION_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_PROXY, MetaDBusRemoteDesktopSessionProxy))
#define META_DBUS_REMOTE_DESKTOP_SESSION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_PROXY, MetaDBusRemoteDesktopSessionProxyClass))
#define META_DBUS_REMOTE_DESKTOP_SESSION_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_PROXY, MetaDBusRemoteDesktopSessionProxyClass))
#define META_DBUS_IS_REMOTE_DESKTOP_SESSION_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_PROXY))
#define META_DBUS_IS_REMOTE_DESKTOP_SESSION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_PROXY))

typedef struct _MetaDBusRemoteDesktopSessionProxy MetaDBusRemoteDesktopSessionProxy;
typedef struct _MetaDBusRemoteDesktopSessionProxyClass MetaDBusRemoteDesktopSessionProxyClass;
typedef struct _MetaDBusRemoteDesktopSessionProxyPrivate MetaDBusRemoteDesktopSessionProxyPrivate;

struct _MetaDBusRemoteDesktopSessionProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MetaDBusRemoteDesktopSessionProxyPrivate *priv;
};

struct _MetaDBusRemoteDesktopSessionProxyClass
{
  GDBusProxyClass parent_class;
};

GType meta_dbus_remote_desktop_session_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusRemoteDesktopSessionProxy, g_object_unref)
#endif

void meta_dbus_remote_desktop_session_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusRemoteDesktopSession *meta_dbus_remote_desktop_session_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusRemoteDesktopSession *meta_dbus_remote_desktop_session_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void meta_dbus_remote_desktop_session_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusRemoteDesktopSession *meta_dbus_remote_desktop_session_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusRemoteDesktopSession *meta_dbus_remote_desktop_session_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_SKELETON (meta_dbus_remote_desktop_session_skeleton_get_type ())
#define META_DBUS_REMOTE_DESKTOP_SESSION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_SKELETON, MetaDBusRemoteDesktopSessionSkeleton))
#define META_DBUS_REMOTE_DESKTOP_SESSION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_SKELETON, MetaDBusRemoteDesktopSessionSkeletonClass))
#define META_DBUS_REMOTE_DESKTOP_SESSION_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_SKELETON, MetaDBusRemoteDesktopSessionSkeletonClass))
#define META_DBUS_IS_REMOTE_DESKTOP_SESSION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_SKELETON))
#define META_DBUS_IS_REMOTE_DESKTOP_SESSION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_REMOTE_DESKTOP_SESSION_SKELETON))

typedef struct _MetaDBusRemoteDesktopSessionSkeleton MetaDBusRemoteDesktopSessionSkeleton;
typedef struct _MetaDBusRemoteDesktopSessionSkeletonClass MetaDBusRemoteDesktopSessionSkeletonClass;
typedef struct _MetaDBusRemoteDesktopSessionSkeletonPrivate MetaDBusRemoteDesktopSessionSkeletonPrivate;

struct _MetaDBusRemoteDesktopSessionSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MetaDBusRemoteDesktopSessionSkeletonPrivate *priv;
};

struct _MetaDBusRemoteDesktopSessionSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType meta_dbus_remote_desktop_session_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusRemoteDesktopSessionSkeleton, g_object_unref)
#endif

MetaDBusRemoteDesktopSession *meta_dbus_remote_desktop_session_skeleton_new (void);


G_END_DECLS

#endif /* __META_DBUS_REMOTE_DESKTOP_H__ */
