
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "clutter/clutter-enum-types.h"

/* enumerations from "/build/mutter/src/build/../mutter/clutter/clutter/clutter-color-state-params.h" */
#include "/build/mutter/src/build/../mutter/clutter/clutter/clutter-color-state-params.h"
GType
clutter_colorspace_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_COLORSPACE_SRGB, "CLUTTER_COLORSPACE_SRGB", "srgb" },
        { CLUTTER_COLORSPACE_BT2020, "CLUTTER_COLORSPACE_BT2020", "bt2020" },
        { CLUTTER_COLORSPACE_NTSC, "CLUTTER_COLORSPACE_NTSC", "ntsc" },
        { CLUTTER_COLORSPACE_PAL, "CLUTTER_COLORSPACE_PAL", "pal" },
        { CLUTTER_COLORSPACE_P3, "CLUTTER_COLORSPACE_P3", "p3" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterColorspace"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_transfer_function_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_TRANSFER_FUNCTION_SRGB_PIECEWISE, "CLUTTER_TRANSFER_FUNCTION_SRGB_PIECEWISE", "srgb-piecewise" },
        { CLUTTER_TRANSFER_FUNCTION_GAMMA22, "CLUTTER_TRANSFER_FUNCTION_GAMMA22", "gamma22" },
        { CLUTTER_TRANSFER_FUNCTION_PQ, "CLUTTER_TRANSFER_FUNCTION_PQ", "pq" },
        { CLUTTER_TRANSFER_FUNCTION_BT1886, "CLUTTER_TRANSFER_FUNCTION_BT1886", "bt1886" },
        { CLUTTER_TRANSFER_FUNCTION_LINEAR, "CLUTTER_TRANSFER_FUNCTION_LINEAR", "linear" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterTransferFunction"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_colorimetry_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_COLORIMETRY_TYPE_COLORSPACE, "CLUTTER_COLORIMETRY_TYPE_COLORSPACE", "colorspace" },
        { CLUTTER_COLORIMETRY_TYPE_PRIMARIES, "CLUTTER_COLORIMETRY_TYPE_PRIMARIES", "primaries" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterColorimetryType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_eotf_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_EOTF_TYPE_NAMED, "CLUTTER_EOTF_TYPE_NAMED", "named" },
        { CLUTTER_EOTF_TYPE_GAMMA, "CLUTTER_EOTF_TYPE_GAMMA", "gamma" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterEOTFType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_luminance_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_LUMINANCE_TYPE_DERIVED, "CLUTTER_LUMINANCE_TYPE_DERIVED", "derived" },
        { CLUTTER_LUMINANCE_TYPE_EXPLICIT, "CLUTTER_LUMINANCE_TYPE_EXPLICIT", "explicit" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterLuminanceType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_cicp_primaries_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_CICP_PRIMARIES_SRGB, "CLUTTER_CICP_PRIMARIES_SRGB", "srgb" },
        { CLUTTER_CICP_PRIMARIES_PAL, "CLUTTER_CICP_PRIMARIES_PAL", "pal" },
        { CLUTTER_CICP_PRIMARIES_NTSC, "CLUTTER_CICP_PRIMARIES_NTSC", "ntsc" },
        { CLUTTER_CICP_PRIMARIES_NTSC_2, "CLUTTER_CICP_PRIMARIES_NTSC_2", "ntsc-2" },
        { CLUTTER_CICP_PRIMARIES_BT2020, "CLUTTER_CICP_PRIMARIES_BT2020", "bt2020" },
        { CLUTTER_CICP_PRIMARIES_P3, "CLUTTER_CICP_PRIMARIES_P3", "p3" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterCicpPrimaries"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_cicp_transfer_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_CICP_TRANSFER_BT709, "CLUTTER_CICP_TRANSFER_BT709", "bt709" },
        { CLUTTER_CICP_TRANSFER_GAMMA22, "CLUTTER_CICP_TRANSFER_GAMMA22", "gamma22" },
        { CLUTTER_CICP_TRANSFER_GAMMA28, "CLUTTER_CICP_TRANSFER_GAMMA28", "gamma28" },
        { CLUTTER_CICP_TRANSFER_BT601, "CLUTTER_CICP_TRANSFER_BT601", "bt601" },
        { CLUTTER_CICP_TRANSFER_LINEAR, "CLUTTER_CICP_TRANSFER_LINEAR", "linear" },
        { CLUTTER_CICP_TRANSFER_SRGB, "CLUTTER_CICP_TRANSFER_SRGB", "srgb" },
        { CLUTTER_CICP_TRANSFER_BT2020, "CLUTTER_CICP_TRANSFER_BT2020", "bt2020" },
        { CLUTTER_CICP_TRANSFER_BT2020_2, "CLUTTER_CICP_TRANSFER_BT2020_2", "bt2020-2" },
        { CLUTTER_CICP_TRANSFER_PQ, "CLUTTER_CICP_TRANSFER_PQ", "pq" },
        { CLUTTER_CICP_TRANSFER_HLG, "CLUTTER_CICP_TRANSFER_HLG", "hlg" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterCicpTransfer"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/clutter/clutter/clutter-color-state.h" */
#include "/build/mutter/src/build/../mutter/clutter/clutter/clutter-color-state.h"
GType
clutter_color_state_transform_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_COLOR_STATE_TRANSFORM_OPAQUE, "CLUTTER_COLOR_STATE_TRANSFORM_OPAQUE", "opaque" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterColorStateTransformFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/clutter/clutter/clutter-enums.h" */
#include "/build/mutter/src/build/../mutter/clutter/clutter/clutter-enums.h"
GType
clutter_rotate_axis_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_X_AXIS, "CLUTTER_X_AXIS", "x-axis" },
        { CLUTTER_Y_AXIS, "CLUTTER_Y_AXIS", "y-axis" },
        { CLUTTER_Z_AXIS, "CLUTTER_Z_AXIS", "z-axis" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterRotateAxis"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_request_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_REQUEST_HEIGHT_FOR_WIDTH, "CLUTTER_REQUEST_HEIGHT_FOR_WIDTH", "height-for-width" },
        { CLUTTER_REQUEST_WIDTH_FOR_HEIGHT, "CLUTTER_REQUEST_WIDTH_FOR_HEIGHT", "width-for-height" },
        { CLUTTER_REQUEST_CONTENT_SIZE, "CLUTTER_REQUEST_CONTENT_SIZE", "content-size" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterRequestMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_animation_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_CUSTOM_MODE, "CLUTTER_CUSTOM_MODE", "custom-mode" },
        { CLUTTER_LINEAR, "CLUTTER_LINEAR", "linear" },
        { CLUTTER_EASE_IN_QUAD, "CLUTTER_EASE_IN_QUAD", "ease-in-quad" },
        { CLUTTER_EASE_OUT_QUAD, "CLUTTER_EASE_OUT_QUAD", "ease-out-quad" },
        { CLUTTER_EASE_IN_OUT_QUAD, "CLUTTER_EASE_IN_OUT_QUAD", "ease-in-out-quad" },
        { CLUTTER_EASE_IN_CUBIC, "CLUTTER_EASE_IN_CUBIC", "ease-in-cubic" },
        { CLUTTER_EASE_OUT_CUBIC, "CLUTTER_EASE_OUT_CUBIC", "ease-out-cubic" },
        { CLUTTER_EASE_IN_OUT_CUBIC, "CLUTTER_EASE_IN_OUT_CUBIC", "ease-in-out-cubic" },
        { CLUTTER_EASE_IN_QUART, "CLUTTER_EASE_IN_QUART", "ease-in-quart" },
        { CLUTTER_EASE_OUT_QUART, "CLUTTER_EASE_OUT_QUART", "ease-out-quart" },
        { CLUTTER_EASE_IN_OUT_QUART, "CLUTTER_EASE_IN_OUT_QUART", "ease-in-out-quart" },
        { CLUTTER_EASE_IN_QUINT, "CLUTTER_EASE_IN_QUINT", "ease-in-quint" },
        { CLUTTER_EASE_OUT_QUINT, "CLUTTER_EASE_OUT_QUINT", "ease-out-quint" },
        { CLUTTER_EASE_IN_OUT_QUINT, "CLUTTER_EASE_IN_OUT_QUINT", "ease-in-out-quint" },
        { CLUTTER_EASE_IN_SINE, "CLUTTER_EASE_IN_SINE", "ease-in-sine" },
        { CLUTTER_EASE_OUT_SINE, "CLUTTER_EASE_OUT_SINE", "ease-out-sine" },
        { CLUTTER_EASE_IN_OUT_SINE, "CLUTTER_EASE_IN_OUT_SINE", "ease-in-out-sine" },
        { CLUTTER_EASE_IN_EXPO, "CLUTTER_EASE_IN_EXPO", "ease-in-expo" },
        { CLUTTER_EASE_OUT_EXPO, "CLUTTER_EASE_OUT_EXPO", "ease-out-expo" },
        { CLUTTER_EASE_IN_OUT_EXPO, "CLUTTER_EASE_IN_OUT_EXPO", "ease-in-out-expo" },
        { CLUTTER_EASE_IN_CIRC, "CLUTTER_EASE_IN_CIRC", "ease-in-circ" },
        { CLUTTER_EASE_OUT_CIRC, "CLUTTER_EASE_OUT_CIRC", "ease-out-circ" },
        { CLUTTER_EASE_IN_OUT_CIRC, "CLUTTER_EASE_IN_OUT_CIRC", "ease-in-out-circ" },
        { CLUTTER_EASE_IN_ELASTIC, "CLUTTER_EASE_IN_ELASTIC", "ease-in-elastic" },
        { CLUTTER_EASE_OUT_ELASTIC, "CLUTTER_EASE_OUT_ELASTIC", "ease-out-elastic" },
        { CLUTTER_EASE_IN_OUT_ELASTIC, "CLUTTER_EASE_IN_OUT_ELASTIC", "ease-in-out-elastic" },
        { CLUTTER_EASE_IN_BACK, "CLUTTER_EASE_IN_BACK", "ease-in-back" },
        { CLUTTER_EASE_OUT_BACK, "CLUTTER_EASE_OUT_BACK", "ease-out-back" },
        { CLUTTER_EASE_IN_OUT_BACK, "CLUTTER_EASE_IN_OUT_BACK", "ease-in-out-back" },
        { CLUTTER_EASE_IN_BOUNCE, "CLUTTER_EASE_IN_BOUNCE", "ease-in-bounce" },
        { CLUTTER_EASE_OUT_BOUNCE, "CLUTTER_EASE_OUT_BOUNCE", "ease-out-bounce" },
        { CLUTTER_EASE_IN_OUT_BOUNCE, "CLUTTER_EASE_IN_OUT_BOUNCE", "ease-in-out-bounce" },
        { CLUTTER_STEPS, "CLUTTER_STEPS", "steps" },
        { CLUTTER_STEP_START, "CLUTTER_STEP_START", "step-start" },
        { CLUTTER_STEP_END, "CLUTTER_STEP_END", "step-end" },
        { CLUTTER_CUBIC_BEZIER, "CLUTTER_CUBIC_BEZIER", "cubic-bezier" },
        { CLUTTER_EASE, "CLUTTER_EASE", "ease" },
        { CLUTTER_EASE_IN, "CLUTTER_EASE_IN", "ease-in" },
        { CLUTTER_EASE_OUT, "CLUTTER_EASE_OUT", "ease-out" },
        { CLUTTER_EASE_IN_OUT, "CLUTTER_EASE_IN_OUT", "ease-in-out" },
        { CLUTTER_ANIMATION_LAST, "CLUTTER_ANIMATION_LAST", "animation-last" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterAnimationMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_text_direction_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_TEXT_DIRECTION_DEFAULT, "CLUTTER_TEXT_DIRECTION_DEFAULT", "default" },
        { CLUTTER_TEXT_DIRECTION_LTR, "CLUTTER_TEXT_DIRECTION_LTR", "ltr" },
        { CLUTTER_TEXT_DIRECTION_RTL, "CLUTTER_TEXT_DIRECTION_RTL", "rtl" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterTextDirection"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_modifier_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_SHIFT_MASK, "CLUTTER_SHIFT_MASK", "shift-mask" },
        { CLUTTER_LOCK_MASK, "CLUTTER_LOCK_MASK", "lock-mask" },
        { CLUTTER_CONTROL_MASK, "CLUTTER_CONTROL_MASK", "control-mask" },
        { CLUTTER_MOD1_MASK, "CLUTTER_MOD1_MASK", "mod1-mask" },
        { CLUTTER_MOD2_MASK, "CLUTTER_MOD2_MASK", "mod2-mask" },
        { CLUTTER_MOD3_MASK, "CLUTTER_MOD3_MASK", "mod3-mask" },
        { CLUTTER_MOD4_MASK, "CLUTTER_MOD4_MASK", "mod4-mask" },
        { CLUTTER_MOD5_MASK, "CLUTTER_MOD5_MASK", "mod5-mask" },
        { CLUTTER_BUTTON1_MASK, "CLUTTER_BUTTON1_MASK", "button1-mask" },
        { CLUTTER_BUTTON2_MASK, "CLUTTER_BUTTON2_MASK", "button2-mask" },
        { CLUTTER_BUTTON3_MASK, "CLUTTER_BUTTON3_MASK", "button3-mask" },
        { CLUTTER_BUTTON4_MASK, "CLUTTER_BUTTON4_MASK", "button4-mask" },
        { CLUTTER_BUTTON5_MASK, "CLUTTER_BUTTON5_MASK", "button5-mask" },
        { CLUTTER_MODIFIER_RESERVED_13_MASK, "CLUTTER_MODIFIER_RESERVED_13_MASK", "modifier-reserved-13-mask" },
        { CLUTTER_MODIFIER_RESERVED_14_MASK, "CLUTTER_MODIFIER_RESERVED_14_MASK", "modifier-reserved-14-mask" },
        { CLUTTER_MODIFIER_RESERVED_15_MASK, "CLUTTER_MODIFIER_RESERVED_15_MASK", "modifier-reserved-15-mask" },
        { CLUTTER_MODIFIER_RESERVED_16_MASK, "CLUTTER_MODIFIER_RESERVED_16_MASK", "modifier-reserved-16-mask" },
        { CLUTTER_MODIFIER_RESERVED_17_MASK, "CLUTTER_MODIFIER_RESERVED_17_MASK", "modifier-reserved-17-mask" },
        { CLUTTER_MODIFIER_RESERVED_18_MASK, "CLUTTER_MODIFIER_RESERVED_18_MASK", "modifier-reserved-18-mask" },
        { CLUTTER_MODIFIER_RESERVED_19_MASK, "CLUTTER_MODIFIER_RESERVED_19_MASK", "modifier-reserved-19-mask" },
        { CLUTTER_MODIFIER_RESERVED_20_MASK, "CLUTTER_MODIFIER_RESERVED_20_MASK", "modifier-reserved-20-mask" },
        { CLUTTER_MODIFIER_RESERVED_21_MASK, "CLUTTER_MODIFIER_RESERVED_21_MASK", "modifier-reserved-21-mask" },
        { CLUTTER_MODIFIER_RESERVED_22_MASK, "CLUTTER_MODIFIER_RESERVED_22_MASK", "modifier-reserved-22-mask" },
        { CLUTTER_MODIFIER_RESERVED_23_MASK, "CLUTTER_MODIFIER_RESERVED_23_MASK", "modifier-reserved-23-mask" },
        { CLUTTER_MODIFIER_RESERVED_24_MASK, "CLUTTER_MODIFIER_RESERVED_24_MASK", "modifier-reserved-24-mask" },
        { CLUTTER_MODIFIER_RESERVED_25_MASK, "CLUTTER_MODIFIER_RESERVED_25_MASK", "modifier-reserved-25-mask" },
        { CLUTTER_SUPER_MASK, "CLUTTER_SUPER_MASK", "super-mask" },
        { CLUTTER_HYPER_MASK, "CLUTTER_HYPER_MASK", "hyper-mask" },
        { CLUTTER_META_MASK, "CLUTTER_META_MASK", "meta-mask" },
        { CLUTTER_MODIFIER_RESERVED_29_MASK, "CLUTTER_MODIFIER_RESERVED_29_MASK", "modifier-reserved-29-mask" },
        { CLUTTER_RELEASE_MASK, "CLUTTER_RELEASE_MASK", "release-mask" },
        { CLUTTER_MODIFIER_MASK, "CLUTTER_MODIFIER_MASK", "modifier-mask" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterModifierType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_pointer_a11y_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_A11Y_SECONDARY_CLICK_ENABLED, "CLUTTER_A11Y_SECONDARY_CLICK_ENABLED", "secondary-click-enabled" },
        { CLUTTER_A11Y_DWELL_ENABLED, "CLUTTER_A11Y_DWELL_ENABLED", "dwell-enabled" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterPointerA11yFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_pointer_a11y_dwell_click_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_A11Y_DWELL_CLICK_TYPE_NONE, "CLUTTER_A11Y_DWELL_CLICK_TYPE_NONE", "none" },
        { CLUTTER_A11Y_DWELL_CLICK_TYPE_PRIMARY, "CLUTTER_A11Y_DWELL_CLICK_TYPE_PRIMARY", "primary" },
        { CLUTTER_A11Y_DWELL_CLICK_TYPE_SECONDARY, "CLUTTER_A11Y_DWELL_CLICK_TYPE_SECONDARY", "secondary" },
        { CLUTTER_A11Y_DWELL_CLICK_TYPE_MIDDLE, "CLUTTER_A11Y_DWELL_CLICK_TYPE_MIDDLE", "middle" },
        { CLUTTER_A11Y_DWELL_CLICK_TYPE_DOUBLE, "CLUTTER_A11Y_DWELL_CLICK_TYPE_DOUBLE", "double" },
        { CLUTTER_A11Y_DWELL_CLICK_TYPE_DRAG, "CLUTTER_A11Y_DWELL_CLICK_TYPE_DRAG", "drag" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterPointerA11yDwellClickType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_pointer_a11y_dwell_direction_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_A11Y_DWELL_DIRECTION_NONE, "CLUTTER_A11Y_DWELL_DIRECTION_NONE", "none" },
        { CLUTTER_A11Y_DWELL_DIRECTION_LEFT, "CLUTTER_A11Y_DWELL_DIRECTION_LEFT", "left" },
        { CLUTTER_A11Y_DWELL_DIRECTION_RIGHT, "CLUTTER_A11Y_DWELL_DIRECTION_RIGHT", "right" },
        { CLUTTER_A11Y_DWELL_DIRECTION_UP, "CLUTTER_A11Y_DWELL_DIRECTION_UP", "up" },
        { CLUTTER_A11Y_DWELL_DIRECTION_DOWN, "CLUTTER_A11Y_DWELL_DIRECTION_DOWN", "down" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterPointerA11yDwellDirection"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_pointer_a11y_dwell_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_A11Y_DWELL_MODE_WINDOW, "CLUTTER_A11Y_DWELL_MODE_WINDOW", "window" },
        { CLUTTER_A11Y_DWELL_MODE_GESTURE, "CLUTTER_A11Y_DWELL_MODE_GESTURE", "gesture" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterPointerA11yDwellMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_pointer_a11y_timeout_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_A11Y_TIMEOUT_TYPE_SECONDARY_CLICK, "CLUTTER_A11Y_TIMEOUT_TYPE_SECONDARY_CLICK", "secondary-click" },
        { CLUTTER_A11Y_TIMEOUT_TYPE_DWELL, "CLUTTER_A11Y_TIMEOUT_TYPE_DWELL", "dwell" },
        { CLUTTER_A11Y_TIMEOUT_TYPE_GESTURE, "CLUTTER_A11Y_TIMEOUT_TYPE_GESTURE", "gesture" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterPointerA11yTimeoutType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_offscreen_redirect_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_OFFSCREEN_REDIRECT_AUTOMATIC_FOR_OPACITY, "CLUTTER_OFFSCREEN_REDIRECT_AUTOMATIC_FOR_OPACITY", "automatic-for-opacity" },
        { CLUTTER_OFFSCREEN_REDIRECT_ALWAYS, "CLUTTER_OFFSCREEN_REDIRECT_ALWAYS", "always" },
        { CLUTTER_OFFSCREEN_REDIRECT_ON_IDLE, "CLUTTER_OFFSCREEN_REDIRECT_ON_IDLE", "on-idle" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterOffscreenRedirect"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_align_axis_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_ALIGN_X_AXIS, "CLUTTER_ALIGN_X_AXIS", "x-axis" },
        { CLUTTER_ALIGN_Y_AXIS, "CLUTTER_ALIGN_Y_AXIS", "y-axis" },
        { CLUTTER_ALIGN_BOTH, "CLUTTER_ALIGN_BOTH", "both" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterAlignAxis"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_bind_coordinate_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_BIND_X, "CLUTTER_BIND_X", "x" },
        { CLUTTER_BIND_Y, "CLUTTER_BIND_Y", "y" },
        { CLUTTER_BIND_WIDTH, "CLUTTER_BIND_WIDTH", "width" },
        { CLUTTER_BIND_HEIGHT, "CLUTTER_BIND_HEIGHT", "height" },
        { CLUTTER_BIND_POSITION, "CLUTTER_BIND_POSITION", "position" },
        { CLUTTER_BIND_SIZE, "CLUTTER_BIND_SIZE", "size" },
        { CLUTTER_BIND_ALL, "CLUTTER_BIND_ALL", "all" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterBindCoordinate"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_effect_paint_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_EFFECT_PAINT_ACTOR_DIRTY, "CLUTTER_EFFECT_PAINT_ACTOR_DIRTY", "actor-dirty" },
        { CLUTTER_EFFECT_PAINT_BYPASS_EFFECT, "CLUTTER_EFFECT_PAINT_BYPASS_EFFECT", "bypass-effect" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterEffectPaintFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_event_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_EVENT_NONE, "CLUTTER_EVENT_NONE", "none" },
        { CLUTTER_EVENT_FLAG_SYNTHETIC, "CLUTTER_EVENT_FLAG_SYNTHETIC", "flag-synthetic" },
        { CLUTTER_EVENT_FLAG_INPUT_METHOD, "CLUTTER_EVENT_FLAG_INPUT_METHOD", "flag-input-method" },
        { CLUTTER_EVENT_FLAG_REPEATED, "CLUTTER_EVENT_FLAG_REPEATED", "flag-repeated" },
        { CLUTTER_EVENT_FLAG_RELATIVE_MOTION, "CLUTTER_EVENT_FLAG_RELATIVE_MOTION", "flag-relative-motion" },
        { CLUTTER_EVENT_FLAG_GRAB_NOTIFY, "CLUTTER_EVENT_FLAG_GRAB_NOTIFY", "flag-grab-notify" },
        { CLUTTER_EVENT_FLAG_POINTER_EMULATED, "CLUTTER_EVENT_FLAG_POINTER_EMULATED", "flag-pointer-emulated" },
        { CLUTTER_EVENT_FLAG_A11Y_MODIFIER_FIRST_CLICK, "CLUTTER_EVENT_FLAG_A11Y_MODIFIER_FIRST_CLICK", "flag-a11y-modifier-first-click" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterEventFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_event_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_NOTHING, "CLUTTER_NOTHING", "nothing" },
        { CLUTTER_KEY_PRESS, "CLUTTER_KEY_PRESS", "key-press" },
        { CLUTTER_KEY_RELEASE, "CLUTTER_KEY_RELEASE", "key-release" },
        { CLUTTER_MOTION, "CLUTTER_MOTION", "motion" },
        { CLUTTER_ENTER, "CLUTTER_ENTER", "enter" },
        { CLUTTER_LEAVE, "CLUTTER_LEAVE", "leave" },
        { CLUTTER_BUTTON_PRESS, "CLUTTER_BUTTON_PRESS", "button-press" },
        { CLUTTER_BUTTON_RELEASE, "CLUTTER_BUTTON_RELEASE", "button-release" },
        { CLUTTER_SCROLL, "CLUTTER_SCROLL", "scroll" },
        { CLUTTER_TOUCH_BEGIN, "CLUTTER_TOUCH_BEGIN", "touch-begin" },
        { CLUTTER_TOUCH_UPDATE, "CLUTTER_TOUCH_UPDATE", "touch-update" },
        { CLUTTER_TOUCH_END, "CLUTTER_TOUCH_END", "touch-end" },
        { CLUTTER_TOUCH_CANCEL, "CLUTTER_TOUCH_CANCEL", "touch-cancel" },
        { CLUTTER_TOUCHPAD_PINCH, "CLUTTER_TOUCHPAD_PINCH", "touchpad-pinch" },
        { CLUTTER_TOUCHPAD_SWIPE, "CLUTTER_TOUCHPAD_SWIPE", "touchpad-swipe" },
        { CLUTTER_TOUCHPAD_HOLD, "CLUTTER_TOUCHPAD_HOLD", "touchpad-hold" },
        { CLUTTER_PROXIMITY_IN, "CLUTTER_PROXIMITY_IN", "proximity-in" },
        { CLUTTER_PROXIMITY_OUT, "CLUTTER_PROXIMITY_OUT", "proximity-out" },
        { CLUTTER_PAD_BUTTON_PRESS, "CLUTTER_PAD_BUTTON_PRESS", "pad-button-press" },
        { CLUTTER_PAD_BUTTON_RELEASE, "CLUTTER_PAD_BUTTON_RELEASE", "pad-button-release" },
        { CLUTTER_PAD_STRIP, "CLUTTER_PAD_STRIP", "pad-strip" },
        { CLUTTER_PAD_RING, "CLUTTER_PAD_RING", "pad-ring" },
        { CLUTTER_PAD_DIAL, "CLUTTER_PAD_DIAL", "pad-dial" },
        { CLUTTER_DEVICE_ADDED, "CLUTTER_DEVICE_ADDED", "device-added" },
        { CLUTTER_DEVICE_REMOVED, "CLUTTER_DEVICE_REMOVED", "device-removed" },
        { CLUTTER_IM_COMMIT, "CLUTTER_IM_COMMIT", "im-commit" },
        { CLUTTER_IM_DELETE, "CLUTTER_IM_DELETE", "im-delete" },
        { CLUTTER_IM_PREEDIT, "CLUTTER_IM_PREEDIT", "im-preedit" },
        { CLUTTER_KEY_STATE, "CLUTTER_KEY_STATE", "key-state" },
        { CLUTTER_EVENT_LAST, "CLUTTER_EVENT_LAST", "event-last" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterEventType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_scroll_direction_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_SCROLL_UP, "CLUTTER_SCROLL_UP", "up" },
        { CLUTTER_SCROLL_DOWN, "CLUTTER_SCROLL_DOWN", "down" },
        { CLUTTER_SCROLL_LEFT, "CLUTTER_SCROLL_LEFT", "left" },
        { CLUTTER_SCROLL_RIGHT, "CLUTTER_SCROLL_RIGHT", "right" },
        { CLUTTER_SCROLL_SMOOTH, "CLUTTER_SCROLL_SMOOTH", "smooth" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterScrollDirection"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_capabilities_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_INPUT_CAPABILITY_NONE, "CLUTTER_INPUT_CAPABILITY_NONE", "none" },
        { CLUTTER_INPUT_CAPABILITY_POINTER, "CLUTTER_INPUT_CAPABILITY_POINTER", "pointer" },
        { CLUTTER_INPUT_CAPABILITY_KEYBOARD, "CLUTTER_INPUT_CAPABILITY_KEYBOARD", "keyboard" },
        { CLUTTER_INPUT_CAPABILITY_TOUCHPAD, "CLUTTER_INPUT_CAPABILITY_TOUCHPAD", "touchpad" },
        { CLUTTER_INPUT_CAPABILITY_TOUCH, "CLUTTER_INPUT_CAPABILITY_TOUCH", "touch" },
        { CLUTTER_INPUT_CAPABILITY_TABLET_TOOL, "CLUTTER_INPUT_CAPABILITY_TABLET_TOOL", "tablet-tool" },
        { CLUTTER_INPUT_CAPABILITY_TABLET_PAD, "CLUTTER_INPUT_CAPABILITY_TABLET_PAD", "tablet-pad" },
        { CLUTTER_INPUT_CAPABILITY_TRACKBALL, "CLUTTER_INPUT_CAPABILITY_TRACKBALL", "trackball" },
        { CLUTTER_INPUT_CAPABILITY_TRACKPOINT, "CLUTTER_INPUT_CAPABILITY_TRACKPOINT", "trackpoint" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterInputCapabilities"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_device_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_POINTER_DEVICE, "CLUTTER_POINTER_DEVICE", "pointer-device" },
        { CLUTTER_KEYBOARD_DEVICE, "CLUTTER_KEYBOARD_DEVICE", "keyboard-device" },
        { CLUTTER_EXTENSION_DEVICE, "CLUTTER_EXTENSION_DEVICE", "extension-device" },
        { CLUTTER_JOYSTICK_DEVICE, "CLUTTER_JOYSTICK_DEVICE", "joystick-device" },
        { CLUTTER_TABLET_DEVICE, "CLUTTER_TABLET_DEVICE", "tablet-device" },
        { CLUTTER_TOUCHPAD_DEVICE, "CLUTTER_TOUCHPAD_DEVICE", "touchpad-device" },
        { CLUTTER_TOUCHSCREEN_DEVICE, "CLUTTER_TOUCHSCREEN_DEVICE", "touchscreen-device" },
        { CLUTTER_PEN_DEVICE, "CLUTTER_PEN_DEVICE", "pen-device" },
        { CLUTTER_ERASER_DEVICE, "CLUTTER_ERASER_DEVICE", "eraser-device" },
        { CLUTTER_CURSOR_DEVICE, "CLUTTER_CURSOR_DEVICE", "cursor-device" },
        { CLUTTER_PAD_DEVICE, "CLUTTER_PAD_DEVICE", "pad-device" },
        { CLUTTER_N_DEVICE_TYPES, "CLUTTER_N_DEVICE_TYPES", "n-device-types" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterInputDeviceType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_axis_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_INPUT_AXIS_IGNORE, "CLUTTER_INPUT_AXIS_IGNORE", "ignore" },
        { CLUTTER_INPUT_AXIS_X, "CLUTTER_INPUT_AXIS_X", "x" },
        { CLUTTER_INPUT_AXIS_Y, "CLUTTER_INPUT_AXIS_Y", "y" },
        { CLUTTER_INPUT_AXIS_PRESSURE, "CLUTTER_INPUT_AXIS_PRESSURE", "pressure" },
        { CLUTTER_INPUT_AXIS_XTILT, "CLUTTER_INPUT_AXIS_XTILT", "xtilt" },
        { CLUTTER_INPUT_AXIS_YTILT, "CLUTTER_INPUT_AXIS_YTILT", "ytilt" },
        { CLUTTER_INPUT_AXIS_WHEEL, "CLUTTER_INPUT_AXIS_WHEEL", "wheel" },
        { CLUTTER_INPUT_AXIS_DISTANCE, "CLUTTER_INPUT_AXIS_DISTANCE", "distance" },
        { CLUTTER_INPUT_AXIS_ROTATION, "CLUTTER_INPUT_AXIS_ROTATION", "rotation" },
        { CLUTTER_INPUT_AXIS_SLIDER, "CLUTTER_INPUT_AXIS_SLIDER", "slider" },
        { CLUTTER_INPUT_AXIS_LAST, "CLUTTER_INPUT_AXIS_LAST", "last" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterInputAxis"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_axis_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_INPUT_AXIS_FLAG_NONE, "CLUTTER_INPUT_AXIS_FLAG_NONE", "none" },
        { CLUTTER_INPUT_AXIS_FLAG_X, "CLUTTER_INPUT_AXIS_FLAG_X", "x" },
        { CLUTTER_INPUT_AXIS_FLAG_Y, "CLUTTER_INPUT_AXIS_FLAG_Y", "y" },
        { CLUTTER_INPUT_AXIS_FLAG_PRESSURE, "CLUTTER_INPUT_AXIS_FLAG_PRESSURE", "pressure" },
        { CLUTTER_INPUT_AXIS_FLAG_XTILT, "CLUTTER_INPUT_AXIS_FLAG_XTILT", "xtilt" },
        { CLUTTER_INPUT_AXIS_FLAG_YTILT, "CLUTTER_INPUT_AXIS_FLAG_YTILT", "ytilt" },
        { CLUTTER_INPUT_AXIS_FLAG_WHEEL, "CLUTTER_INPUT_AXIS_FLAG_WHEEL", "wheel" },
        { CLUTTER_INPUT_AXIS_FLAG_DISTANCE, "CLUTTER_INPUT_AXIS_FLAG_DISTANCE", "distance" },
        { CLUTTER_INPUT_AXIS_FLAG_ROTATION, "CLUTTER_INPUT_AXIS_FLAG_ROTATION", "rotation" },
        { CLUTTER_INPUT_AXIS_FLAG_SLIDER, "CLUTTER_INPUT_AXIS_FLAG_SLIDER", "slider" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterInputAxisFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_snap_edge_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_SNAP_EDGE_TOP, "CLUTTER_SNAP_EDGE_TOP", "top" },
        { CLUTTER_SNAP_EDGE_RIGHT, "CLUTTER_SNAP_EDGE_RIGHT", "right" },
        { CLUTTER_SNAP_EDGE_BOTTOM, "CLUTTER_SNAP_EDGE_BOTTOM", "bottom" },
        { CLUTTER_SNAP_EDGE_LEFT, "CLUTTER_SNAP_EDGE_LEFT", "left" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterSnapEdge"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_pick_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_PICK_NONE, "CLUTTER_PICK_NONE", "none" },
        { CLUTTER_PICK_REACTIVE, "CLUTTER_PICK_REACTIVE", "reactive" },
        { CLUTTER_PICK_ALL, "CLUTTER_PICK_ALL", "all" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterPickMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_pan_axis_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_PAN_AXIS_BOTH, "CLUTTER_PAN_AXIS_BOTH", "both" },
        { CLUTTER_PAN_AXIS_X, "CLUTTER_PAN_AXIS_X", "x" },
        { CLUTTER_PAN_AXIS_Y, "CLUTTER_PAN_AXIS_Y", "y" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterPanAxis"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_timeline_direction_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_TIMELINE_FORWARD, "CLUTTER_TIMELINE_FORWARD", "forward" },
        { CLUTTER_TIMELINE_BACKWARD, "CLUTTER_TIMELINE_BACKWARD", "backward" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterTimelineDirection"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_actor_align_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_ACTOR_ALIGN_FILL, "CLUTTER_ACTOR_ALIGN_FILL", "fill" },
        { CLUTTER_ACTOR_ALIGN_START, "CLUTTER_ACTOR_ALIGN_START", "start" },
        { CLUTTER_ACTOR_ALIGN_CENTER, "CLUTTER_ACTOR_ALIGN_CENTER", "center" },
        { CLUTTER_ACTOR_ALIGN_END, "CLUTTER_ACTOR_ALIGN_END", "end" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterActorAlign"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_repaint_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_REPAINT_FLAGS_PRE_PAINT, "CLUTTER_REPAINT_FLAGS_PRE_PAINT", "pre-paint" },
        { CLUTTER_REPAINT_FLAGS_POST_PAINT, "CLUTTER_REPAINT_FLAGS_POST_PAINT", "post-paint" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterRepaintFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_content_gravity_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_CONTENT_GRAVITY_TOP_LEFT, "CLUTTER_CONTENT_GRAVITY_TOP_LEFT", "top-left" },
        { CLUTTER_CONTENT_GRAVITY_TOP, "CLUTTER_CONTENT_GRAVITY_TOP", "top" },
        { CLUTTER_CONTENT_GRAVITY_TOP_RIGHT, "CLUTTER_CONTENT_GRAVITY_TOP_RIGHT", "top-right" },
        { CLUTTER_CONTENT_GRAVITY_LEFT, "CLUTTER_CONTENT_GRAVITY_LEFT", "left" },
        { CLUTTER_CONTENT_GRAVITY_CENTER, "CLUTTER_CONTENT_GRAVITY_CENTER", "center" },
        { CLUTTER_CONTENT_GRAVITY_RIGHT, "CLUTTER_CONTENT_GRAVITY_RIGHT", "right" },
        { CLUTTER_CONTENT_GRAVITY_BOTTOM_LEFT, "CLUTTER_CONTENT_GRAVITY_BOTTOM_LEFT", "bottom-left" },
        { CLUTTER_CONTENT_GRAVITY_BOTTOM, "CLUTTER_CONTENT_GRAVITY_BOTTOM", "bottom" },
        { CLUTTER_CONTENT_GRAVITY_BOTTOM_RIGHT, "CLUTTER_CONTENT_GRAVITY_BOTTOM_RIGHT", "bottom-right" },
        { CLUTTER_CONTENT_GRAVITY_RESIZE_FILL, "CLUTTER_CONTENT_GRAVITY_RESIZE_FILL", "resize-fill" },
        { CLUTTER_CONTENT_GRAVITY_RESIZE_ASPECT, "CLUTTER_CONTENT_GRAVITY_RESIZE_ASPECT", "resize-aspect" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterContentGravity"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_scaling_filter_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_SCALING_FILTER_LINEAR, "CLUTTER_SCALING_FILTER_LINEAR", "linear" },
        { CLUTTER_SCALING_FILTER_NEAREST, "CLUTTER_SCALING_FILTER_NEAREST", "nearest" },
        { CLUTTER_SCALING_FILTER_TRILINEAR, "CLUTTER_SCALING_FILTER_TRILINEAR", "trilinear" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterScalingFilter"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_orientation_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_ORIENTATION_HORIZONTAL, "CLUTTER_ORIENTATION_HORIZONTAL", "horizontal" },
        { CLUTTER_ORIENTATION_VERTICAL, "CLUTTER_ORIENTATION_VERTICAL", "vertical" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterOrientation"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_grid_position_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_GRID_POSITION_LEFT, "CLUTTER_GRID_POSITION_LEFT", "left" },
        { CLUTTER_GRID_POSITION_RIGHT, "CLUTTER_GRID_POSITION_RIGHT", "right" },
        { CLUTTER_GRID_POSITION_TOP, "CLUTTER_GRID_POSITION_TOP", "top" },
        { CLUTTER_GRID_POSITION_BOTTOM, "CLUTTER_GRID_POSITION_BOTTOM", "bottom" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterGridPosition"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_content_repeat_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_REPEAT_NONE, "CLUTTER_REPEAT_NONE", "none" },
        { CLUTTER_REPEAT_X_AXIS, "CLUTTER_REPEAT_X_AXIS", "x-axis" },
        { CLUTTER_REPEAT_Y_AXIS, "CLUTTER_REPEAT_Y_AXIS", "y-axis" },
        { CLUTTER_REPEAT_BOTH, "CLUTTER_REPEAT_BOTH", "both" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterContentRepeat"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_encoding_required_format_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_ENCODING_REQUIRED_FORMAT_UINT8, "CLUTTER_ENCODING_REQUIRED_FORMAT_UINT8", "uint8" },
        { CLUTTER_ENCODING_REQUIRED_FORMAT_UINT10, "CLUTTER_ENCODING_REQUIRED_FORMAT_UINT10", "uint10" },
        { CLUTTER_ENCODING_REQUIRED_FORMAT_FP16, "CLUTTER_ENCODING_REQUIRED_FORMAT_FP16", "fp16" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterEncodingRequiredFormat"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_step_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_STEP_MODE_START, "CLUTTER_STEP_MODE_START", "start" },
        { CLUTTER_STEP_MODE_END, "CLUTTER_STEP_MODE_END", "end" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterStepMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_touchpad_gesture_phase_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_TOUCHPAD_GESTURE_PHASE_BEGIN, "CLUTTER_TOUCHPAD_GESTURE_PHASE_BEGIN", "begin" },
        { CLUTTER_TOUCHPAD_GESTURE_PHASE_UPDATE, "CLUTTER_TOUCHPAD_GESTURE_PHASE_UPDATE", "update" },
        { CLUTTER_TOUCHPAD_GESTURE_PHASE_END, "CLUTTER_TOUCHPAD_GESTURE_PHASE_END", "end" },
        { CLUTTER_TOUCHPAD_GESTURE_PHASE_CANCEL, "CLUTTER_TOUCHPAD_GESTURE_PHASE_CANCEL", "cancel" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterTouchpadGesturePhase"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_scroll_source_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_SCROLL_SOURCE_UNKNOWN, "CLUTTER_SCROLL_SOURCE_UNKNOWN", "unknown" },
        { CLUTTER_SCROLL_SOURCE_WHEEL, "CLUTTER_SCROLL_SOURCE_WHEEL", "wheel" },
        { CLUTTER_SCROLL_SOURCE_FINGER, "CLUTTER_SCROLL_SOURCE_FINGER", "finger" },
        { CLUTTER_SCROLL_SOURCE_CONTINUOUS, "CLUTTER_SCROLL_SOURCE_CONTINUOUS", "continuous" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterScrollSource"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_scroll_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_SCROLL_NONE, "CLUTTER_SCROLL_NONE", "none" },
        { CLUTTER_SCROLL_INVERTED, "CLUTTER_SCROLL_INVERTED", "inverted" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterScrollFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_scroll_finish_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_SCROLL_FINISHED_NONE, "CLUTTER_SCROLL_FINISHED_NONE", "none" },
        { CLUTTER_SCROLL_FINISHED_HORIZONTAL, "CLUTTER_SCROLL_FINISHED_HORIZONTAL", "horizontal" },
        { CLUTTER_SCROLL_FINISHED_VERTICAL, "CLUTTER_SCROLL_FINISHED_VERTICAL", "vertical" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterScrollFinishFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_device_tool_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_INPUT_DEVICE_TOOL_NONE, "CLUTTER_INPUT_DEVICE_TOOL_NONE", "none" },
        { CLUTTER_INPUT_DEVICE_TOOL_PEN, "CLUTTER_INPUT_DEVICE_TOOL_PEN", "pen" },
        { CLUTTER_INPUT_DEVICE_TOOL_ERASER, "CLUTTER_INPUT_DEVICE_TOOL_ERASER", "eraser" },
        { CLUTTER_INPUT_DEVICE_TOOL_BRUSH, "CLUTTER_INPUT_DEVICE_TOOL_BRUSH", "brush" },
        { CLUTTER_INPUT_DEVICE_TOOL_PENCIL, "CLUTTER_INPUT_DEVICE_TOOL_PENCIL", "pencil" },
        { CLUTTER_INPUT_DEVICE_TOOL_AIRBRUSH, "CLUTTER_INPUT_DEVICE_TOOL_AIRBRUSH", "airbrush" },
        { CLUTTER_INPUT_DEVICE_TOOL_MOUSE, "CLUTTER_INPUT_DEVICE_TOOL_MOUSE", "mouse" },
        { CLUTTER_INPUT_DEVICE_TOOL_LENS, "CLUTTER_INPUT_DEVICE_TOOL_LENS", "lens" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterInputDeviceToolType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_device_pad_source_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_INPUT_DEVICE_PAD_SOURCE_UNKNOWN, "CLUTTER_INPUT_DEVICE_PAD_SOURCE_UNKNOWN", "unknown" },
        { CLUTTER_INPUT_DEVICE_PAD_SOURCE_FINGER, "CLUTTER_INPUT_DEVICE_PAD_SOURCE_FINGER", "finger" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterInputDevicePadSource"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_device_pad_feature_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_PAD_FEATURE_BUTTON, "CLUTTER_PAD_FEATURE_BUTTON", "button" },
        { CLUTTER_PAD_FEATURE_RING, "CLUTTER_PAD_FEATURE_RING", "ring" },
        { CLUTTER_PAD_FEATURE_STRIP, "CLUTTER_PAD_FEATURE_STRIP", "strip" },
        { CLUTTER_PAD_FEATURE_DIAL, "CLUTTER_PAD_FEATURE_DIAL", "dial" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterInputDevicePadFeature"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_content_hint_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_INPUT_CONTENT_HINT_COMPLETION, "CLUTTER_INPUT_CONTENT_HINT_COMPLETION", "completion" },
        { CLUTTER_INPUT_CONTENT_HINT_SPELLCHECK, "CLUTTER_INPUT_CONTENT_HINT_SPELLCHECK", "spellcheck" },
        { CLUTTER_INPUT_CONTENT_HINT_AUTO_CAPITALIZATION, "CLUTTER_INPUT_CONTENT_HINT_AUTO_CAPITALIZATION", "auto-capitalization" },
        { CLUTTER_INPUT_CONTENT_HINT_LOWERCASE, "CLUTTER_INPUT_CONTENT_HINT_LOWERCASE", "lowercase" },
        { CLUTTER_INPUT_CONTENT_HINT_UPPERCASE, "CLUTTER_INPUT_CONTENT_HINT_UPPERCASE", "uppercase" },
        { CLUTTER_INPUT_CONTENT_HINT_TITLECASE, "CLUTTER_INPUT_CONTENT_HINT_TITLECASE", "titlecase" },
        { CLUTTER_INPUT_CONTENT_HINT_HIDDEN_TEXT, "CLUTTER_INPUT_CONTENT_HINT_HIDDEN_TEXT", "hidden-text" },
        { CLUTTER_INPUT_CONTENT_HINT_SENSITIVE_DATA, "CLUTTER_INPUT_CONTENT_HINT_SENSITIVE_DATA", "sensitive-data" },
        { CLUTTER_INPUT_CONTENT_HINT_LATIN, "CLUTTER_INPUT_CONTENT_HINT_LATIN", "latin" },
        { CLUTTER_INPUT_CONTENT_HINT_MULTILINE, "CLUTTER_INPUT_CONTENT_HINT_MULTILINE", "multiline" },
        { CLUTTER_INPUT_CONTENT_HINT_INHIBIT_OSK, "CLUTTER_INPUT_CONTENT_HINT_INHIBIT_OSK", "inhibit-osk" },
        { CLUTTER_INPUT_CONTENT_HINT_NO_EMOJI, "CLUTTER_INPUT_CONTENT_HINT_NO_EMOJI", "no-emoji" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterInputContentHintFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_content_purpose_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_INPUT_CONTENT_PURPOSE_NORMAL, "CLUTTER_INPUT_CONTENT_PURPOSE_NORMAL", "normal" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_ALPHA, "CLUTTER_INPUT_CONTENT_PURPOSE_ALPHA", "alpha" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_DIGITS, "CLUTTER_INPUT_CONTENT_PURPOSE_DIGITS", "digits" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_NUMBER, "CLUTTER_INPUT_CONTENT_PURPOSE_NUMBER", "number" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_PHONE, "CLUTTER_INPUT_CONTENT_PURPOSE_PHONE", "phone" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_URL, "CLUTTER_INPUT_CONTENT_PURPOSE_URL", "url" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_EMAIL, "CLUTTER_INPUT_CONTENT_PURPOSE_EMAIL", "email" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_NAME, "CLUTTER_INPUT_CONTENT_PURPOSE_NAME", "name" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_PASSWORD, "CLUTTER_INPUT_CONTENT_PURPOSE_PASSWORD", "password" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_DATE, "CLUTTER_INPUT_CONTENT_PURPOSE_DATE", "date" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_TIME, "CLUTTER_INPUT_CONTENT_PURPOSE_TIME", "time" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_DATETIME, "CLUTTER_INPUT_CONTENT_PURPOSE_DATETIME", "datetime" },
        { CLUTTER_INPUT_CONTENT_PURPOSE_TERMINAL, "CLUTTER_INPUT_CONTENT_PURPOSE_TERMINAL", "terminal" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterInputContentPurpose"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_action_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_INPUT_ACTION_SUBMIT, "CLUTTER_INPUT_ACTION_SUBMIT", "submit" },
        { CLUTTER_INPUT_ACTION_LAST, "CLUTTER_INPUT_ACTION_LAST", "last" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterInputAction"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_action_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_INPUT_ACTION_FLAG_NONE, "CLUTTER_INPUT_ACTION_FLAG_NONE", "none" },
        { CLUTTER_INPUT_ACTION_FLAG_SUBMIT, "CLUTTER_INPUT_ACTION_FLAG_SUBMIT", "submit" },
        { CLUTTER_INPUT_ACTION_FLAG_ALL, "CLUTTER_INPUT_ACTION_FLAG_ALL", "all" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterInputActionFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_input_panel_state_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_INPUT_PANEL_STATE_OFF, "CLUTTER_INPUT_PANEL_STATE_OFF", "off" },
        { CLUTTER_INPUT_PANEL_STATE_ON, "CLUTTER_INPUT_PANEL_STATE_ON", "on" },
        { CLUTTER_INPUT_PANEL_STATE_TOGGLE, "CLUTTER_INPUT_PANEL_STATE_TOGGLE", "toggle" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterInputPanelState"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_preedit_reset_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_PREEDIT_RESET_CLEAR, "CLUTTER_PREEDIT_RESET_CLEAR", "clear" },
        { CLUTTER_PREEDIT_RESET_COMMIT, "CLUTTER_PREEDIT_RESET_COMMIT", "commit" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterPreeditResetMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_preedit_style_hint_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_PREEDIT_STYLE_NONE, "CLUTTER_PREEDIT_STYLE_NONE", "none" },
        { CLUTTER_PREEDIT_STYLE_WHOLE, "CLUTTER_PREEDIT_STYLE_WHOLE", "whole" },
        { CLUTTER_PREEDIT_STYLE_SELECTION, "CLUTTER_PREEDIT_STYLE_SELECTION", "selection" },
        { CLUTTER_PREEDIT_STYLE_PREDICTION, "CLUTTER_PREEDIT_STYLE_PREDICTION", "prediction" },
        { CLUTTER_PREEDIT_STYLE_PREFIX, "CLUTTER_PREEDIT_STYLE_PREFIX", "prefix" },
        { CLUTTER_PREEDIT_STYLE_SUFFIX, "CLUTTER_PREEDIT_STYLE_SUFFIX", "suffix" },
        { CLUTTER_PREEDIT_STYLE_SPELLING_ERROR, "CLUTTER_PREEDIT_STYLE_SPELLING_ERROR", "spelling-error" },
        { CLUTTER_PREEDIT_STYLE_COMPOSE_ERROR, "CLUTTER_PREEDIT_STYLE_COMPOSE_ERROR", "compose-error" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterPreeditStyleHint"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_event_phase_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_PHASE_CAPTURE, "CLUTTER_PHASE_CAPTURE", "capture" },
        { CLUTTER_PHASE_TARGET, "CLUTTER_PHASE_TARGET", "target" },
        { CLUTTER_PHASE_BUBBLE, "CLUTTER_PHASE_BUBBLE", "bubble" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterEventPhase"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_gesture_state_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_GESTURE_STATE_WAITING, "CLUTTER_GESTURE_STATE_WAITING", "waiting" },
        { CLUTTER_GESTURE_STATE_POSSIBLE, "CLUTTER_GESTURE_STATE_POSSIBLE", "possible" },
        { CLUTTER_GESTURE_STATE_RECOGNIZING, "CLUTTER_GESTURE_STATE_RECOGNIZING", "recognizing" },
        { CLUTTER_GESTURE_STATE_COMPLETED, "CLUTTER_GESTURE_STATE_COMPLETED", "completed" },
        { CLUTTER_GESTURE_STATE_CANCELLED, "CLUTTER_GESTURE_STATE_CANCELLED", "cancelled" },
        { CLUTTER_N_GESTURE_STATES, "CLUTTER_N_GESTURE_STATES", "clutter-n-gesture-states" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterGestureState"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_sprite_role_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_SPRITE_ROLE_POINTER, "CLUTTER_SPRITE_ROLE_POINTER", "pointer" },
        { CLUTTER_SPRITE_ROLE_TABLET, "CLUTTER_SPRITE_ROLE_TABLET", "tablet" },
        { CLUTTER_SPRITE_ROLE_TOUCHPOINT, "CLUTTER_SPRITE_ROLE_TOUCHPOINT", "touchpoint" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterSpriteRole"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_cursor_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_CURSOR_INHERIT, "CLUTTER_CURSOR_INHERIT", "inherit" },
        { CLUTTER_CURSOR_NONE, "CLUTTER_CURSOR_NONE", "none" },
        { CLUTTER_CURSOR_DEFAULT, "CLUTTER_CURSOR_DEFAULT", "default" },
        { CLUTTER_CURSOR_CONTEXT_MENU, "CLUTTER_CURSOR_CONTEXT_MENU", "context-menu" },
        { CLUTTER_CURSOR_HELP, "CLUTTER_CURSOR_HELP", "help" },
        { CLUTTER_CURSOR_POINTER, "CLUTTER_CURSOR_POINTER", "pointer" },
        { CLUTTER_CURSOR_PROGRESS, "CLUTTER_CURSOR_PROGRESS", "progress" },
        { CLUTTER_CURSOR_WAIT, "CLUTTER_CURSOR_WAIT", "wait" },
        { CLUTTER_CURSOR_CELL, "CLUTTER_CURSOR_CELL", "cell" },
        { CLUTTER_CURSOR_CROSSHAIR, "CLUTTER_CURSOR_CROSSHAIR", "crosshair" },
        { CLUTTER_CURSOR_TEXT, "CLUTTER_CURSOR_TEXT", "text" },
        { CLUTTER_CURSOR_VERTICAL_TEXT, "CLUTTER_CURSOR_VERTICAL_TEXT", "vertical-text" },
        { CLUTTER_CURSOR_ALIAS, "CLUTTER_CURSOR_ALIAS", "alias" },
        { CLUTTER_CURSOR_COPY, "CLUTTER_CURSOR_COPY", "copy" },
        { CLUTTER_CURSOR_MOVE, "CLUTTER_CURSOR_MOVE", "move" },
        { CLUTTER_CURSOR_NO_DROP, "CLUTTER_CURSOR_NO_DROP", "no-drop" },
        { CLUTTER_CURSOR_NOT_ALLOWED, "CLUTTER_CURSOR_NOT_ALLOWED", "not-allowed" },
        { CLUTTER_CURSOR_GRAB, "CLUTTER_CURSOR_GRAB", "grab" },
        { CLUTTER_CURSOR_GRABBING, "CLUTTER_CURSOR_GRABBING", "grabbing" },
        { CLUTTER_CURSOR_E_RESIZE, "CLUTTER_CURSOR_E_RESIZE", "e-resize" },
        { CLUTTER_CURSOR_N_RESIZE, "CLUTTER_CURSOR_N_RESIZE", "n-resize" },
        { CLUTTER_CURSOR_NE_RESIZE, "CLUTTER_CURSOR_NE_RESIZE", "ne-resize" },
        { CLUTTER_CURSOR_NW_RESIZE, "CLUTTER_CURSOR_NW_RESIZE", "nw-resize" },
        { CLUTTER_CURSOR_S_RESIZE, "CLUTTER_CURSOR_S_RESIZE", "s-resize" },
        { CLUTTER_CURSOR_SE_RESIZE, "CLUTTER_CURSOR_SE_RESIZE", "se-resize" },
        { CLUTTER_CURSOR_SW_RESIZE, "CLUTTER_CURSOR_SW_RESIZE", "sw-resize" },
        { CLUTTER_CURSOR_W_RESIZE, "CLUTTER_CURSOR_W_RESIZE", "w-resize" },
        { CLUTTER_CURSOR_EW_RESIZE, "CLUTTER_CURSOR_EW_RESIZE", "ew-resize" },
        { CLUTTER_CURSOR_NS_RESIZE, "CLUTTER_CURSOR_NS_RESIZE", "ns-resize" },
        { CLUTTER_CURSOR_NESW_RESIZE, "CLUTTER_CURSOR_NESW_RESIZE", "nesw-resize" },
        { CLUTTER_CURSOR_NWSE_RESIZE, "CLUTTER_CURSOR_NWSE_RESIZE", "nwse-resize" },
        { CLUTTER_CURSOR_COL_RESIZE, "CLUTTER_CURSOR_COL_RESIZE", "col-resize" },
        { CLUTTER_CURSOR_ROW_RESIZE, "CLUTTER_CURSOR_ROW_RESIZE", "row-resize" },
        { CLUTTER_CURSOR_ALL_SCROLL, "CLUTTER_CURSOR_ALL_SCROLL", "all-scroll" },
        { CLUTTER_CURSOR_ZOOM_IN, "CLUTTER_CURSOR_ZOOM_IN", "zoom-in" },
        { CLUTTER_CURSOR_ZOOM_OUT, "CLUTTER_CURSOR_ZOOM_OUT", "zoom-out" },
        { CLUTTER_CURSOR_DND_ASK, "CLUTTER_CURSOR_DND_ASK", "dnd-ask" },
        { CLUTTER_CURSOR_ALL_RESIZE, "CLUTTER_CURSOR_ALL_RESIZE", "all-resize" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterCursorType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_scroll_controller_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_SCROLL_CONTROLLER_FLAG_NONE, "CLUTTER_SCROLL_CONTROLLER_FLAG_NONE", "none" },
        { CLUTTER_SCROLL_CONTROLLER_FLAG_SCROLL_VERTICAL, "CLUTTER_SCROLL_CONTROLLER_FLAG_SCROLL_VERTICAL", "scroll-vertical" },
        { CLUTTER_SCROLL_CONTROLLER_FLAG_SCROLL_HORIZONTAL, "CLUTTER_SCROLL_CONTROLLER_FLAG_SCROLL_HORIZONTAL", "scroll-horizontal" },
        { CLUTTER_SCROLL_CONTROLLER_FLAG_DISCRETE, "CLUTTER_SCROLL_CONTROLLER_FLAG_DISCRETE", "discrete" },
        { CLUTTER_SCROLL_CONTROLLER_FLAG_PHYSICAL_DIRECTION, "CLUTTER_SCROLL_CONTROLLER_FLAG_PHYSICAL_DIRECTION", "physical-direction" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterScrollControllerFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/clutter/clutter/clutter-frame-clock.h" */
#include "/build/mutter/src/build/../mutter/clutter/clutter/clutter-frame-clock.h"
GType
clutter_frame_result_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_FRAME_RESULT_PENDING_PRESENTED, "CLUTTER_FRAME_RESULT_PENDING_PRESENTED", "pending-presented" },
        { CLUTTER_FRAME_RESULT_IDLE, "CLUTTER_FRAME_RESULT_IDLE", "idle" },
        { CLUTTER_FRAME_RESULT_IGNORED, "CLUTTER_FRAME_RESULT_IGNORED", "ignored" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterFrameResult"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_frame_clock_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_FRAME_CLOCK_MODE_FIXED, "CLUTTER_FRAME_CLOCK_MODE_FIXED", "fixed" },
        { CLUTTER_FRAME_CLOCK_MODE_VARIABLE, "CLUTTER_FRAME_CLOCK_MODE_VARIABLE", "variable" },
        { CLUTTER_FRAME_CLOCK_MODE_PASSIVE, "CLUTTER_FRAME_CLOCK_MODE_PASSIVE", "passive" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterFrameClockMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/clutter/clutter/clutter-main.h" */
#include "/build/mutter/src/build/../mutter/clutter/clutter/clutter-main.h"
GType
clutter_debug_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_DEBUG_MISC, "CLUTTER_DEBUG_MISC", "misc" },
        { CLUTTER_DEBUG_ACTOR, "CLUTTER_DEBUG_ACTOR", "actor" },
        { CLUTTER_DEBUG_TEXTURE, "CLUTTER_DEBUG_TEXTURE", "texture" },
        { CLUTTER_DEBUG_EVENT, "CLUTTER_DEBUG_EVENT", "event" },
        { CLUTTER_DEBUG_PAINT, "CLUTTER_DEBUG_PAINT", "paint" },
        { CLUTTER_DEBUG_PANGO, "CLUTTER_DEBUG_PANGO", "pango" },
        { CLUTTER_DEBUG_BACKEND, "CLUTTER_DEBUG_BACKEND", "backend" },
        { CLUTTER_DEBUG_SCHEDULER, "CLUTTER_DEBUG_SCHEDULER", "scheduler" },
        { CLUTTER_DEBUG_SCRIPT, "CLUTTER_DEBUG_SCRIPT", "script" },
        { CLUTTER_DEBUG_SHADER, "CLUTTER_DEBUG_SHADER", "shader" },
        { CLUTTER_DEBUG_MULTISTAGE, "CLUTTER_DEBUG_MULTISTAGE", "multistage" },
        { CLUTTER_DEBUG_ANIMATION, "CLUTTER_DEBUG_ANIMATION", "animation" },
        { CLUTTER_DEBUG_LAYOUT, "CLUTTER_DEBUG_LAYOUT", "layout" },
        { CLUTTER_DEBUG_PICK, "CLUTTER_DEBUG_PICK", "pick" },
        { CLUTTER_DEBUG_EVENTLOOP, "CLUTTER_DEBUG_EVENTLOOP", "eventloop" },
        { CLUTTER_DEBUG_CLIPPING, "CLUTTER_DEBUG_CLIPPING", "clipping" },
        { CLUTTER_DEBUG_OOB_TRANSFORMS, "CLUTTER_DEBUG_OOB_TRANSFORMS", "oob-transforms" },
        { CLUTTER_DEBUG_FRAME_TIMINGS, "CLUTTER_DEBUG_FRAME_TIMINGS", "frame-timings" },
        { CLUTTER_DEBUG_DETAILED_TRACE, "CLUTTER_DEBUG_DETAILED_TRACE", "detailed-trace" },
        { CLUTTER_DEBUG_GRABS, "CLUTTER_DEBUG_GRABS", "grabs" },
        { CLUTTER_DEBUG_FRAME_CLOCK, "CLUTTER_DEBUG_FRAME_CLOCK", "frame-clock" },
        { CLUTTER_DEBUG_GESTURES, "CLUTTER_DEBUG_GESTURES", "gestures" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterDebugFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_pick_debug_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_DEBUG_NOP_PICKING, "CLUTTER_DEBUG_NOP_PICKING", "picking" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterPickDebugFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_draw_debug_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_DEBUG_DISABLE_SWAP_EVENTS, "CLUTTER_DEBUG_DISABLE_SWAP_EVENTS", "disable-swap-events" },
        { CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS, "CLUTTER_DEBUG_DISABLE_CLIPPED_REDRAWS", "disable-clipped-redraws" },
        { CLUTTER_DEBUG_REDRAWS, "CLUTTER_DEBUG_REDRAWS", "redraws" },
        { CLUTTER_DEBUG_PAINT_VOLUMES, "CLUTTER_DEBUG_PAINT_VOLUMES", "paint-volumes" },
        { CLUTTER_DEBUG_DISABLE_CULLING, "CLUTTER_DEBUG_DISABLE_CULLING", "disable-culling" },
        { CLUTTER_DEBUG_DISABLE_OFFSCREEN_REDIRECT, "CLUTTER_DEBUG_DISABLE_OFFSCREEN_REDIRECT", "disable-offscreen-redirect" },
        { CLUTTER_DEBUG_CONTINUOUS_REDRAW, "CLUTTER_DEBUG_CONTINUOUS_REDRAW", "continuous-redraw" },
        { CLUTTER_DEBUG_PAINT_DEFORM_TILES, "CLUTTER_DEBUG_PAINT_DEFORM_TILES", "paint-deform-tiles" },
        { CLUTTER_DEBUG_PAINT_DAMAGE_REGION, "CLUTTER_DEBUG_PAINT_DAMAGE_REGION", "paint-damage-region" },
        { CLUTTER_DEBUG_DISABLE_DYNAMIC_MAX_RENDER_TIME, "CLUTTER_DEBUG_DISABLE_DYNAMIC_MAX_RENDER_TIME", "disable-dynamic-max-render-time" },
        { CLUTTER_DEBUG_PAINT_MAX_RENDER_TIME, "CLUTTER_DEBUG_PAINT_MAX_RENDER_TIME", "paint-max-render-time" },
        { CLUTTER_DEBUG_DISABLE_TRIPLE_BUFFERING, "CLUTTER_DEBUG_DISABLE_TRIPLE_BUFFERING", "disable-triple-buffering" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterDrawDebugFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_pipeline_capability_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_PIPELINE_CAPABILITY_COLOR_STATE, "CLUTTER_PIPELINE_CAPABILITY_COLOR_STATE", "state" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterPipelineCapability"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/clutter/clutter/clutter-paint-context.h" */
#include "/build/mutter/src/build/../mutter/clutter/clutter/clutter-paint-context.h"
GType
clutter_paint_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_PAINT_FLAG_NONE, "CLUTTER_PAINT_FLAG_NONE", "none" },
        { CLUTTER_PAINT_FLAG_NO_CURSORS, "CLUTTER_PAINT_FLAG_NO_CURSORS", "no-cursors" },
        { CLUTTER_PAINT_FLAG_FORCE_CURSORS, "CLUTTER_PAINT_FLAG_FORCE_CURSORS", "force-cursors" },
        { CLUTTER_PAINT_FLAG_CLEAR, "CLUTTER_PAINT_FLAG_CLEAR", "clear" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterPaintFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/clutter/clutter/clutter-seat.h" */
#include "/build/mutter/src/build/../mutter/clutter/clutter/clutter-seat.h"
GType
clutter_virtual_device_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_VIRTUAL_DEVICE_TYPE_NONE, "CLUTTER_VIRTUAL_DEVICE_TYPE_NONE", "none" },
        { CLUTTER_VIRTUAL_DEVICE_TYPE_KEYBOARD, "CLUTTER_VIRTUAL_DEVICE_TYPE_KEYBOARD", "keyboard" },
        { CLUTTER_VIRTUAL_DEVICE_TYPE_POINTER, "CLUTTER_VIRTUAL_DEVICE_TYPE_POINTER", "pointer" },
        { CLUTTER_VIRTUAL_DEVICE_TYPE_TOUCHSCREEN, "CLUTTER_VIRTUAL_DEVICE_TYPE_TOUCHSCREEN", "touchscreen" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterVirtualDeviceType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/clutter/clutter/clutter-stage.h" */
#include "/build/mutter/src/build/../mutter/clutter/clutter/clutter-stage.h"
GType
clutter_frame_info_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { CLUTTER_FRAME_INFO_FLAG_NONE, "CLUTTER_FRAME_INFO_FLAG_NONE", "none" },
        { CLUTTER_FRAME_INFO_FLAG_HW_CLOCK, "CLUTTER_FRAME_INFO_FLAG_HW_CLOCK", "hw-clock" },
        { CLUTTER_FRAME_INFO_FLAG_ZERO_COPY, "CLUTTER_FRAME_INFO_FLAG_ZERO_COPY", "zero-copy" },
        { CLUTTER_FRAME_INFO_FLAG_VSYNC, "CLUTTER_FRAME_INFO_FLAG_VSYNC", "vsync" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("ClutterFrameInfoFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/clutter/clutter/clutter-virtual-input-device.h" */
#include "/build/mutter/src/build/../mutter/clutter/clutter/clutter-virtual-input-device.h"
GType
clutter_button_state_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_BUTTON_STATE_RELEASED, "CLUTTER_BUTTON_STATE_RELEASED", "released" },
        { CLUTTER_BUTTON_STATE_PRESSED, "CLUTTER_BUTTON_STATE_PRESSED", "pressed" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterButtonState"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
clutter_key_state_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { CLUTTER_KEY_STATE_RELEASED, "CLUTTER_KEY_STATE_RELEASED", "released" },
        { CLUTTER_KEY_STATE_PRESSED, "CLUTTER_KEY_STATE_PRESSED", "pressed" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("ClutterKeyState"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* Generated data ends here */

