
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "meta/meta-enum-types.h"

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/barrier.h" */
#include "/build/mutter/src/build/../mutter/src/meta/barrier.h"
GType
meta_barrier_direction_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_BARRIER_DIRECTION_POSITIVE_X, "META_BARRIER_DIRECTION_POSITIVE_X", "positive-x" },
        { META_BARRIER_DIRECTION_POSITIVE_Y, "META_BARRIER_DIRECTION_POSITIVE_Y", "positive-y" },
        { META_BARRIER_DIRECTION_NEGATIVE_X, "META_BARRIER_DIRECTION_NEGATIVE_X", "negative-x" },
        { META_BARRIER_DIRECTION_NEGATIVE_Y, "META_BARRIER_DIRECTION_NEGATIVE_Y", "negative-y" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaBarrierDirection"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_barrier_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_BARRIER_FLAG_NONE, "META_BARRIER_FLAG_NONE", "none" },
        { META_BARRIER_FLAG_STICKY, "META_BARRIER_FLAG_STICKY", "sticky" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaBarrierFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/boxes.h" */
#include "/build/mutter/src/build/../mutter/src/meta/boxes.h"
GType
meta_edge_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_EDGE_WINDOW, "META_EDGE_WINDOW", "window" },
        { META_EDGE_MONITOR, "META_EDGE_MONITOR", "monitor" },
        { META_EDGE_SCREEN, "META_EDGE_SCREEN", "screen" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaEdgeType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/compositor.h" */
#include "/build/mutter/src/build/../mutter/src/meta/compositor.h"
GType
meta_comp_effect_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_COMP_EFFECT_CREATE, "META_COMP_EFFECT_CREATE", "create" },
        { META_COMP_EFFECT_UNMINIMIZE, "META_COMP_EFFECT_UNMINIMIZE", "unminimize" },
        { META_COMP_EFFECT_DESTROY, "META_COMP_EFFECT_DESTROY", "destroy" },
        { META_COMP_EFFECT_MINIMIZE, "META_COMP_EFFECT_MINIMIZE", "minimize" },
        { META_COMP_EFFECT_NONE, "META_COMP_EFFECT_NONE", "none" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaCompEffect"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_size_change_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_SIZE_CHANGE_MAXIMIZE, "META_SIZE_CHANGE_MAXIMIZE", "maximize" },
        { META_SIZE_CHANGE_UNMAXIMIZE, "META_SIZE_CHANGE_UNMAXIMIZE", "unmaximize" },
        { META_SIZE_CHANGE_FULLSCREEN, "META_SIZE_CHANGE_FULLSCREEN", "fullscreen" },
        { META_SIZE_CHANGE_UNFULLSCREEN, "META_SIZE_CHANGE_UNFULLSCREEN", "unfullscreen" },
        { META_SIZE_CHANGE_MONITOR_MOVE, "META_SIZE_CHANGE_MONITOR_MOVE", "monitor-move" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaSizeChange"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/display.h" */
#include "/build/mutter/src/build/../mutter/src/meta/display.h"
GType
meta_tab_list_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_TAB_LIST_NORMAL, "META_TAB_LIST_NORMAL", "normal" },
        { META_TAB_LIST_DOCKS, "META_TAB_LIST_DOCKS", "docks" },
        { META_TAB_LIST_GROUP, "META_TAB_LIST_GROUP", "group" },
        { META_TAB_LIST_NORMAL_ALL, "META_TAB_LIST_NORMAL_ALL", "normal-all" },
        { META_TAB_LIST_NORMAL_ALL_MRU, "META_TAB_LIST_NORMAL_ALL_MRU", "normal-all-mru" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaTabList"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_tab_show_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_TAB_SHOW_ICON, "META_TAB_SHOW_ICON", "icon" },
        { META_TAB_SHOW_INSTANTLY, "META_TAB_SHOW_INSTANTLY", "instantly" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaTabShowType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_pad_feature_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_PAD_FEATURE_RING, "META_PAD_FEATURE_RING", "ring" },
        { META_PAD_FEATURE_STRIP, "META_PAD_FEATURE_STRIP", "strip" },
        { META_PAD_FEATURE_DIAL, "META_PAD_FEATURE_DIAL", "dial" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaPadFeatureType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_pad_direction_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_PAD_DIRECTION_UP, "META_PAD_DIRECTION_UP", "up" },
        { META_PAD_DIRECTION_DOWN, "META_PAD_DIRECTION_DOWN", "down" },
        { META_PAD_DIRECTION_CW, "META_PAD_DIRECTION_CW", "cw" },
        { META_PAD_DIRECTION_CCW, "META_PAD_DIRECTION_CCW", "ccw" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaPadDirection"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_display_direction_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_DISPLAY_UP, "META_DISPLAY_UP", "up" },
        { META_DISPLAY_DOWN, "META_DISPLAY_DOWN", "down" },
        { META_DISPLAY_LEFT, "META_DISPLAY_LEFT", "left" },
        { META_DISPLAY_RIGHT, "META_DISPLAY_RIGHT", "right" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaDisplayDirection"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_display_corner_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_DISPLAY_TOPLEFT, "META_DISPLAY_TOPLEFT", "topleft" },
        { META_DISPLAY_TOPRIGHT, "META_DISPLAY_TOPRIGHT", "topright" },
        { META_DISPLAY_BOTTOMLEFT, "META_DISPLAY_BOTTOMLEFT", "bottomleft" },
        { META_DISPLAY_BOTTOMRIGHT, "META_DISPLAY_BOTTOMRIGHT", "bottomright" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaDisplayCorner"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/main.h" */
#include "/build/mutter/src/build/../mutter/src/meta/main.h"
GType
meta_exit_code_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_EXIT_SUCCESS, "META_EXIT_SUCCESS", "success" },
        { META_EXIT_ERROR, "META_EXIT_ERROR", "error" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaExitCode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-backend.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-backend.h"
GType
meta_backend_capabilities_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_BACKEND_CAPABILITY_NONE, "META_BACKEND_CAPABILITY_NONE", "none" },
        { META_BACKEND_CAPABILITY_BARRIERS, "META_BACKEND_CAPABILITY_BARRIERS", "barriers" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaBackendCapabilities"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-close-dialog.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-close-dialog.h"
GType
meta_close_dialog_response_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_CLOSE_DIALOG_RESPONSE_WAIT, "META_CLOSE_DIALOG_RESPONSE_WAIT", "wait" },
        { META_CLOSE_DIALOG_RESPONSE_FORCE_CLOSE, "META_CLOSE_DIALOG_RESPONSE_FORCE_CLOSE", "force-close" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaCloseDialogResponse"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-debug.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-debug.h"
GType
meta_debug_topic_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_DEBUG_VERBOSE, "META_DEBUG_VERBOSE", "verbose" },
        { META_DEBUG_FOCUS, "META_DEBUG_FOCUS", "focus" },
        { META_DEBUG_WORKAREA, "META_DEBUG_WORKAREA", "workarea" },
        { META_DEBUG_STACK, "META_DEBUG_STACK", "stack" },
        { META_DEBUG_SM, "META_DEBUG_SM", "sm" },
        { META_DEBUG_EVENTS, "META_DEBUG_EVENTS", "events" },
        { META_DEBUG_WINDOW_STATE, "META_DEBUG_WINDOW_STATE", "window-state" },
        { META_DEBUG_WINDOW_OPS, "META_DEBUG_WINDOW_OPS", "window-ops" },
        { META_DEBUG_GEOMETRY, "META_DEBUG_GEOMETRY", "geometry" },
        { META_DEBUG_PLACEMENT, "META_DEBUG_PLACEMENT", "placement" },
        { META_DEBUG_DISPLAY, "META_DEBUG_DISPLAY", "display" },
        { META_DEBUG_KEYBINDINGS, "META_DEBUG_KEYBINDINGS", "keybindings" },
        { META_DEBUG_SYNC, "META_DEBUG_SYNC", "sync" },
        { META_DEBUG_STARTUP, "META_DEBUG_STARTUP", "startup" },
        { META_DEBUG_PREFS, "META_DEBUG_PREFS", "prefs" },
        { META_DEBUG_EDGE_RESISTANCE, "META_DEBUG_EDGE_RESISTANCE", "edge-resistance" },
        { META_DEBUG_DBUS, "META_DEBUG_DBUS", "dbus" },
        { META_DEBUG_INPUT, "META_DEBUG_INPUT", "input" },
        { META_DEBUG_WAYLAND, "META_DEBUG_WAYLAND", "wayland" },
        { META_DEBUG_KMS, "META_DEBUG_KMS", "kms" },
        { META_DEBUG_SCREEN_CAST, "META_DEBUG_SCREEN_CAST", "screen-cast" },
        { META_DEBUG_REMOTE_DESKTOP, "META_DEBUG_REMOTE_DESKTOP", "remote-desktop" },
        { META_DEBUG_BACKEND, "META_DEBUG_BACKEND", "backend" },
        { META_DEBUG_RENDER, "META_DEBUG_RENDER", "render" },
        { META_DEBUG_COLOR, "META_DEBUG_COLOR", "color" },
        { META_DEBUG_INPUT_EVENTS, "META_DEBUG_INPUT_EVENTS", "input-events" },
        { META_DEBUG_EIS, "META_DEBUG_EIS", "eis" },
        { META_DEBUG_KMS_DEADLINE, "META_DEBUG_KMS_DEADLINE", "kms-deadline" },
        { META_DEBUG_SESSION_MANAGEMENT, "META_DEBUG_SESSION_MANAGEMENT", "session-management" },
        { META_DEBUG_X11, "META_DEBUG_X11", "x11" },
        { META_DEBUG_WORKSPACES, "META_DEBUG_WORKSPACES", "workspaces" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaDebugTopic"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-enums.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-enums.h"
GType
meta_grab_op_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_GRAB_OP_NONE, "META_GRAB_OP_NONE", "none" },
        { META_GRAB_OP_WINDOW_BASE, "META_GRAB_OP_WINDOW_BASE", "window-base" },
        { META_GRAB_OP_MOVING, "META_GRAB_OP_MOVING", "moving" },
        { META_GRAB_OP_MOVING_UNCONSTRAINED, "META_GRAB_OP_MOVING_UNCONSTRAINED", "moving-unconstrained" },
        { META_GRAB_OP_RESIZING_NW, "META_GRAB_OP_RESIZING_NW", "resizing-nw" },
        { META_GRAB_OP_RESIZING_N, "META_GRAB_OP_RESIZING_N", "resizing-n" },
        { META_GRAB_OP_RESIZING_NE, "META_GRAB_OP_RESIZING_NE", "resizing-ne" },
        { META_GRAB_OP_RESIZING_E, "META_GRAB_OP_RESIZING_E", "resizing-e" },
        { META_GRAB_OP_RESIZING_SW, "META_GRAB_OP_RESIZING_SW", "resizing-sw" },
        { META_GRAB_OP_RESIZING_S, "META_GRAB_OP_RESIZING_S", "resizing-s" },
        { META_GRAB_OP_RESIZING_SE, "META_GRAB_OP_RESIZING_SE", "resizing-se" },
        { META_GRAB_OP_RESIZING_W, "META_GRAB_OP_RESIZING_W", "resizing-w" },
        { META_GRAB_OP_KEYBOARD_MOVING, "META_GRAB_OP_KEYBOARD_MOVING", "keyboard-moving" },
        { META_GRAB_OP_KEYBOARD_RESIZING_UNKNOWN, "META_GRAB_OP_KEYBOARD_RESIZING_UNKNOWN", "keyboard-resizing-unknown" },
        { META_GRAB_OP_KEYBOARD_RESIZING_NW, "META_GRAB_OP_KEYBOARD_RESIZING_NW", "keyboard-resizing-nw" },
        { META_GRAB_OP_KEYBOARD_RESIZING_N, "META_GRAB_OP_KEYBOARD_RESIZING_N", "keyboard-resizing-n" },
        { META_GRAB_OP_KEYBOARD_RESIZING_NE, "META_GRAB_OP_KEYBOARD_RESIZING_NE", "keyboard-resizing-ne" },
        { META_GRAB_OP_KEYBOARD_RESIZING_E, "META_GRAB_OP_KEYBOARD_RESIZING_E", "keyboard-resizing-e" },
        { META_GRAB_OP_KEYBOARD_RESIZING_SW, "META_GRAB_OP_KEYBOARD_RESIZING_SW", "keyboard-resizing-sw" },
        { META_GRAB_OP_KEYBOARD_RESIZING_S, "META_GRAB_OP_KEYBOARD_RESIZING_S", "keyboard-resizing-s" },
        { META_GRAB_OP_KEYBOARD_RESIZING_SE, "META_GRAB_OP_KEYBOARD_RESIZING_SE", "keyboard-resizing-se" },
        { META_GRAB_OP_KEYBOARD_RESIZING_W, "META_GRAB_OP_KEYBOARD_RESIZING_W", "keyboard-resizing-w" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaGrabOp"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_frame_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_FRAME_TYPE_NORMAL, "META_FRAME_TYPE_NORMAL", "normal" },
        { META_FRAME_TYPE_DIALOG, "META_FRAME_TYPE_DIALOG", "dialog" },
        { META_FRAME_TYPE_MODAL_DIALOG, "META_FRAME_TYPE_MODAL_DIALOG", "modal-dialog" },
        { META_FRAME_TYPE_UTILITY, "META_FRAME_TYPE_UTILITY", "utility" },
        { META_FRAME_TYPE_MENU, "META_FRAME_TYPE_MENU", "menu" },
        { META_FRAME_TYPE_BORDER, "META_FRAME_TYPE_BORDER", "border" },
        { META_FRAME_TYPE_ATTACHED, "META_FRAME_TYPE_ATTACHED", "attached" },
        { META_FRAME_TYPE_LAST, "META_FRAME_TYPE_LAST", "last" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaFrameType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_direction_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_DIRECTION_LEFT, "META_DIRECTION_LEFT", "left" },
        { META_DIRECTION_RIGHT, "META_DIRECTION_RIGHT", "right" },
        { META_DIRECTION_TOP, "META_DIRECTION_TOP", "top" },
        { META_DIRECTION_BOTTOM, "META_DIRECTION_BOTTOM", "bottom" },
        { META_DIRECTION_UP, "META_DIRECTION_UP", "up" },
        { META_DIRECTION_DOWN, "META_DIRECTION_DOWN", "down" },
        { META_DIRECTION_HORIZONTAL, "META_DIRECTION_HORIZONTAL", "horizontal" },
        { META_DIRECTION_VERTICAL, "META_DIRECTION_VERTICAL", "vertical" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaDirection"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_motion_direction_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_MOTION_UP, "META_MOTION_UP", "up" },
        { META_MOTION_DOWN, "META_MOTION_DOWN", "down" },
        { META_MOTION_LEFT, "META_MOTION_LEFT", "left" },
        { META_MOTION_RIGHT, "META_MOTION_RIGHT", "right" },
        { META_MOTION_UP_LEFT, "META_MOTION_UP_LEFT", "up-left" },
        { META_MOTION_UP_RIGHT, "META_MOTION_UP_RIGHT", "up-right" },
        { META_MOTION_DOWN_LEFT, "META_MOTION_DOWN_LEFT", "down-left" },
        { META_MOTION_DOWN_RIGHT, "META_MOTION_DOWN_RIGHT", "down-right" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaMotionDirection"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_side_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_SIDE_LEFT, "META_SIDE_LEFT", "left" },
        { META_SIDE_RIGHT, "META_SIDE_RIGHT", "right" },
        { META_SIDE_TOP, "META_SIDE_TOP", "top" },
        { META_SIDE_BOTTOM, "META_SIDE_BOTTOM", "bottom" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaSide"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_button_function_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_BUTTON_FUNCTION_MENU, "META_BUTTON_FUNCTION_MENU", "menu" },
        { META_BUTTON_FUNCTION_MINIMIZE, "META_BUTTON_FUNCTION_MINIMIZE", "minimize" },
        { META_BUTTON_FUNCTION_MAXIMIZE, "META_BUTTON_FUNCTION_MAXIMIZE", "maximize" },
        { META_BUTTON_FUNCTION_CLOSE, "META_BUTTON_FUNCTION_CLOSE", "close" },
        { META_BUTTON_FUNCTION_LAST, "META_BUTTON_FUNCTION_LAST", "last" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaButtonFunction"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_window_menu_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_WINDOW_MENU_WM, "META_WINDOW_MENU_WM", "wm" },
        { META_WINDOW_MENU_APP, "META_WINDOW_MENU_APP", "app" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaWindowMenuType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_stack_layer_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_LAYER_DESKTOP, "META_LAYER_DESKTOP", "desktop" },
        { META_LAYER_BOTTOM, "META_LAYER_BOTTOM", "bottom" },
        { META_LAYER_NORMAL, "META_LAYER_NORMAL", "normal" },
        { META_LAYER_TOP, "META_LAYER_TOP", "top" },
        { META_LAYER_DOCK, "META_LAYER_DOCK", "dock" },
        { META_LAYER_OVERRIDE_REDIRECT, "META_LAYER_OVERRIDE_REDIRECT", "override-redirect" },
        { META_LAYER_LAST, "META_LAYER_LAST", "last" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaStackLayer"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_gravity_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_GRAVITY_NONE, "META_GRAVITY_NONE", "none" },
        { META_GRAVITY_NORTH_WEST, "META_GRAVITY_NORTH_WEST", "north-west" },
        { META_GRAVITY_NORTH, "META_GRAVITY_NORTH", "north" },
        { META_GRAVITY_NORTH_EAST, "META_GRAVITY_NORTH_EAST", "north-east" },
        { META_GRAVITY_WEST, "META_GRAVITY_WEST", "west" },
        { META_GRAVITY_CENTER, "META_GRAVITY_CENTER", "center" },
        { META_GRAVITY_EAST, "META_GRAVITY_EAST", "east" },
        { META_GRAVITY_SOUTH_WEST, "META_GRAVITY_SOUTH_WEST", "south-west" },
        { META_GRAVITY_SOUTH, "META_GRAVITY_SOUTH", "south" },
        { META_GRAVITY_SOUTH_EAST, "META_GRAVITY_SOUTH_EAST", "south-east" },
        { META_GRAVITY_STATIC, "META_GRAVITY_STATIC", "static" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaGravity"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_keyboard_a11y_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_A11Y_KEYBOARD_ENABLED, "META_A11Y_KEYBOARD_ENABLED", "keyboard-enabled" },
        { META_A11Y_TIMEOUT_ENABLED, "META_A11Y_TIMEOUT_ENABLED", "timeout-enabled" },
        { META_A11Y_MOUSE_KEYS_ENABLED, "META_A11Y_MOUSE_KEYS_ENABLED", "mouse-keys-enabled" },
        { META_A11Y_SLOW_KEYS_ENABLED, "META_A11Y_SLOW_KEYS_ENABLED", "slow-keys-enabled" },
        { META_A11Y_SLOW_KEYS_BEEP_PRESS, "META_A11Y_SLOW_KEYS_BEEP_PRESS", "slow-keys-beep-press" },
        { META_A11Y_SLOW_KEYS_BEEP_ACCEPT, "META_A11Y_SLOW_KEYS_BEEP_ACCEPT", "slow-keys-beep-accept" },
        { META_A11Y_SLOW_KEYS_BEEP_REJECT, "META_A11Y_SLOW_KEYS_BEEP_REJECT", "slow-keys-beep-reject" },
        { META_A11Y_BOUNCE_KEYS_ENABLED, "META_A11Y_BOUNCE_KEYS_ENABLED", "bounce-keys-enabled" },
        { META_A11Y_BOUNCE_KEYS_BEEP_REJECT, "META_A11Y_BOUNCE_KEYS_BEEP_REJECT", "bounce-keys-beep-reject" },
        { META_A11Y_TOGGLE_KEYS_ENABLED, "META_A11Y_TOGGLE_KEYS_ENABLED", "toggle-keys-enabled" },
        { META_A11Y_STICKY_KEYS_ENABLED, "META_A11Y_STICKY_KEYS_ENABLED", "sticky-keys-enabled" },
        { META_A11Y_STICKY_KEYS_TWO_KEY_OFF, "META_A11Y_STICKY_KEYS_TWO_KEY_OFF", "sticky-keys-two-key-off" },
        { META_A11Y_STICKY_KEYS_BEEP, "META_A11Y_STICKY_KEYS_BEEP", "sticky-keys-beep" },
        { META_A11Y_FEATURE_STATE_CHANGE_BEEP, "META_A11Y_FEATURE_STATE_CHANGE_BEEP", "feature-state-change-beep" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaKeyboardA11yFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_external_constraint_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_EXTERNAL_CONSTRAINT_FLAGS_NONE, "META_EXTERNAL_CONSTRAINT_FLAGS_NONE", "none" },
        { META_EXTERNAL_CONSTRAINT_FLAGS_MOVE, "META_EXTERNAL_CONSTRAINT_FLAGS_MOVE", "move" },
        { META_EXTERNAL_CONSTRAINT_FLAGS_RESIZE, "META_EXTERNAL_CONSTRAINT_FLAGS_RESIZE", "resize" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaExternalConstraintFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-idle-monitor.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-idle-monitor.h"
GType
meta_idle_monitor_watch_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_IDLE_MONITOR_WATCH_FLAGS_NONE, "META_IDLE_MONITOR_WATCH_FLAGS_NONE", "none" },
        { META_IDLE_MONITOR_WATCH_FLAGS_UNINHIBITABLE, "META_IDLE_MONITOR_WATCH_FLAGS_UNINHIBITABLE", "uninhibitable" },
        { META_IDLE_MONITOR_WATCH_FLAGS_START_NOW, "META_IDLE_MONITOR_WATCH_FLAGS_START_NOW", "start-now" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaIdleMonitorWatchFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-inhibit-shortcuts-dialog.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-inhibit-shortcuts-dialog.h"
GType
meta_inhibit_shortcuts_dialog_response_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_INHIBIT_SHORTCUTS_DIALOG_RESPONSE_ALLOW, "META_INHIBIT_SHORTCUTS_DIALOG_RESPONSE_ALLOW", "allow" },
        { META_INHIBIT_SHORTCUTS_DIALOG_RESPONSE_DENY, "META_INHIBIT_SHORTCUTS_DIALOG_RESPONSE_DENY", "deny" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaInhibitShortcutsDialogResponse"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-later.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-later.h"
GType
meta_later_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_LATER_RESIZE, "META_LATER_RESIZE", "resize" },
        { META_LATER_CALC_SHOWING, "META_LATER_CALC_SHOWING", "calc-showing" },
        { META_LATER_CHECK_FULLSCREEN, "META_LATER_CHECK_FULLSCREEN", "check-fullscreen" },
        { META_LATER_SYNC_STACK, "META_LATER_SYNC_STACK", "sync-stack" },
        { META_LATER_BEFORE_REDRAW, "META_LATER_BEFORE_REDRAW", "before-redraw" },
        { META_LATER_IDLE, "META_LATER_IDLE", "idle" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaLaterType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-monitor-manager.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-monitor-manager.h"
GType
meta_monitor_switch_config_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_MONITOR_SWITCH_CONFIG_ALL_MIRROR, "META_MONITOR_SWITCH_CONFIG_ALL_MIRROR", "all-mirror" },
        { META_MONITOR_SWITCH_CONFIG_ALL_LINEAR, "META_MONITOR_SWITCH_CONFIG_ALL_LINEAR", "all-linear" },
        { META_MONITOR_SWITCH_CONFIG_EXTERNAL, "META_MONITOR_SWITCH_CONFIG_EXTERNAL", "external" },
        { META_MONITOR_SWITCH_CONFIG_BUILTIN, "META_MONITOR_SWITCH_CONFIG_BUILTIN", "builtin" },
        { META_MONITOR_SWITCH_CONFIG_UNKNOWN, "META_MONITOR_SWITCH_CONFIG_UNKNOWN", "unknown" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaMonitorSwitchConfigType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_power_save_change_reason_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_POWER_SAVE_CHANGE_REASON_MODE_CHANGE, "META_POWER_SAVE_CHANGE_REASON_MODE_CHANGE", "mode-change" },
        { META_POWER_SAVE_CHANGE_REASON_HOTPLUG, "META_POWER_SAVE_CHANGE_REASON_HOTPLUG", "hotplug" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaPowerSaveChangeReason"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-multi-texture-format.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-multi-texture-format.h"
GType
meta_multi_texture_format_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_MULTI_TEXTURE_FORMAT_INVALID, "META_MULTI_TEXTURE_FORMAT_INVALID", "meta-multi-texture-format-invalid" },
        { META_MULTI_TEXTURE_FORMAT_SIMPLE, "META_MULTI_TEXTURE_FORMAT_SIMPLE", "meta-multi-texture-format-simple" },
        { META_MULTI_TEXTURE_FORMAT_YUYV, "META_MULTI_TEXTURE_FORMAT_YUYV", "meta-multi-texture-format-yuyv" },
        { META_MULTI_TEXTURE_FORMAT_YVYU, "META_MULTI_TEXTURE_FORMAT_YVYU", "meta-multi-texture-format-yvyu" },
        { META_MULTI_TEXTURE_FORMAT_UYVY, "META_MULTI_TEXTURE_FORMAT_UYVY", "meta-multi-texture-format-uyvy" },
        { META_MULTI_TEXTURE_FORMAT_VYUY, "META_MULTI_TEXTURE_FORMAT_VYUY", "meta-multi-texture-format-vyuy" },
        { META_MULTI_TEXTURE_FORMAT_NV12, "META_MULTI_TEXTURE_FORMAT_NV12", "meta-multi-texture-format-nv12" },
        { META_MULTI_TEXTURE_FORMAT_NV21, "META_MULTI_TEXTURE_FORMAT_NV21", "meta-multi-texture-format-nv21" },
        { META_MULTI_TEXTURE_FORMAT_NV16, "META_MULTI_TEXTURE_FORMAT_NV16", "meta-multi-texture-format-nv16" },
        { META_MULTI_TEXTURE_FORMAT_NV61, "META_MULTI_TEXTURE_FORMAT_NV61", "meta-multi-texture-format-nv61" },
        { META_MULTI_TEXTURE_FORMAT_NV24, "META_MULTI_TEXTURE_FORMAT_NV24", "meta-multi-texture-format-nv24" },
        { META_MULTI_TEXTURE_FORMAT_NV42, "META_MULTI_TEXTURE_FORMAT_NV42", "meta-multi-texture-format-nv42" },
        { META_MULTI_TEXTURE_FORMAT_P010, "META_MULTI_TEXTURE_FORMAT_P010", "meta-multi-texture-format-p010" },
        { META_MULTI_TEXTURE_FORMAT_P012, "META_MULTI_TEXTURE_FORMAT_P012", "meta-multi-texture-format-p012" },
        { META_MULTI_TEXTURE_FORMAT_P016, "META_MULTI_TEXTURE_FORMAT_P016", "meta-multi-texture-format-p016" },
        { META_MULTI_TEXTURE_FORMAT_YUV420, "META_MULTI_TEXTURE_FORMAT_YUV420", "meta-multi-texture-format-yuv420" },
        { META_MULTI_TEXTURE_FORMAT_YVU420, "META_MULTI_TEXTURE_FORMAT_YVU420", "meta-multi-texture-format-yvu420" },
        { META_MULTI_TEXTURE_FORMAT_YUV422, "META_MULTI_TEXTURE_FORMAT_YUV422", "meta-multi-texture-format-yuv422" },
        { META_MULTI_TEXTURE_FORMAT_YVU422, "META_MULTI_TEXTURE_FORMAT_YVU422", "meta-multi-texture-format-yvu422" },
        { META_MULTI_TEXTURE_FORMAT_YUV444, "META_MULTI_TEXTURE_FORMAT_YUV444", "meta-multi-texture-format-yuv444" },
        { META_MULTI_TEXTURE_FORMAT_YVU444, "META_MULTI_TEXTURE_FORMAT_YVU444", "meta-multi-texture-format-yvu444" },
        { META_MULTI_TEXTURE_FORMAT_S010, "META_MULTI_TEXTURE_FORMAT_S010", "meta-multi-texture-format-s010" },
        { META_MULTI_TEXTURE_FORMAT_S210, "META_MULTI_TEXTURE_FORMAT_S210", "meta-multi-texture-format-s210" },
        { META_MULTI_TEXTURE_FORMAT_S410, "META_MULTI_TEXTURE_FORMAT_S410", "meta-multi-texture-format-s410" },
        { META_MULTI_TEXTURE_FORMAT_S012, "META_MULTI_TEXTURE_FORMAT_S012", "meta-multi-texture-format-s012" },
        { META_MULTI_TEXTURE_FORMAT_S212, "META_MULTI_TEXTURE_FORMAT_S212", "meta-multi-texture-format-s212" },
        { META_MULTI_TEXTURE_FORMAT_S412, "META_MULTI_TEXTURE_FORMAT_S412", "meta-multi-texture-format-s412" },
        { META_MULTI_TEXTURE_FORMAT_S016, "META_MULTI_TEXTURE_FORMAT_S016", "meta-multi-texture-format-s016" },
        { META_MULTI_TEXTURE_FORMAT_S216, "META_MULTI_TEXTURE_FORMAT_S216", "meta-multi-texture-format-s216" },
        { META_MULTI_TEXTURE_FORMAT_S416, "META_MULTI_TEXTURE_FORMAT_S416", "meta-multi-texture-format-s416" },
        { N_META_MULTI_TEXTURE_FORMATS, "N_META_MULTI_TEXTURE_FORMATS", "n-meta-multi-texture-formats" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaMultiTextureFormat"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_multi_texture_alpha_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_MULTI_TEXTURE_ALPHA_MODE_NONE, "META_MULTI_TEXTURE_ALPHA_MODE_NONE", "meta-multi-texture-alpha-mode-none" },
        { META_MULTI_TEXTURE_ALPHA_MODE_PREMULT_ELECTRICAL, "META_MULTI_TEXTURE_ALPHA_MODE_PREMULT_ELECTRICAL", "meta-multi-texture-alpha-mode-premult-electrical" },
        { META_MULTI_TEXTURE_ALPHA_MODE_STRAIGHT, "META_MULTI_TEXTURE_ALPHA_MODE_STRAIGHT", "meta-multi-texture-alpha-mode-straight" },
        { N_META_MULTI_TEXTURE_ALPHA_MODES, "N_META_MULTI_TEXTURE_ALPHA_MODES", "n-meta-multi-texture-alpha-modes" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaMultiTextureAlphaMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_multi_texture_coefficients_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_MULTI_TEXTURE_COEFFICIENTS_NONE, "META_MULTI_TEXTURE_COEFFICIENTS_NONE", "meta-multi-texture-coefficients-none" },
        { META_MULTI_TEXTURE_COEFFICIENTS_IDENTITY_FULL, "META_MULTI_TEXTURE_COEFFICIENTS_IDENTITY_FULL", "meta-multi-texture-coefficients-identity-full" },
        { META_MULTI_TEXTURE_COEFFICIENTS_IDENTITY_LIMITED, "META_MULTI_TEXTURE_COEFFICIENTS_IDENTITY_LIMITED", "meta-multi-texture-coefficients-identity-limited" },
        { META_MULTI_TEXTURE_COEFFICIENTS_BT709_FULL, "META_MULTI_TEXTURE_COEFFICIENTS_BT709_FULL", "meta-multi-texture-coefficients-bt709-full" },
        { META_MULTI_TEXTURE_COEFFICIENTS_BT709_LIMITED, "META_MULTI_TEXTURE_COEFFICIENTS_BT709_LIMITED", "meta-multi-texture-coefficients-bt709-limited" },
        { META_MULTI_TEXTURE_COEFFICIENTS_BT601_FULL, "META_MULTI_TEXTURE_COEFFICIENTS_BT601_FULL", "meta-multi-texture-coefficients-bt601-full" },
        { META_MULTI_TEXTURE_COEFFICIENTS_BT601_LIMITED, "META_MULTI_TEXTURE_COEFFICIENTS_BT601_LIMITED", "meta-multi-texture-coefficients-bt601-limited" },
        { META_MULTI_TEXTURE_COEFFICIENTS_BT2020_FULL, "META_MULTI_TEXTURE_COEFFICIENTS_BT2020_FULL", "meta-multi-texture-coefficients-bt2020-full" },
        { META_MULTI_TEXTURE_COEFFICIENTS_BT2020_LIMITED, "META_MULTI_TEXTURE_COEFFICIENTS_BT2020_LIMITED", "meta-multi-texture-coefficients-bt2020-limited" },
        { N_META_MULTI_TEXTURE_COEFFICIENTS, "N_META_MULTI_TEXTURE_COEFFICIENTS", "n-meta-multi-texture-coefficients" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaMultiTextureCoefficients"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_multi_texture_chroma_loc_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_MULTI_TEXTURE_CHROMA_LOC_NONE, "META_MULTI_TEXTURE_CHROMA_LOC_NONE", "none" },
        { META_MULTI_TEXTURE_CHROMA_LOC_DEFINED, "META_MULTI_TEXTURE_CHROMA_LOC_DEFINED", "defined" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaMultiTextureChromaLoc"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-orientation-manager.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-orientation-manager.h"
GType
meta_orientation_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_ORIENTATION_UNDEFINED, "META_ORIENTATION_UNDEFINED", "undefined" },
        { META_ORIENTATION_NORMAL, "META_ORIENTATION_NORMAL", "normal" },
        { META_ORIENTATION_BOTTOM_UP, "META_ORIENTATION_BOTTOM_UP", "bottom-up" },
        { META_ORIENTATION_LEFT_UP, "META_ORIENTATION_LEFT_UP", "left-up" },
        { META_ORIENTATION_RIGHT_UP, "META_ORIENTATION_RIGHT_UP", "right-up" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaOrientation"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/meta-selection-source.h" */
#include "/build/mutter/src/build/../mutter/src/meta/meta-selection-source.h"
GType
meta_selection_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_SELECTION_PRIMARY, "META_SELECTION_PRIMARY", "selection-primary" },
        { META_SELECTION_CLIPBOARD, "META_SELECTION_CLIPBOARD", "selection-clipboard" },
        { META_SELECTION_DND, "META_SELECTION_DND", "selection-dnd" },
        { META_N_SELECTION_TYPES, "META_N_SELECTION_TYPES", "n-selection-types" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaSelectionType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/prefs.h" */
#include "/build/mutter/src/build/../mutter/src/meta/prefs.h"
GType
meta_preference_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_PREF_MOUSE_BUTTON_MODS, "META_PREF_MOUSE_BUTTON_MODS", "mouse-button-mods" },
        { META_PREF_FOCUS_MODE, "META_PREF_FOCUS_MODE", "focus-mode" },
        { META_PREF_FOCUS_NEW_WINDOWS, "META_PREF_FOCUS_NEW_WINDOWS", "focus-new-windows" },
        { META_PREF_ATTACH_MODAL_DIALOGS, "META_PREF_ATTACH_MODAL_DIALOGS", "attach-modal-dialogs" },
        { META_PREF_RAISE_ON_CLICK, "META_PREF_RAISE_ON_CLICK", "raise-on-click" },
        { META_PREF_ACTION_DOUBLE_CLICK_TITLEBAR, "META_PREF_ACTION_DOUBLE_CLICK_TITLEBAR", "action-double-click-titlebar" },
        { META_PREF_ACTION_MIDDLE_CLICK_TITLEBAR, "META_PREF_ACTION_MIDDLE_CLICK_TITLEBAR", "action-middle-click-titlebar" },
        { META_PREF_ACTION_RIGHT_CLICK_TITLEBAR, "META_PREF_ACTION_RIGHT_CLICK_TITLEBAR", "action-right-click-titlebar" },
        { META_PREF_AUTO_RAISE, "META_PREF_AUTO_RAISE", "auto-raise" },
        { META_PREF_AUTO_RAISE_DELAY, "META_PREF_AUTO_RAISE_DELAY", "auto-raise-delay" },
        { META_PREF_FOCUS_CHANGE_ON_POINTER_REST, "META_PREF_FOCUS_CHANGE_ON_POINTER_REST", "focus-change-on-pointer-rest" },
        { META_PREF_NUM_WORKSPACES, "META_PREF_NUM_WORKSPACES", "num-workspaces" },
        { META_PREF_DYNAMIC_WORKSPACES, "META_PREF_DYNAMIC_WORKSPACES", "dynamic-workspaces" },
        { META_PREF_KEYBINDINGS, "META_PREF_KEYBINDINGS", "keybindings" },
        { META_PREF_DISABLE_WORKAROUNDS, "META_PREF_DISABLE_WORKAROUNDS", "disable-workarounds" },
        { META_PREF_BUTTON_LAYOUT, "META_PREF_BUTTON_LAYOUT", "button-layout" },
        { META_PREF_WORKSPACE_NAMES, "META_PREF_WORKSPACE_NAMES", "workspace-names" },
        { META_PREF_VISUAL_BELL, "META_PREF_VISUAL_BELL", "visual-bell" },
        { META_PREF_AUDIBLE_BELL, "META_PREF_AUDIBLE_BELL", "audible-bell" },
        { META_PREF_VISUAL_BELL_TYPE, "META_PREF_VISUAL_BELL_TYPE", "visual-bell-type" },
        { META_PREF_GNOME_ACCESSIBILITY, "META_PREF_GNOME_ACCESSIBILITY", "gnome-accessibility" },
        { META_PREF_GNOME_ANIMATIONS, "META_PREF_GNOME_ANIMATIONS", "gnome-animations" },
        { META_PREF_CURSOR_THEME, "META_PREF_CURSOR_THEME", "cursor-theme" },
        { META_PREF_CURSOR_SIZE, "META_PREF_CURSOR_SIZE", "cursor-size" },
        { META_PREF_RESIZE_WITH_RIGHT_BUTTON, "META_PREF_RESIZE_WITH_RIGHT_BUTTON", "resize-with-right-button" },
        { META_PREF_EDGE_TILING, "META_PREF_EDGE_TILING", "edge-tiling" },
        { META_PREF_FORCE_FULLSCREEN, "META_PREF_FORCE_FULLSCREEN", "force-fullscreen" },
        { META_PREF_WORKSPACES_ONLY_ON_PRIMARY, "META_PREF_WORKSPACES_ONLY_ON_PRIMARY", "workspaces-only-on-primary" },
        { META_PREF_DRAGGABLE_BORDER_WIDTH, "META_PREF_DRAGGABLE_BORDER_WIDTH", "draggable-border-width" },
        { META_PREF_AUTO_MAXIMIZE, "META_PREF_AUTO_MAXIMIZE", "auto-maximize" },
        { META_PREF_CENTER_NEW_WINDOWS, "META_PREF_CENTER_NEW_WINDOWS", "center-new-windows" },
        { META_PREF_DRAG_THRESHOLD, "META_PREF_DRAG_THRESHOLD", "drag-threshold" },
        { META_PREF_LOCATE_POINTER, "META_PREF_LOCATE_POINTER", "locate-pointer" },
        { META_PREF_CHECK_ALIVE_TIMEOUT, "META_PREF_CHECK_ALIVE_TIMEOUT", "check-alive-timeout" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaPreference"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_key_binding_action_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_KEYBINDING_ACTION_NONE, "META_KEYBINDING_ACTION_NONE", "none" },
        { META_KEYBINDING_ACTION_WORKSPACE_1, "META_KEYBINDING_ACTION_WORKSPACE_1", "workspace-1" },
        { META_KEYBINDING_ACTION_WORKSPACE_2, "META_KEYBINDING_ACTION_WORKSPACE_2", "workspace-2" },
        { META_KEYBINDING_ACTION_WORKSPACE_3, "META_KEYBINDING_ACTION_WORKSPACE_3", "workspace-3" },
        { META_KEYBINDING_ACTION_WORKSPACE_4, "META_KEYBINDING_ACTION_WORKSPACE_4", "workspace-4" },
        { META_KEYBINDING_ACTION_WORKSPACE_5, "META_KEYBINDING_ACTION_WORKSPACE_5", "workspace-5" },
        { META_KEYBINDING_ACTION_WORKSPACE_6, "META_KEYBINDING_ACTION_WORKSPACE_6", "workspace-6" },
        { META_KEYBINDING_ACTION_WORKSPACE_7, "META_KEYBINDING_ACTION_WORKSPACE_7", "workspace-7" },
        { META_KEYBINDING_ACTION_WORKSPACE_8, "META_KEYBINDING_ACTION_WORKSPACE_8", "workspace-8" },
        { META_KEYBINDING_ACTION_WORKSPACE_9, "META_KEYBINDING_ACTION_WORKSPACE_9", "workspace-9" },
        { META_KEYBINDING_ACTION_WORKSPACE_10, "META_KEYBINDING_ACTION_WORKSPACE_10", "workspace-10" },
        { META_KEYBINDING_ACTION_WORKSPACE_11, "META_KEYBINDING_ACTION_WORKSPACE_11", "workspace-11" },
        { META_KEYBINDING_ACTION_WORKSPACE_12, "META_KEYBINDING_ACTION_WORKSPACE_12", "workspace-12" },
        { META_KEYBINDING_ACTION_WORKSPACE_LEFT, "META_KEYBINDING_ACTION_WORKSPACE_LEFT", "workspace-left" },
        { META_KEYBINDING_ACTION_WORKSPACE_RIGHT, "META_KEYBINDING_ACTION_WORKSPACE_RIGHT", "workspace-right" },
        { META_KEYBINDING_ACTION_WORKSPACE_UP, "META_KEYBINDING_ACTION_WORKSPACE_UP", "workspace-up" },
        { META_KEYBINDING_ACTION_WORKSPACE_DOWN, "META_KEYBINDING_ACTION_WORKSPACE_DOWN", "workspace-down" },
        { META_KEYBINDING_ACTION_WORKSPACE_LAST, "META_KEYBINDING_ACTION_WORKSPACE_LAST", "workspace-last" },
        { META_KEYBINDING_ACTION_SWITCH_APPLICATIONS, "META_KEYBINDING_ACTION_SWITCH_APPLICATIONS", "switch-applications" },
        { META_KEYBINDING_ACTION_SWITCH_APPLICATIONS_BACKWARD, "META_KEYBINDING_ACTION_SWITCH_APPLICATIONS_BACKWARD", "switch-applications-backward" },
        { META_KEYBINDING_ACTION_SWITCH_GROUP, "META_KEYBINDING_ACTION_SWITCH_GROUP", "switch-group" },
        { META_KEYBINDING_ACTION_SWITCH_GROUP_BACKWARD, "META_KEYBINDING_ACTION_SWITCH_GROUP_BACKWARD", "switch-group-backward" },
        { META_KEYBINDING_ACTION_SWITCH_WINDOWS, "META_KEYBINDING_ACTION_SWITCH_WINDOWS", "switch-windows" },
        { META_KEYBINDING_ACTION_SWITCH_WINDOWS_BACKWARD, "META_KEYBINDING_ACTION_SWITCH_WINDOWS_BACKWARD", "switch-windows-backward" },
        { META_KEYBINDING_ACTION_SWITCH_PANELS, "META_KEYBINDING_ACTION_SWITCH_PANELS", "switch-panels" },
        { META_KEYBINDING_ACTION_SWITCH_PANELS_BACKWARD, "META_KEYBINDING_ACTION_SWITCH_PANELS_BACKWARD", "switch-panels-backward" },
        { META_KEYBINDING_ACTION_CYCLE_GROUP, "META_KEYBINDING_ACTION_CYCLE_GROUP", "cycle-group" },
        { META_KEYBINDING_ACTION_CYCLE_GROUP_BACKWARD, "META_KEYBINDING_ACTION_CYCLE_GROUP_BACKWARD", "cycle-group-backward" },
        { META_KEYBINDING_ACTION_CYCLE_WINDOWS, "META_KEYBINDING_ACTION_CYCLE_WINDOWS", "cycle-windows" },
        { META_KEYBINDING_ACTION_CYCLE_WINDOWS_BACKWARD, "META_KEYBINDING_ACTION_CYCLE_WINDOWS_BACKWARD", "cycle-windows-backward" },
        { META_KEYBINDING_ACTION_CYCLE_PANELS, "META_KEYBINDING_ACTION_CYCLE_PANELS", "cycle-panels" },
        { META_KEYBINDING_ACTION_CYCLE_PANELS_BACKWARD, "META_KEYBINDING_ACTION_CYCLE_PANELS_BACKWARD", "cycle-panels-backward" },
        { META_KEYBINDING_ACTION_SHOW_DESKTOP, "META_KEYBINDING_ACTION_SHOW_DESKTOP", "show-desktop" },
        { META_KEYBINDING_ACTION_PANEL_RUN_DIALOG, "META_KEYBINDING_ACTION_PANEL_RUN_DIALOG", "panel-run-dialog" },
        { META_KEYBINDING_ACTION_TOGGLE_RECORDING, "META_KEYBINDING_ACTION_TOGGLE_RECORDING", "toggle-recording" },
        { META_KEYBINDING_ACTION_SET_SPEW_MARK, "META_KEYBINDING_ACTION_SET_SPEW_MARK", "set-spew-mark" },
        { META_KEYBINDING_ACTION_ACTIVATE_WINDOW_MENU, "META_KEYBINDING_ACTION_ACTIVATE_WINDOW_MENU", "activate-window-menu" },
        { META_KEYBINDING_ACTION_TOGGLE_FULLSCREEN, "META_KEYBINDING_ACTION_TOGGLE_FULLSCREEN", "toggle-fullscreen" },
        { META_KEYBINDING_ACTION_TOGGLE_MAXIMIZED, "META_KEYBINDING_ACTION_TOGGLE_MAXIMIZED", "toggle-maximized" },
        { META_KEYBINDING_ACTION_TOGGLE_TILED_LEFT, "META_KEYBINDING_ACTION_TOGGLE_TILED_LEFT", "toggle-tiled-left" },
        { META_KEYBINDING_ACTION_TOGGLE_TILED_RIGHT, "META_KEYBINDING_ACTION_TOGGLE_TILED_RIGHT", "toggle-tiled-right" },
        { META_KEYBINDING_ACTION_TOGGLE_ABOVE, "META_KEYBINDING_ACTION_TOGGLE_ABOVE", "toggle-above" },
        { META_KEYBINDING_ACTION_MAXIMIZE, "META_KEYBINDING_ACTION_MAXIMIZE", "maximize" },
        { META_KEYBINDING_ACTION_UNMAXIMIZE, "META_KEYBINDING_ACTION_UNMAXIMIZE", "unmaximize" },
        { META_KEYBINDING_ACTION_TOGGLE_SHADED, "META_KEYBINDING_ACTION_TOGGLE_SHADED", "toggle-shaded" },
        { META_KEYBINDING_ACTION_MINIMIZE, "META_KEYBINDING_ACTION_MINIMIZE", "minimize" },
        { META_KEYBINDING_ACTION_CLOSE, "META_KEYBINDING_ACTION_CLOSE", "close" },
        { META_KEYBINDING_ACTION_BEGIN_MOVE, "META_KEYBINDING_ACTION_BEGIN_MOVE", "begin-move" },
        { META_KEYBINDING_ACTION_BEGIN_RESIZE, "META_KEYBINDING_ACTION_BEGIN_RESIZE", "begin-resize" },
        { META_KEYBINDING_ACTION_TOGGLE_ON_ALL_WORKSPACES, "META_KEYBINDING_ACTION_TOGGLE_ON_ALL_WORKSPACES", "toggle-on-all-workspaces" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_1, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_1", "move-to-workspace-1" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_2, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_2", "move-to-workspace-2" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_3, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_3", "move-to-workspace-3" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_4, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_4", "move-to-workspace-4" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_5, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_5", "move-to-workspace-5" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_6, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_6", "move-to-workspace-6" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_7, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_7", "move-to-workspace-7" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_8, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_8", "move-to-workspace-8" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_9, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_9", "move-to-workspace-9" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_10, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_10", "move-to-workspace-10" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_11, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_11", "move-to-workspace-11" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_12, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_12", "move-to-workspace-12" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_LEFT, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_LEFT", "move-to-workspace-left" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_RIGHT, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_RIGHT", "move-to-workspace-right" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_UP, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_UP", "move-to-workspace-up" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_DOWN, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_DOWN", "move-to-workspace-down" },
        { META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_LAST, "META_KEYBINDING_ACTION_MOVE_TO_WORKSPACE_LAST", "move-to-workspace-last" },
        { META_KEYBINDING_ACTION_MOVE_TO_MONITOR_LEFT, "META_KEYBINDING_ACTION_MOVE_TO_MONITOR_LEFT", "move-to-monitor-left" },
        { META_KEYBINDING_ACTION_MOVE_TO_MONITOR_RIGHT, "META_KEYBINDING_ACTION_MOVE_TO_MONITOR_RIGHT", "move-to-monitor-right" },
        { META_KEYBINDING_ACTION_MOVE_TO_MONITOR_UP, "META_KEYBINDING_ACTION_MOVE_TO_MONITOR_UP", "move-to-monitor-up" },
        { META_KEYBINDING_ACTION_MOVE_TO_MONITOR_DOWN, "META_KEYBINDING_ACTION_MOVE_TO_MONITOR_DOWN", "move-to-monitor-down" },
        { META_KEYBINDING_ACTION_RAISE_OR_LOWER, "META_KEYBINDING_ACTION_RAISE_OR_LOWER", "raise-or-lower" },
        { META_KEYBINDING_ACTION_RAISE, "META_KEYBINDING_ACTION_RAISE", "raise" },
        { META_KEYBINDING_ACTION_LOWER, "META_KEYBINDING_ACTION_LOWER", "lower" },
        { META_KEYBINDING_ACTION_MAXIMIZE_VERTICALLY, "META_KEYBINDING_ACTION_MAXIMIZE_VERTICALLY", "maximize-vertically" },
        { META_KEYBINDING_ACTION_MAXIMIZE_HORIZONTALLY, "META_KEYBINDING_ACTION_MAXIMIZE_HORIZONTALLY", "maximize-horizontally" },
        { META_KEYBINDING_ACTION_MOVE_TO_CORNER_NW, "META_KEYBINDING_ACTION_MOVE_TO_CORNER_NW", "move-to-corner-nw" },
        { META_KEYBINDING_ACTION_MOVE_TO_CORNER_NE, "META_KEYBINDING_ACTION_MOVE_TO_CORNER_NE", "move-to-corner-ne" },
        { META_KEYBINDING_ACTION_MOVE_TO_CORNER_SW, "META_KEYBINDING_ACTION_MOVE_TO_CORNER_SW", "move-to-corner-sw" },
        { META_KEYBINDING_ACTION_MOVE_TO_CORNER_SE, "META_KEYBINDING_ACTION_MOVE_TO_CORNER_SE", "move-to-corner-se" },
        { META_KEYBINDING_ACTION_MOVE_TO_SIDE_N, "META_KEYBINDING_ACTION_MOVE_TO_SIDE_N", "move-to-side-n" },
        { META_KEYBINDING_ACTION_MOVE_TO_SIDE_S, "META_KEYBINDING_ACTION_MOVE_TO_SIDE_S", "move-to-side-s" },
        { META_KEYBINDING_ACTION_MOVE_TO_SIDE_E, "META_KEYBINDING_ACTION_MOVE_TO_SIDE_E", "move-to-side-e" },
        { META_KEYBINDING_ACTION_MOVE_TO_SIDE_W, "META_KEYBINDING_ACTION_MOVE_TO_SIDE_W", "move-to-side-w" },
        { META_KEYBINDING_ACTION_MOVE_TO_CENTER, "META_KEYBINDING_ACTION_MOVE_TO_CENTER", "move-to-center" },
        { META_KEYBINDING_ACTION_OVERLAY_KEY, "META_KEYBINDING_ACTION_OVERLAY_KEY", "overlay-key" },
        { META_KEYBINDING_ACTION_LOCATE_POINTER_KEY, "META_KEYBINDING_ACTION_LOCATE_POINTER_KEY", "locate-pointer-key" },
        { META_KEYBINDING_ACTION_ISO_NEXT_GROUP, "META_KEYBINDING_ACTION_ISO_NEXT_GROUP", "iso-next-group" },
        { META_KEYBINDING_ACTION_ALWAYS_ON_TOP, "META_KEYBINDING_ACTION_ALWAYS_ON_TOP", "always-on-top" },
        { META_KEYBINDING_ACTION_SWITCH_MONITOR, "META_KEYBINDING_ACTION_SWITCH_MONITOR", "switch-monitor" },
        { META_KEYBINDING_ACTION_ROTATE_MONITOR, "META_KEYBINDING_ACTION_ROTATE_MONITOR", "rotate-monitor" },
        { META_KEYBINDING_ACTION_LAST, "META_KEYBINDING_ACTION_LAST", "last" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaKeyBindingAction"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_key_binding_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_KEY_BINDING_NONE, "META_KEY_BINDING_NONE", "none" },
        { META_KEY_BINDING_PER_WINDOW, "META_KEY_BINDING_PER_WINDOW", "per-window" },
        { META_KEY_BINDING_BUILTIN, "META_KEY_BINDING_BUILTIN", "builtin" },
        { META_KEY_BINDING_IS_REVERSED, "META_KEY_BINDING_IS_REVERSED", "is-reversed" },
        { META_KEY_BINDING_NON_MASKABLE, "META_KEY_BINDING_NON_MASKABLE", "non-maskable" },
        { META_KEY_BINDING_IGNORE_AUTOREPEAT, "META_KEY_BINDING_IGNORE_AUTOREPEAT", "ignore-autorepeat" },
        { META_KEY_BINDING_NO_AUTO_GRAB, "META_KEY_BINDING_NO_AUTO_GRAB", "no-auto-grab" },
        { META_KEY_BINDING_CUSTOM_TRIGGER, "META_KEY_BINDING_CUSTOM_TRIGGER", "custom-trigger" },
        { META_KEY_BINDING_TRIGGER_RELEASE, "META_KEY_BINDING_TRIGGER_RELEASE", "trigger-release" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaKeyBindingFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/util.h" */
#include "/build/mutter/src/build/../mutter/src/meta/util.h"
GType
meta_debug_paint_flag_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_DEBUG_PAINT_NONE, "META_DEBUG_PAINT_NONE", "none" },
        { META_DEBUG_PAINT_OPAQUE_REGION, "META_DEBUG_PAINT_OPAQUE_REGION", "opaque-region" },
        { META_DEBUG_PAINT_SYNC_CURSOR_PRIMARY, "META_DEBUG_PAINT_SYNC_CURSOR_PRIMARY", "sync-cursor-primary" },
        { META_DEBUG_PAINT_DISABLE_DIRECT_SCANOUT, "META_DEBUG_PAINT_DISABLE_DIRECT_SCANOUT", "disable-direct-scanout" },
        { META_DEBUG_PAINT_IGNORE_COLOR_STATE_FOR_DIRECT_SCANOUT, "META_DEBUG_PAINT_IGNORE_COLOR_STATE_FOR_DIRECT_SCANOUT", "ignore-color-state-for-direct-scanout" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaDebugPaintFlag"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/src/meta/window.h" */
#include "/build/mutter/src/build/../mutter/src/meta/window.h"
GType
meta_window_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_WINDOW_NORMAL, "META_WINDOW_NORMAL", "normal" },
        { META_WINDOW_DESKTOP, "META_WINDOW_DESKTOP", "desktop" },
        { META_WINDOW_DOCK, "META_WINDOW_DOCK", "dock" },
        { META_WINDOW_DIALOG, "META_WINDOW_DIALOG", "dialog" },
        { META_WINDOW_MODAL_DIALOG, "META_WINDOW_MODAL_DIALOG", "modal-dialog" },
        { META_WINDOW_TOOLBAR, "META_WINDOW_TOOLBAR", "toolbar" },
        { META_WINDOW_MENU, "META_WINDOW_MENU", "menu" },
        { META_WINDOW_UTILITY, "META_WINDOW_UTILITY", "utility" },
        { META_WINDOW_SPLASHSCREEN, "META_WINDOW_SPLASHSCREEN", "splashscreen" },
        { META_WINDOW_DROPDOWN_MENU, "META_WINDOW_DROPDOWN_MENU", "dropdown-menu" },
        { META_WINDOW_POPUP_MENU, "META_WINDOW_POPUP_MENU", "popup-menu" },
        { META_WINDOW_TOOLTIP, "META_WINDOW_TOOLTIP", "tooltip" },
        { META_WINDOW_NOTIFICATION, "META_WINDOW_NOTIFICATION", "notification" },
        { META_WINDOW_COMBO, "META_WINDOW_COMBO", "combo" },
        { META_WINDOW_DND, "META_WINDOW_DND", "dnd" },
        { META_WINDOW_OVERRIDE_OTHER, "META_WINDOW_OVERRIDE_OTHER", "override-other" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaWindowType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_maximize_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { META_MAXIMIZE_HORIZONTAL, "META_MAXIMIZE_HORIZONTAL", "horizontal" },
        { META_MAXIMIZE_VERTICAL, "META_MAXIMIZE_VERTICAL", "vertical" },
        { META_MAXIMIZE_BOTH, "META_MAXIMIZE_BOTH", "both" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("MetaMaximizeFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
meta_window_client_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { META_WINDOW_CLIENT_TYPE_WAYLAND, "META_WINDOW_CLIENT_TYPE_WAYLAND", "wayland" },
        { META_WINDOW_CLIENT_TYPE_X11, "META_WINDOW_CLIENT_TYPE_X11", "x11" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("MetaWindowClientType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* Generated data ends here */

