/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __META_DBUS_GSD_POWER_SCREEN_H__
#define __META_DBUS_GSD_POWER_SCREEN_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.SettingsDaemon.Power.Screen */

#define META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN (meta_dbus_settings_daemon_power_screen_get_type ())
#define META_DBUS_SETTINGS_DAEMON_POWER_SCREEN(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN, MetaDBusSettingsDaemonPowerScreen))
#define META_DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN))
#define META_DBUS_SETTINGS_DAEMON_POWER_SCREEN_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN, MetaDBusSettingsDaemonPowerScreenIface))

struct _MetaDBusSettingsDaemonPowerScreen;
typedef struct _MetaDBusSettingsDaemonPowerScreen MetaDBusSettingsDaemonPowerScreen;
typedef struct _MetaDBusSettingsDaemonPowerScreenIface MetaDBusSettingsDaemonPowerScreenIface;

struct _MetaDBusSettingsDaemonPowerScreenIface
{
  GTypeInterface parent_iface;

  gint  (*get_brightness) (MetaDBusSettingsDaemonPowerScreen *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusSettingsDaemonPowerScreen, g_object_unref)
#endif

GType meta_dbus_settings_daemon_power_screen_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *meta_dbus_settings_daemon_power_screen_interface_info (void);
guint meta_dbus_settings_daemon_power_screen_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus property accessors: */
gint meta_dbus_settings_daemon_power_screen_get_brightness (MetaDBusSettingsDaemonPowerScreen *object);
void meta_dbus_settings_daemon_power_screen_set_brightness (MetaDBusSettingsDaemonPowerScreen *object, gint value);


/* ---- */

#define META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY (meta_dbus_settings_daemon_power_screen_proxy_get_type ())
#define META_DBUS_SETTINGS_DAEMON_POWER_SCREEN_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY, MetaDBusSettingsDaemonPowerScreenProxy))
#define META_DBUS_SETTINGS_DAEMON_POWER_SCREEN_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY, MetaDBusSettingsDaemonPowerScreenProxyClass))
#define META_DBUS_SETTINGS_DAEMON_POWER_SCREEN_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY, MetaDBusSettingsDaemonPowerScreenProxyClass))
#define META_DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY))
#define META_DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_PROXY))

typedef struct _MetaDBusSettingsDaemonPowerScreenProxy MetaDBusSettingsDaemonPowerScreenProxy;
typedef struct _MetaDBusSettingsDaemonPowerScreenProxyClass MetaDBusSettingsDaemonPowerScreenProxyClass;
typedef struct _MetaDBusSettingsDaemonPowerScreenProxyPrivate MetaDBusSettingsDaemonPowerScreenProxyPrivate;

struct _MetaDBusSettingsDaemonPowerScreenProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MetaDBusSettingsDaemonPowerScreenProxyPrivate *priv;
};

struct _MetaDBusSettingsDaemonPowerScreenProxyClass
{
  GDBusProxyClass parent_class;
};

GType meta_dbus_settings_daemon_power_screen_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusSettingsDaemonPowerScreenProxy, g_object_unref)
#endif

void meta_dbus_settings_daemon_power_screen_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusSettingsDaemonPowerScreen *meta_dbus_settings_daemon_power_screen_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusSettingsDaemonPowerScreen *meta_dbus_settings_daemon_power_screen_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void meta_dbus_settings_daemon_power_screen_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusSettingsDaemonPowerScreen *meta_dbus_settings_daemon_power_screen_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusSettingsDaemonPowerScreen *meta_dbus_settings_daemon_power_screen_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON (meta_dbus_settings_daemon_power_screen_skeleton_get_type ())
#define META_DBUS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON, MetaDBusSettingsDaemonPowerScreenSkeleton))
#define META_DBUS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON, MetaDBusSettingsDaemonPowerScreenSkeletonClass))
#define META_DBUS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON, MetaDBusSettingsDaemonPowerScreenSkeletonClass))
#define META_DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON))
#define META_DBUS_IS_SETTINGS_DAEMON_POWER_SCREEN_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_SETTINGS_DAEMON_POWER_SCREEN_SKELETON))

typedef struct _MetaDBusSettingsDaemonPowerScreenSkeleton MetaDBusSettingsDaemonPowerScreenSkeleton;
typedef struct _MetaDBusSettingsDaemonPowerScreenSkeletonClass MetaDBusSettingsDaemonPowerScreenSkeletonClass;
typedef struct _MetaDBusSettingsDaemonPowerScreenSkeletonPrivate MetaDBusSettingsDaemonPowerScreenSkeletonPrivate;

struct _MetaDBusSettingsDaemonPowerScreenSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MetaDBusSettingsDaemonPowerScreenSkeletonPrivate *priv;
};

struct _MetaDBusSettingsDaemonPowerScreenSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType meta_dbus_settings_daemon_power_screen_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusSettingsDaemonPowerScreenSkeleton, g_object_unref)
#endif

MetaDBusSettingsDaemonPowerScreen *meta_dbus_settings_daemon_power_screen_skeleton_new (void);


G_END_DECLS

#endif /* __META_DBUS_GSD_POWER_SCREEN_H__ */
