/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __META_DBUS_COLOR_MANAGER_H__
#define __META_DBUS_COLOR_MANAGER_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.ColorManager */

#define META_DBUS_TYPE_COLOR_MANAGER (meta_dbus_color_manager_get_type ())
#define META_DBUS_COLOR_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_COLOR_MANAGER, MetaDBusColorManager))
#define META_DBUS_IS_COLOR_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_COLOR_MANAGER))
#define META_DBUS_COLOR_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), META_DBUS_TYPE_COLOR_MANAGER, MetaDBusColorManagerIface))

struct _MetaDBusColorManager;
typedef struct _MetaDBusColorManager MetaDBusColorManager;
typedef struct _MetaDBusColorManagerIface MetaDBusColorManagerIface;

struct _MetaDBusColorManagerIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_calibrate_monitor) (
    MetaDBusColorManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_connector);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusColorManager, g_object_unref)
#endif

GType meta_dbus_color_manager_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *meta_dbus_color_manager_interface_info (void);
guint meta_dbus_color_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void meta_dbus_color_manager_complete_calibrate_monitor (
    MetaDBusColorManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *session);



/* D-Bus method calls: */
void meta_dbus_color_manager_call_calibrate_monitor (
    MetaDBusColorManager *proxy,
    const gchar *arg_connector,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_color_manager_call_calibrate_monitor_finish (
    MetaDBusColorManager *proxy,
    gchar ** out_session,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_color_manager_call_calibrate_monitor_sync (
    MetaDBusColorManager *proxy,
    const gchar *arg_connector,
    gchar ** out_session,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define META_DBUS_TYPE_COLOR_MANAGER_PROXY (meta_dbus_color_manager_proxy_get_type ())
#define META_DBUS_COLOR_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_COLOR_MANAGER_PROXY, MetaDBusColorManagerProxy))
#define META_DBUS_COLOR_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_COLOR_MANAGER_PROXY, MetaDBusColorManagerProxyClass))
#define META_DBUS_COLOR_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_COLOR_MANAGER_PROXY, MetaDBusColorManagerProxyClass))
#define META_DBUS_IS_COLOR_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_COLOR_MANAGER_PROXY))
#define META_DBUS_IS_COLOR_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_COLOR_MANAGER_PROXY))

typedef struct _MetaDBusColorManagerProxy MetaDBusColorManagerProxy;
typedef struct _MetaDBusColorManagerProxyClass MetaDBusColorManagerProxyClass;
typedef struct _MetaDBusColorManagerProxyPrivate MetaDBusColorManagerProxyPrivate;

struct _MetaDBusColorManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MetaDBusColorManagerProxyPrivate *priv;
};

struct _MetaDBusColorManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType meta_dbus_color_manager_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusColorManagerProxy, g_object_unref)
#endif

void meta_dbus_color_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusColorManager *meta_dbus_color_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusColorManager *meta_dbus_color_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void meta_dbus_color_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusColorManager *meta_dbus_color_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusColorManager *meta_dbus_color_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define META_DBUS_TYPE_COLOR_MANAGER_SKELETON (meta_dbus_color_manager_skeleton_get_type ())
#define META_DBUS_COLOR_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_COLOR_MANAGER_SKELETON, MetaDBusColorManagerSkeleton))
#define META_DBUS_COLOR_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_COLOR_MANAGER_SKELETON, MetaDBusColorManagerSkeletonClass))
#define META_DBUS_COLOR_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_COLOR_MANAGER_SKELETON, MetaDBusColorManagerSkeletonClass))
#define META_DBUS_IS_COLOR_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_COLOR_MANAGER_SKELETON))
#define META_DBUS_IS_COLOR_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_COLOR_MANAGER_SKELETON))

typedef struct _MetaDBusColorManagerSkeleton MetaDBusColorManagerSkeleton;
typedef struct _MetaDBusColorManagerSkeletonClass MetaDBusColorManagerSkeletonClass;
typedef struct _MetaDBusColorManagerSkeletonPrivate MetaDBusColorManagerSkeletonPrivate;

struct _MetaDBusColorManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MetaDBusColorManagerSkeletonPrivate *priv;
};

struct _MetaDBusColorManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType meta_dbus_color_manager_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusColorManagerSkeleton, g_object_unref)
#endif

MetaDBusColorManager *meta_dbus_color_manager_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.ColorManager.Calibration */

#define META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION (meta_dbus_color_manager_calibration_get_type ())
#define META_DBUS_COLOR_MANAGER_CALIBRATION(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION, MetaDBusColorManagerCalibration))
#define META_DBUS_IS_COLOR_MANAGER_CALIBRATION(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION))
#define META_DBUS_COLOR_MANAGER_CALIBRATION_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION, MetaDBusColorManagerCalibrationIface))

struct _MetaDBusColorManagerCalibration;
typedef struct _MetaDBusColorManagerCalibration MetaDBusColorManagerCalibration;
typedef struct _MetaDBusColorManagerCalibrationIface MetaDBusColorManagerCalibrationIface;

struct _MetaDBusColorManagerCalibrationIface
{
  GTypeInterface parent_iface;



  gboolean (*handle_set_crtc_gamma_lut) (
    MetaDBusColorManagerCalibration *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_red,
    GVariant *arg_green,
    GVariant *arg_blue);

  gboolean (*handle_stop) (
    MetaDBusColorManagerCalibration *object,
    GDBusMethodInvocation *invocation);

  guint  (*get_gamma_lut_size) (MetaDBusColorManagerCalibration *object);

  void (*stopped) (
    MetaDBusColorManagerCalibration *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusColorManagerCalibration, g_object_unref)
#endif

GType meta_dbus_color_manager_calibration_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *meta_dbus_color_manager_calibration_interface_info (void);
guint meta_dbus_color_manager_calibration_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void meta_dbus_color_manager_calibration_complete_set_crtc_gamma_lut (
    MetaDBusColorManagerCalibration *object,
    GDBusMethodInvocation *invocation);

void meta_dbus_color_manager_calibration_complete_stop (
    MetaDBusColorManagerCalibration *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void meta_dbus_color_manager_calibration_emit_stopped (
    MetaDBusColorManagerCalibration *object);



/* D-Bus method calls: */
void meta_dbus_color_manager_calibration_call_set_crtc_gamma_lut (
    MetaDBusColorManagerCalibration *proxy,
    GVariant *arg_red,
    GVariant *arg_green,
    GVariant *arg_blue,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_color_manager_calibration_call_set_crtc_gamma_lut_finish (
    MetaDBusColorManagerCalibration *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_color_manager_calibration_call_set_crtc_gamma_lut_sync (
    MetaDBusColorManagerCalibration *proxy,
    GVariant *arg_red,
    GVariant *arg_green,
    GVariant *arg_blue,
    GCancellable *cancellable,
    GError **error);

void meta_dbus_color_manager_calibration_call_stop (
    MetaDBusColorManagerCalibration *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean meta_dbus_color_manager_calibration_call_stop_finish (
    MetaDBusColorManagerCalibration *proxy,
    GAsyncResult *res,
    GError **error);

gboolean meta_dbus_color_manager_calibration_call_stop_sync (
    MetaDBusColorManagerCalibration *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
guint meta_dbus_color_manager_calibration_get_gamma_lut_size (MetaDBusColorManagerCalibration *object);
void meta_dbus_color_manager_calibration_set_gamma_lut_size (MetaDBusColorManagerCalibration *object, guint value);


/* ---- */

#define META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY (meta_dbus_color_manager_calibration_proxy_get_type ())
#define META_DBUS_COLOR_MANAGER_CALIBRATION_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY, MetaDBusColorManagerCalibrationProxy))
#define META_DBUS_COLOR_MANAGER_CALIBRATION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY, MetaDBusColorManagerCalibrationProxyClass))
#define META_DBUS_COLOR_MANAGER_CALIBRATION_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY, MetaDBusColorManagerCalibrationProxyClass))
#define META_DBUS_IS_COLOR_MANAGER_CALIBRATION_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY))
#define META_DBUS_IS_COLOR_MANAGER_CALIBRATION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY))

typedef struct _MetaDBusColorManagerCalibrationProxy MetaDBusColorManagerCalibrationProxy;
typedef struct _MetaDBusColorManagerCalibrationProxyClass MetaDBusColorManagerCalibrationProxyClass;
typedef struct _MetaDBusColorManagerCalibrationProxyPrivate MetaDBusColorManagerCalibrationProxyPrivate;

struct _MetaDBusColorManagerCalibrationProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  MetaDBusColorManagerCalibrationProxyPrivate *priv;
};

struct _MetaDBusColorManagerCalibrationProxyClass
{
  GDBusProxyClass parent_class;
};

GType meta_dbus_color_manager_calibration_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusColorManagerCalibrationProxy, g_object_unref)
#endif

void meta_dbus_color_manager_calibration_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusColorManagerCalibration *meta_dbus_color_manager_calibration_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusColorManagerCalibration *meta_dbus_color_manager_calibration_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void meta_dbus_color_manager_calibration_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
MetaDBusColorManagerCalibration *meta_dbus_color_manager_calibration_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
MetaDBusColorManagerCalibration *meta_dbus_color_manager_calibration_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON (meta_dbus_color_manager_calibration_skeleton_get_type ())
#define META_DBUS_COLOR_MANAGER_CALIBRATION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON, MetaDBusColorManagerCalibrationSkeleton))
#define META_DBUS_COLOR_MANAGER_CALIBRATION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON, MetaDBusColorManagerCalibrationSkeletonClass))
#define META_DBUS_COLOR_MANAGER_CALIBRATION_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON, MetaDBusColorManagerCalibrationSkeletonClass))
#define META_DBUS_IS_COLOR_MANAGER_CALIBRATION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON))
#define META_DBUS_IS_COLOR_MANAGER_CALIBRATION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), META_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON))

typedef struct _MetaDBusColorManagerCalibrationSkeleton MetaDBusColorManagerCalibrationSkeleton;
typedef struct _MetaDBusColorManagerCalibrationSkeletonClass MetaDBusColorManagerCalibrationSkeletonClass;
typedef struct _MetaDBusColorManagerCalibrationSkeletonPrivate MetaDBusColorManagerCalibrationSkeletonPrivate;

struct _MetaDBusColorManagerCalibrationSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  MetaDBusColorManagerCalibrationSkeletonPrivate *priv;
};

struct _MetaDBusColorManagerCalibrationSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType meta_dbus_color_manager_calibration_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (MetaDBusColorManagerCalibrationSkeleton, g_object_unref)
#endif

MetaDBusColorManagerCalibration *meta_dbus_color_manager_calibration_skeleton_new (void);


G_END_DECLS

#endif /* __META_DBUS_COLOR_MANAGER_H__ */
