
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"
#include "cogl/cogl-enum-types.h"

/* enumerations from "/build/mutter/src/build/../mutter/cogl/cogl/cogl-buffer.h" */
#include "/build/mutter/src/build/../mutter/cogl/cogl/cogl-buffer.h"
GType
cogl_buffer_update_hint_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_BUFFER_UPDATE_HINT_STATIC, "COGL_BUFFER_UPDATE_HINT_STATIC", "static" },
        { COGL_BUFFER_UPDATE_HINT_DYNAMIC, "COGL_BUFFER_UPDATE_HINT_DYNAMIC", "dynamic" },
        { COGL_BUFFER_UPDATE_HINT_STREAM, "COGL_BUFFER_UPDATE_HINT_STREAM", "stream" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglBufferUpdateHint"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_buffer_access_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { COGL_BUFFER_ACCESS_READ, "COGL_BUFFER_ACCESS_READ", "read" },
        { COGL_BUFFER_ACCESS_WRITE, "COGL_BUFFER_ACCESS_WRITE", "write" },
        { COGL_BUFFER_ACCESS_READ_WRITE, "COGL_BUFFER_ACCESS_READ_WRITE", "read-write" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("CoglBufferAccess"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_buffer_map_hint_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { COGL_BUFFER_MAP_HINT_DISCARD, "COGL_BUFFER_MAP_HINT_DISCARD", "discard" },
        { COGL_BUFFER_MAP_HINT_DISCARD_RANGE, "COGL_BUFFER_MAP_HINT_DISCARD_RANGE", "discard-range" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("CoglBufferMapHint"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_buffer_bind_target_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_BUFFER_BIND_TARGET_PIXEL_PACK, "COGL_BUFFER_BIND_TARGET_PIXEL_PACK", "pixel-pack" },
        { COGL_BUFFER_BIND_TARGET_PIXEL_UNPACK, "COGL_BUFFER_BIND_TARGET_PIXEL_UNPACK", "pixel-unpack" },
        { COGL_BUFFER_BIND_TARGET_ATTRIBUTE_BUFFER, "COGL_BUFFER_BIND_TARGET_ATTRIBUTE_BUFFER", "attribute-buffer" },
        { COGL_BUFFER_BIND_TARGET_INDEX_BUFFER, "COGL_BUFFER_BIND_TARGET_INDEX_BUFFER", "index-buffer" },
        { COGL_BUFFER_BIND_TARGET_COUNT, "COGL_BUFFER_BIND_TARGET_COUNT", "count" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglBufferBindTarget"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/cogl/cogl/cogl-pixel-format.h" */
#include "/build/mutter/src/build/../mutter/cogl/cogl/cogl-pixel-format.h"
GType
cogl_pixel_format_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_PIXEL_FORMAT_ANY, "COGL_PIXEL_FORMAT_ANY", "any" },
        { COGL_PIXEL_FORMAT_A_8, "COGL_PIXEL_FORMAT_A_8", "a-8" },
        { COGL_PIXEL_FORMAT_RGB_565, "COGL_PIXEL_FORMAT_RGB_565", "rgb-565" },
        { COGL_PIXEL_FORMAT_RGBA_4444, "COGL_PIXEL_FORMAT_RGBA_4444", "rgba-4444" },
        { COGL_PIXEL_FORMAT_RGBA_5551, "COGL_PIXEL_FORMAT_RGBA_5551", "rgba-5551" },
        { COGL_PIXEL_FORMAT_YUV, "COGL_PIXEL_FORMAT_YUV", "yuv" },
        { COGL_PIXEL_FORMAT_R_8, "COGL_PIXEL_FORMAT_R_8", "r-8" },
        { COGL_PIXEL_FORMAT_RG_88, "COGL_PIXEL_FORMAT_RG_88", "rg-88" },
        { COGL_PIXEL_FORMAT_RGB_888, "COGL_PIXEL_FORMAT_RGB_888", "rgb-888" },
        { COGL_PIXEL_FORMAT_BGR_888, "COGL_PIXEL_FORMAT_BGR_888", "bgr-888" },
        { COGL_PIXEL_FORMAT_RGBX_8888, "COGL_PIXEL_FORMAT_RGBX_8888", "rgbx-8888" },
        { COGL_PIXEL_FORMAT_RGBA_8888, "COGL_PIXEL_FORMAT_RGBA_8888", "rgba-8888" },
        { COGL_PIXEL_FORMAT_BGRX_8888, "COGL_PIXEL_FORMAT_BGRX_8888", "bgrx-8888" },
        { COGL_PIXEL_FORMAT_BGRA_8888, "COGL_PIXEL_FORMAT_BGRA_8888", "bgra-8888" },
        { COGL_PIXEL_FORMAT_XRGB_8888, "COGL_PIXEL_FORMAT_XRGB_8888", "xrgb-8888" },
        { COGL_PIXEL_FORMAT_ARGB_8888, "COGL_PIXEL_FORMAT_ARGB_8888", "argb-8888" },
        { COGL_PIXEL_FORMAT_XBGR_8888, "COGL_PIXEL_FORMAT_XBGR_8888", "xbgr-8888" },
        { COGL_PIXEL_FORMAT_ABGR_8888, "COGL_PIXEL_FORMAT_ABGR_8888", "abgr-8888" },
        { COGL_PIXEL_FORMAT_RGBA_8888_PRE, "COGL_PIXEL_FORMAT_RGBA_8888_PRE", "rgba-8888-pre" },
        { COGL_PIXEL_FORMAT_BGRA_8888_PRE, "COGL_PIXEL_FORMAT_BGRA_8888_PRE", "bgra-8888-pre" },
        { COGL_PIXEL_FORMAT_ARGB_8888_PRE, "COGL_PIXEL_FORMAT_ARGB_8888_PRE", "argb-8888-pre" },
        { COGL_PIXEL_FORMAT_ABGR_8888_PRE, "COGL_PIXEL_FORMAT_ABGR_8888_PRE", "abgr-8888-pre" },
        { COGL_PIXEL_FORMAT_RGBA_4444_PRE, "COGL_PIXEL_FORMAT_RGBA_4444_PRE", "rgba-4444-pre" },
        { COGL_PIXEL_FORMAT_RGBA_5551_PRE, "COGL_PIXEL_FORMAT_RGBA_5551_PRE", "rgba-5551-pre" },
        { COGL_PIXEL_FORMAT_RGBA_1010102, "COGL_PIXEL_FORMAT_RGBA_1010102", "rgba-1010102" },
        { COGL_PIXEL_FORMAT_BGRA_1010102, "COGL_PIXEL_FORMAT_BGRA_1010102", "bgra-1010102" },
        { COGL_PIXEL_FORMAT_XRGB_2101010, "COGL_PIXEL_FORMAT_XRGB_2101010", "xrgb-2101010" },
        { COGL_PIXEL_FORMAT_ARGB_2101010, "COGL_PIXEL_FORMAT_ARGB_2101010", "argb-2101010" },
        { COGL_PIXEL_FORMAT_XBGR_2101010, "COGL_PIXEL_FORMAT_XBGR_2101010", "xbgr-2101010" },
        { COGL_PIXEL_FORMAT_ABGR_2101010, "COGL_PIXEL_FORMAT_ABGR_2101010", "abgr-2101010" },
        { COGL_PIXEL_FORMAT_RGBA_1010102_PRE, "COGL_PIXEL_FORMAT_RGBA_1010102_PRE", "rgba-1010102-pre" },
        { COGL_PIXEL_FORMAT_BGRA_1010102_PRE, "COGL_PIXEL_FORMAT_BGRA_1010102_PRE", "bgra-1010102-pre" },
        { COGL_PIXEL_FORMAT_ARGB_2101010_PRE, "COGL_PIXEL_FORMAT_ARGB_2101010_PRE", "argb-2101010-pre" },
        { COGL_PIXEL_FORMAT_ABGR_2101010_PRE, "COGL_PIXEL_FORMAT_ABGR_2101010_PRE", "abgr-2101010-pre" },
        { COGL_PIXEL_FORMAT_RGBX_FP_16161616, "COGL_PIXEL_FORMAT_RGBX_FP_16161616", "rgbx-fp-16161616" },
        { COGL_PIXEL_FORMAT_RGBA_FP_16161616, "COGL_PIXEL_FORMAT_RGBA_FP_16161616", "rgba-fp-16161616" },
        { COGL_PIXEL_FORMAT_BGRX_FP_16161616, "COGL_PIXEL_FORMAT_BGRX_FP_16161616", "bgrx-fp-16161616" },
        { COGL_PIXEL_FORMAT_BGRA_FP_16161616, "COGL_PIXEL_FORMAT_BGRA_FP_16161616", "bgra-fp-16161616" },
        { COGL_PIXEL_FORMAT_XRGB_FP_16161616, "COGL_PIXEL_FORMAT_XRGB_FP_16161616", "xrgb-fp-16161616" },
        { COGL_PIXEL_FORMAT_ARGB_FP_16161616, "COGL_PIXEL_FORMAT_ARGB_FP_16161616", "argb-fp-16161616" },
        { COGL_PIXEL_FORMAT_XBGR_FP_16161616, "COGL_PIXEL_FORMAT_XBGR_FP_16161616", "xbgr-fp-16161616" },
        { COGL_PIXEL_FORMAT_ABGR_FP_16161616, "COGL_PIXEL_FORMAT_ABGR_FP_16161616", "abgr-fp-16161616" },
        { COGL_PIXEL_FORMAT_RGBA_FP_16161616_PRE, "COGL_PIXEL_FORMAT_RGBA_FP_16161616_PRE", "rgba-fp-16161616-pre" },
        { COGL_PIXEL_FORMAT_BGRA_FP_16161616_PRE, "COGL_PIXEL_FORMAT_BGRA_FP_16161616_PRE", "bgra-fp-16161616-pre" },
        { COGL_PIXEL_FORMAT_ARGB_FP_16161616_PRE, "COGL_PIXEL_FORMAT_ARGB_FP_16161616_PRE", "argb-fp-16161616-pre" },
        { COGL_PIXEL_FORMAT_ABGR_FP_16161616_PRE, "COGL_PIXEL_FORMAT_ABGR_FP_16161616_PRE", "abgr-fp-16161616-pre" },
        { COGL_PIXEL_FORMAT_RGBA_FP_32323232, "COGL_PIXEL_FORMAT_RGBA_FP_32323232", "rgba-fp-32323232" },
        { COGL_PIXEL_FORMAT_RGBA_FP_32323232_PRE, "COGL_PIXEL_FORMAT_RGBA_FP_32323232_PRE", "rgba-fp-32323232-pre" },
        { COGL_PIXEL_FORMAT_R_16, "COGL_PIXEL_FORMAT_R_16", "r-16" },
        { COGL_PIXEL_FORMAT_RG_1616, "COGL_PIXEL_FORMAT_RG_1616", "rg-1616" },
        { COGL_PIXEL_FORMAT_RGBX_16161616, "COGL_PIXEL_FORMAT_RGBX_16161616", "rgbx-16161616" },
        { COGL_PIXEL_FORMAT_RGBA_16161616, "COGL_PIXEL_FORMAT_RGBA_16161616", "rgba-16161616" },
        { COGL_PIXEL_FORMAT_RGBA_16161616_PRE, "COGL_PIXEL_FORMAT_RGBA_16161616_PRE", "rgba-16161616-pre" },
        { COGL_PIXEL_FORMAT_DEPTH_16, "COGL_PIXEL_FORMAT_DEPTH_16", "depth-16" },
        { COGL_PIXEL_FORMAT_DEPTH_24_STENCIL_8, "COGL_PIXEL_FORMAT_DEPTH_24_STENCIL_8", "depth-24-stencil-8" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglPixelFormat"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* enumerations from "/build/mutter/src/build/../mutter/cogl/cogl/cogl-types.h" */
#include "/build/mutter/src/build/../mutter/cogl/cogl/cogl-types.h"
GType
cogl_system_error_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_SYSTEM_ERROR_UNSUPPORTED, "COGL_SYSTEM_ERROR_UNSUPPORTED", "cogl-system-error-unsupported" },
        { COGL_SYSTEM_ERROR_NO_MEMORY, "COGL_SYSTEM_ERROR_NO_MEMORY", "cogl-system-error-no-memory" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglSystemError"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_attribute_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_ATTRIBUTE_TYPE_BYTE, "COGL_ATTRIBUTE_TYPE_BYTE", "byte" },
        { COGL_ATTRIBUTE_TYPE_UNSIGNED_BYTE, "COGL_ATTRIBUTE_TYPE_UNSIGNED_BYTE", "unsigned-byte" },
        { COGL_ATTRIBUTE_TYPE_SHORT, "COGL_ATTRIBUTE_TYPE_SHORT", "short" },
        { COGL_ATTRIBUTE_TYPE_UNSIGNED_SHORT, "COGL_ATTRIBUTE_TYPE_UNSIGNED_SHORT", "unsigned-short" },
        { COGL_ATTRIBUTE_TYPE_FLOAT, "COGL_ATTRIBUTE_TYPE_FLOAT", "float" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglAttributeType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_indices_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_INDICES_TYPE_UNSIGNED_BYTE, "COGL_INDICES_TYPE_UNSIGNED_BYTE", "byte" },
        { COGL_INDICES_TYPE_UNSIGNED_SHORT, "COGL_INDICES_TYPE_UNSIGNED_SHORT", "short" },
        { COGL_INDICES_TYPE_UNSIGNED_INT, "COGL_INDICES_TYPE_UNSIGNED_INT", "int" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglIndicesType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_vertices_mode_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_VERTICES_MODE_POINTS, "COGL_VERTICES_MODE_POINTS", "points" },
        { COGL_VERTICES_MODE_LINES, "COGL_VERTICES_MODE_LINES", "lines" },
        { COGL_VERTICES_MODE_LINE_LOOP, "COGL_VERTICES_MODE_LINE_LOOP", "line-loop" },
        { COGL_VERTICES_MODE_LINE_STRIP, "COGL_VERTICES_MODE_LINE_STRIP", "line-strip" },
        { COGL_VERTICES_MODE_TRIANGLES, "COGL_VERTICES_MODE_TRIANGLES", "triangles" },
        { COGL_VERTICES_MODE_TRIANGLE_STRIP, "COGL_VERTICES_MODE_TRIANGLE_STRIP", "triangle-strip" },
        { COGL_VERTICES_MODE_TRIANGLE_FAN, "COGL_VERTICES_MODE_TRIANGLE_FAN", "triangle-fan" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglVerticesMode"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_depth_test_function_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_DEPTH_TEST_FUNCTION_NEVER, "COGL_DEPTH_TEST_FUNCTION_NEVER", "never" },
        { COGL_DEPTH_TEST_FUNCTION_LESS, "COGL_DEPTH_TEST_FUNCTION_LESS", "less" },
        { COGL_DEPTH_TEST_FUNCTION_EQUAL, "COGL_DEPTH_TEST_FUNCTION_EQUAL", "equal" },
        { COGL_DEPTH_TEST_FUNCTION_LEQUAL, "COGL_DEPTH_TEST_FUNCTION_LEQUAL", "lequal" },
        { COGL_DEPTH_TEST_FUNCTION_GREATER, "COGL_DEPTH_TEST_FUNCTION_GREATER", "greater" },
        { COGL_DEPTH_TEST_FUNCTION_NOTEQUAL, "COGL_DEPTH_TEST_FUNCTION_NOTEQUAL", "notequal" },
        { COGL_DEPTH_TEST_FUNCTION_GEQUAL, "COGL_DEPTH_TEST_FUNCTION_GEQUAL", "gequal" },
        { COGL_DEPTH_TEST_FUNCTION_ALWAYS, "COGL_DEPTH_TEST_FUNCTION_ALWAYS", "always" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglDepthTestFunction"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_renderer_error_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_RENDERER_ERROR_XLIB_DISPLAY_OPEN, "COGL_RENDERER_ERROR_XLIB_DISPLAY_OPEN", "xlib-display-open" },
        { COGL_RENDERER_ERROR_BAD_CONSTRAINT, "COGL_RENDERER_ERROR_BAD_CONSTRAINT", "bad-constraint" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglRendererError"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_winsys_feature_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_WINSYS_FEATURE_SWAP_REGION, "COGL_WINSYS_FEATURE_SWAP_REGION", "swap-region" },
        { COGL_WINSYS_FEATURE_BUFFER_AGE, "COGL_WINSYS_FEATURE_BUFFER_AGE", "buffer-age" },
        { COGL_WINSYS_FEATURE_SYNC_AND_COMPLETE_EVENT, "COGL_WINSYS_FEATURE_SYNC_AND_COMPLETE_EVENT", "sync-and-complete-event" },
        { COGL_WINSYS_FEATURE_SYNC_FD, "COGL_WINSYS_FEATURE_SYNC_FD", "sync-fd" },
        { COGL_WINSYS_FEATURE_N_FEATURES, "COGL_WINSYS_FEATURE_N_FEATURES", "n-features" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglWinsysFeature"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_winding_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_WINDING_CLOCKWISE, "COGL_WINDING_CLOCKWISE", "clockwise" },
        { COGL_WINDING_COUNTER_CLOCKWISE, "COGL_WINDING_COUNTER_CLOCKWISE", "counter-clockwise" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglWinding"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_buffer_bit_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { COGL_BUFFER_BIT_COLOR, "COGL_BUFFER_BIT_COLOR", "color" },
        { COGL_BUFFER_BIT_DEPTH, "COGL_BUFFER_BIT_DEPTH", "depth" },
        { COGL_BUFFER_BIT_STENCIL, "COGL_BUFFER_BIT_STENCIL", "stencil" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("CoglBufferBit"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_read_pixels_flags_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GFlagsValue values[] = {
        { COGL_READ_PIXELS_COLOR_BUFFER, "COGL_READ_PIXELS_COLOR_BUFFER", "color-buffer" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_flags_register_static (g_intern_static_string ("CoglReadPixelsFlags"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_shader_type_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_SHADER_TYPE_VERTEX, "COGL_SHADER_TYPE_VERTEX", "vertex" },
        { COGL_SHADER_TYPE_FRAGMENT, "COGL_SHADER_TYPE_FRAGMENT", "fragment" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglShaderType"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}
GType
cogl_scanout_error_get_type (void)
{
  static size_t g_enum_type_id = 0;

  if (g_once_init_enter (&g_enum_type_id))
    {
      static const GEnumValue values[] = {
        { COGL_SCANOUT_ERROR_INHIBITED, "COGL_SCANOUT_ERROR_INHIBITED", "inhibited" },
        { 0, NULL, NULL }
      };
      GType id;

      id = g_enum_register_static (g_intern_static_string ("CoglScanoutError"), values);

      g_once_init_leave (&g_enum_type_id, id);
    }

  return g_enum_type_id;
}

/* Generated data ends here */

